/***************** Cabecera de Mdulo *************************************
*MODULO: transformadas                                                    *
*PERTENECE AL PROYECTO: CICYT 3D                                          *
*DIRECTORIO:                                                              *
*HISTORIA DE VERSIONES:                                                   *
*     (1.0    14/07/97  LUTFI                                             *
*      .......   .....  .....   ........................                  *
*      .......   .....  .....   ........................                  *
*BIBLIOTECAS EXTERNAS: m (matematica)                                     *
**************************************************************************/


/********************** SECCION DESCRIPCION MODULO ************************
 Este modulo promorciona las transformacio es descomposicion y reconstru-
 cion, tanto estandar como no estandar de una imagen 2D.
**************************************************************************/


#ifndef TRANSFORMADAS_DEF
#define TRANSFORMADAS_DEF

/********************** SECCION DE INCLUDES *******************************
**************************************************************************/
#include <math.h>
#include "defs.h"
/********************** SECCION DE MACROS *********************************
**************************************************************************/

/********************** SECCION DE RESUMEN FUNCIONES **********************
**************************************************************************
Nombre                     Descripcion
======                     ===========
Decomposition              Descompone un vector 
DecompositionStep          c se modifica con la semisuma en los primeros 
                           tamano/2 elementos y la semiresta en los ultimos
                           elementos.
Reconstruction             Reconstruye un vector
ReconstructionStep         se deshace la operacion realizada sobre c 
                           mediante DecompositionStep
StandardDecomposition      Descomposicion estandar de una matriz
StandardReconstruction     Reconstrucion estandar de una  matriz 
NonstandardDecomposition   Descomposicion no estandar de una matriz 
NonstandardReconstruction  Reconstrucion estandar de una  matriz
*/


/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: Decomposition                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descompone un vector                                        *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: normalizacion y bucle de llamadas a DecompositionStep         *
**************************************************************************/
int Decomposition (TIPO *c, int tamano);

/******************* Cabecera de Funcion **********************************
*NOMBRE: Decomposition2                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descompone un vector                                        *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*   etapas:   numero de veces que hay que descomponer                     *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: normalizacion y bucle de llamadas a DecompositionStep         *
**************************************************************************/
int Decomposition2(TIPO *c, int tamano, int etapas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: DecompositionStep                                                *
***************************************************************************
*                                                                         *
*DESCRIPCION:                                                             *
* c se modifica con la semisuma en los primeros tamano/2 elementos y      *
* la semiresta en los ultimos elementos.                                  *
*PARAMETROS:                                                              *
*   c:        vector a descomponer                                        *
*   tamano:   numero de elementos del vector. Tiene que ser multiplo de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO:                                                               *
**************************************************************************/
int DecompositionStep (TIPO *c, int tamano);

/******************* Cabecera de Funcion **********************************
*NOMBRE: Reconstruction                                                   *
***************************************************************************
*                                                                         *
*DESCRIPCION: Reconstruye un vector                                       *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la  recomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: bucle de llamadas a ReconstructionStep y  normalizacion       *
**************************************************************************/
int Reconstruction (TIPO *c, int tamano);

/******************* Cabecera de Funcion **********************************
*NOMBRE: Reconstruction2                                                  *
***************************************************************************
*                                                                         *
*DESCRIPCION: Reconstruye un vector                                       *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector. Tiene que ser potencia de 2 *
*   etapas:   numero de veces que hay que reconstruir                     *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la  recomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: bucle de llamadas a ReconstructionStep y  normalizacion       *
**************************************************************************/
int Reconstruction2 (TIPO *c, int tamano, int etapas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: ReconstructionStep                                               *
***************************************************************************
*                                                                         *
*DESCRIPCION:                                                             *
* se deshace la operacion realizada sobre c mediante DecompositionStep    *
*PARAMETROS:                                                              *
*   c:        vector a recomponer                                         *
*   tamano:   numero de elementos del vector.Tiene que ser multiplo de 2  *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la recomposicion se realiza sobre el mismo vector                *
*VAR.GLOBALES:                                                            *
*ALGORITMO:                                                               *
**************************************************************************/
int ReconstructionStep (TIPO *c, int tamano);

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardDecomposition                                            *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion estandar de una matriz                       *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada fila se llama a Decomposition y despues por cada     *
*           columna.                                                      *
**************************************************************************/
int StandardDecomposition (TIPO *c, int filas, int columnas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardReconstruction                                           *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna se llama a Reconstruction y despues por cada *
*           fila.                                                         *
**************************************************************************/
int StandardReconstruction (TIPO *c, int filas, int columnas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardDecomposition2                                           *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion estandar de una matriz                       *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*   etapas:                                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada fila se llama a Decomposition y despues por cada     *
*           columna.                                                      *
**************************************************************************/
int StandardDecomposition2(TIPO *c, int filas, int columnas,
			   int e_filas, int e_columnas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: StandardReconstruction2                                          *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   filas:    numero de filas de la matriz. Tiene que ser potencia de 2   *
*   columnas: numero de columnas de la matriz. Tiene que ser potencia de 2*
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna se llama a Reconstruction y despues por cada *
*           fila.                                                         *
**************************************************************************/
int StandardReconstruction2(TIPO *c, int filas, int columnas,
			   int e_filas, int e_columnas);

/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardDecomposition                                         *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz                    *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*           a Decomposition                                               *
**************************************************************************/
int NonstandardDecomposition (TIPO *c, int tamano);

/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardReconstruction                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna y fila se llama a Reconstruction y despues   *
*           se normaliza.                                                 *
**************************************************************************/
int NonstandardReconstruction (TIPO *c, int tamano);



/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardDecomposition2                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz                    *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene que ser      *
*             potencia de 2                                               *
*   p:        Indica el numero de pasos que se aplicaran a la descompos.  *
*             Su valor debe ser 2^Numero_de_pasos.                        * 
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*           a Decomposition                                               *
**************************************************************************/
int NonstandardDecomposition2(TIPO *c, int tamano, int p);


/******************* Cabecera de Funcion **********************************
*NOMBRE: NonstandardReconstruction2                                       *
***************************************************************************
*                                                                         *
*DESCRIPCION:  Reconstrucion estandar de una  matriz                      *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*   p:        Indica el numero de pasos que se aplicaran a la recompos.   *
*             Su valor debe ser 2^Numero_de_pasos.                        * 
*SALIDA:                                                                  *
*    -1 si hubo error, 0 si no lo hubo                                    *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Por cada columna y fila se llama a Reconstruction y despues   *
*           se normaliza.                                                 *
**************************************************************************/
 int NonstandardReconstruction2(TIPO *c, int tamano, int p);


/******************* Cabecera de Funcion **********************************
*NOMBRE: Escalar                                                          *
***************************************************************************
*                                                                         *
*DESCRIPCION:                                                             *
*PARAMETROS:                                                              *
*   c:        matriz a  recomponer                                        *
*   filas, columnas:                                                      *
*             numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    c:  la descomposicion se realiza sobre el mismo vector               *
*VAR.GLOBALES:                                                            *
*ALGORITMO:                                                               *
*                                                                         *
**************************************************************************/
void Escalar (TIPO *c, int filas, int columnas, char *nombre);


#endif
