/***************** Cabecera de Mdulo *************************************
*MODULO: libpgm (libpgm.c)                                                *
*PERTENECE AL PROYECTO: cicyt3D                                           *
*DIRECTORIO:                                                              *
*HISTORIA DE VERSIONES:                                                   *
*     (VERSION   FECHA, AUTOR Y BREVE DESCRIPCION)                        *
*      linux1   8-7-97  jespa   entrada/salida probada satisfactoriamente *
*BIBLIOTECAS EXTERNAS:                                                    *
**************************************************************************/


/********************** SECCION DESCRIPCION MODULO ************************
* Libreria de entrada/salida en formato RAW y PGM de escala de grises     *
* La entrada y salida se realiza mediante buffer del sistema.             *
* Las matrices se almacenan en un solo bloque, una fila tras otra,        *
*   por lo que ya no se desperdicia nada de informacion                   *
* Las funciones que leen un fichero PGM hacen reserva de memoria de forma *
*   automatica. Las funciones que leen RAW no hacen la reserva.           *
* La funcion leer_dimensiones comprueba que los valores introducios estan *
* dentro de un intervalo y puede abortar (exit) la ejecucion del progrma. *
* Se supone que esta version es la mejor para UNIX.                       *
 **************************************************************************/


/********************** SECCION DE INCLUDES *******************************
**************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpgm.h"

/********************** SECCION DE MACROS *********************************
**************************************************************************/


/********************** SECCION DE VARIABLES GLOBALES *********************
**************************************************************************/


/********************** SECCION DE RESUMEN FUNCIONES **********************
***************************************************************************
Nombre         Descripcion
======         ===========

 escribe_raw_uchar
 Escribe un fichero en formato raw a partir de la matriz img 

 lee_raw_uchar
 Lee un fichero en formato raw en la matriz img de unsigned char

 lee_raw_float
 Lee un fichero en formato raw en la matriz img de float  

 lee_pgm_uchar
 Lee un fichero en formato pgm en la matriz img de unsigned char

 lee_pgm_float
 Lee un fichero en formato pgm en la matriz img de float  

 escribe_pgm_float
 Escribe un fichero en formato pgm a partir de la matriz img de float 
 Se realiza una conversion de [-255.0 , 255.0] a [0 , 255]  

 escribe_pgm_float2
 Escribe un fichero en formato pgm a partir de la matriz img de float 
 Se realiza una conversion de [-255.0 , 255.0] a [0 , 255]  

 escribe_pgm_uchar
 Escribe un fichero en formato pgm a partir de la matriz de unsigned char 

 leer_dimensiones
 Lee el ancho, alto y nombre de fichero desde el teclado, comprueba que
 las dimensiones pertenecen al rango indicado por MAX_ALTO,MAX_ANCHO.  
 En caso de no pertenecer al rango el se termina todo el programa      

 */






/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_raw_uchar                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato raw a partir de la matriz img */
void escribe_raw_uchar (int alto,int ancho,char *nombre,unsigned char *img)
{
  FILE *ffsal;
  
  printf("Escribiendo %s\n",nombre);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  
  /* Escritura de la imagen en formato RAW */
  if(fwrite(img, sizeof(unsigned char), ancho*alto, ffsal) != ancho*alto)
    { 
      printf("Error en la escritura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      exit(1);
    }
  fclose(ffsal);
} /* escribe_raw_uchar */



/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_raw_uchar                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato raw en la matriz img de unsigned char*/
void lee_raw_uchar(int alto, int ancho, char *nombre, unsigned char *img)
{
  FILE *ffent;

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* Lectura de la imagen formato RAW */
  if(fread(img, sizeof(unsigned char), ancho*alto, ffent) != ancho*alto)
    {
      printf("Error en la lectura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      exit(1);
    }
  fclose(ffent);
} /* lee_raw_uchar */





/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_raw_float                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato raw en la matriz img de float */
void lee_raw_float (int alto, int ancho, char *nombre, float *img)
{
  int i,j;
  int aux;
  FILE *ffent;

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* Lectura de la imagen formato RAW */
  for (j=0; j< alto; j++  )
    for (i=0; i< ancho; i++  )
    {
      if ((aux=fgetc(ffent)) < 0)
	{
	  printf("Error en la lectura de %s i: %d j: %d\n",nombre,i,j);
	  printf("Error: %d  %s\n",errno,strerror(errno));
	  exit(1);
	}
      img[ancho*j+i]=aux;
    }
  fclose(ffent);
} /* lee_raw_float */




/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_pgm_uchar                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato pgm en la matriz img de unsigned char*/
void lee_pgm_uchar(int *alto, int *ancho, char *nombre,unsigned char **img)
{
  FILE *ffent;
  unsigned int tamano;
  unsigned char buf[100];

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* lectura del ancho y alto. Salto de la cabecera. */
  fgets(buf, 100, ffent);

  /* Saltamos los comentarios PGM, marcados con '#' */
  while ((buf[0]=getc(ffent)) == '#')
    {
       /* se ha leido un comentario... saltamos toda la linea */
       fgets(buf, 100, ffent);
    }
  /* devolvemos el caracter leido */
  ungetc(buf[0], ffent);

  /* leemos el ancho y el alto */
  fscanf(ffent, "%d %d \n%s \n", ancho, alto, buf);

  /* reserva de la memoria */
  tamano = *ancho * *alto;
  *img = (unsigned char*)malloc(tamano * sizeof(unsigned char));

  /* Lectura de la imagen formato RAW */
  if(fread(*img, sizeof(unsigned char), tamano ,ffent)!= tamano)
    {
      printf("Error en la lectura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      exit(1);
    }
  fclose(ffent);
} /* lee_pgm_uchar */





/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_pgm_float                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato pgm en la matriz img de float */
void lee_pgm_float (int *alto, int *ancho, char *nombre, float **img)
{
  int i,j;
  int aux;
  FILE *ffent;
  unsigned char buf[100];

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* lectura del ancho y alto. Salto de la cabecera. */
  fgets(buf, 100, ffent);

  /* Saltamos los comentarios PGM, marcados con '#' */
  while ((buf[0]=getc(ffent)) == '#')
    {
       /* se ha leido un comentario... saltamos toda la linea */
       fgets(buf, 100, ffent);
    }
  /* devolvemos el caracter leido */
  ungetc(buf[0], ffent);

  /* leemos el ancho y el alto */
  fscanf(ffent, "%d %d \n%s \n", ancho, alto, buf);


  /* reserva de la memoria */
  *img = (float *)malloc(*ancho * *alto * sizeof(float));

  /* Lectura de la imagen formato RAW */
  for (j=0; j< *alto; j++  )
    for (i=0; i< *ancho; i++  )
    {
      aux=fgetc(ffent);
      if (aux < 0)
	{
	  printf("Error en la lectura de %s i: %d j: %d\n",nombre,i,j);
	  printf("Error: %d  %s\n",errno,strerror(errno));
	  exit(1);
	}
	(*img)[*ancho * j +i]= aux;
    }
  fclose(ffent);
} /* lee_pgm_float */




/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_raw_float                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato raw a partir de la matriz img de float */
/* Se realiza una conversion de [-255.0 , 255.0] a [0 , 255] */ 
void escribe_raw_float (int alto, int ancho, char *nombre, float *img)
{
  int i,j;
  FILE *ffsal;
  
  printf("Escribiendo %s\n",nombre);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  
  /* Escritura de la imagen en formato RAW */
  for (j=0;j<alto;j++)
    {
      for (i=0;i<ancho;i++)
      {
	/* Aqui se realiza la conversion */
	if(fputc(img[ancho*j+i], ffsal) == EOF)
	  { 
	    printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	    printf("Error: %d  %s\n",errno,strerror(errno));
	    exit(1);
	  }
      }
    }
  fclose(ffsal);
} /* escribe_raw_float */



/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_float                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de la matriz img de float */
/* Se realiza una conversion de [-255.0 , 255.0] a [0 , 255] */ 
void escribe_pgm_float (int alto, int ancho, char *nombre, float *img)
{
  int i,j;
  FILE *ffsal;
  
  printf("Escribiendo %s\n",nombre);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  /* escribimos la cabecera */
  fprintf(ffsal, "P5\n%d %d\n255\n", ancho, alto);
  
  /* Escritura de la imagen en formato RAW */
  for (j=0;j<alto;j++)
    {
      for (i=0;i<ancho;i++)
      {
	/* Aqui se realiza la conversion */
	if(fputc(127.0+(img[ancho*j+i] / 2.0), ffsal) == EOF)
	  { 
	    printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	    printf("Error: %d  %s\n",errno,strerror(errno));
	    exit(1);
	  }
      }
    }
  fclose(ffsal);
} /* escribe_pgm_float */


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_float2                                               *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de la matriz img de float */
/* Muestra datos positivos y negativos en imagenes distintas */ 
void escribe_pgm_float2 (int alto, int ancho, char *nombre, float *img)
{
  int i,j;
  FILE *ffsal;
  
  printf("Escribiendo %s\n",nombre);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  /* escribimos la cabecera, el ancho es el doble */
  fprintf(ffsal, "P5\n%d %d\n255\n", 2*ancho, alto);
  
  /* Escritura de la imagen en formato RAW */
  for (j=0;j<alto;j++)
    {
      for (i=0;i<ancho;i++)
      {
	/* Filtramos solo los positivos */
	if (fputc((img[ancho*j+i]>0.0) ? img[ancho*j+i] : 0,ffsal) == EOF)
	  { 
	    printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	    printf("Error: %d  %s\n",errno,strerror(errno));
	    exit(1);
	  }
      }
      for (i=0;i<ancho;i++)
      {
	/* Ahora filtramos solo los negativos */
	if (fputc((img[ancho*j+i]<0.0) ? -img[ancho*j+i] : 0,ffsal) == EOF)
	  { 
	    printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	    printf("Error: %d  %s\n",errno,strerror(errno));
	    exit(1);
	  }
      }
    }
  fclose(ffsal);
} /* escribe_pgm_float2 */


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_uchar                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de datos unsigned char */
void escribe_pgm_uchar (int alto,int ancho,char *nombre,unsigned char *img)
{
  FILE *ffsal;
  
  printf("Escribiendo %s\n",nombre);
  
  ffsal=fopen(nombre,"wb");
  if (ffsal == NULL) 
    { 
      printf("Error en la creacion de %s\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }
  /* escribimos la cabecera */
  fprintf(ffsal, "P5\n%d %d\n255\n", ancho, alto);
  
  /* Escritura de la imagen en formato RAW */
  if(fwrite(img, sizeof(unsigned char), ancho*alto, ffsal) != ancho*alto)
    { 
      printf("Error en la escritura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      exit(1);
    }
  fclose(ffsal);
} /* escribe_pgm_uchar */



/******************* Cabecera de Funcion **********************************
*NOMBRE: leer_dimensiones                                                 *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS:                                                              *
*SALIDA: alto, ancho, nombre                                              *
*VAR.GLOBALES:MAX_ALTO,MAX_ANCHO                                          *
*                                                                         *
**************************************************************************/
/* Lee el ancho, alto y nombre de fichero desde el teclado, comprueba que*/
/* las dimensiones pertenecen al rango indicado por MAX_ALTO,MAX_ANCHO.  */
/* En caso de no pertenecer al rango el se termina todo el programa      */
void leer_dimensiones (int *ancho, int *alto, char *nombre)
{
  /* Lectura de las dimensiones de los ficheros */ 
  printf("\nFilas: ");
  scanf("%d",alto);
  printf("\nColumnas: ");
  scanf("%d",ancho);
  printf("\nFichero: ");
  scanf("%s",nombre);
  printf("\n");
  if ((*ancho>MAX_ANCHO) || (*alto>MAX_ALTO))
    {printf("Dimensiones demasiado grandes. Maximos -> Filas:%d\
  Columnas:%d\n", MAX_ALTO, MAX_ANCHO);
    exit(1);
    }
  if ((*ancho<1) || (*alto<1))
    {printf("Dimensiones demasiado pequeas. Minimos -> Filas:1\
  Columnas:1\n");
    exit(1);
    }
}/* leer_dimensiones */









