/***************** Cabecera de Mdulo *************************************
*MODULO: libpgm (libpgm.h)                                                *
*PERTENECE AL PROYECTO: cicyt3D                                           *
*DIRECTORIO:                                                              *
*HISTORIA DE VERSIONES:                                                   *
*     (VERSION   FECHA, AUTOR Y BREVE DESCRIPCION)                        *
*      linux1   8-7-97  jespa   entrada/salida probada satisfactoriamente *
*BIBLIOTECAS EXTERNAS:                                                    *
**************************************************************************/


/********************** SECCION DESCRIPCION MODULO ************************
* Libreria de entrada/salida en formato RAW y PGM de escala de grises     *
* La entrada y salida se realiza mediante buffer del sistema.             *
* Las matrices se almacenan en un solo bloque, una fila tras otra,        *
*   por lo que ya no se desperdicia nada de informacion                   *
* Las funciones que leen un fichero PGM hacen reserva de memoria de forma *
*   automatica. Las funciones que leen RAW no hacen la reserva.           *
* La funcion leer_dimensiones comprueba que los valores introducios estan *
* dentro de un intervalo y puede abortar (exit) la ejecucion del progrma. *
* Se supone que esta version es la mejor para UNIX.                       *
 **************************************************************************/


/********************** SECCION DE INCLUDES *******************************
**************************************************************************/


/********************** SECCION DE MACROS *********************************
**************************************************************************/
/* Tamanyo maximo de las matrices empleadas. En realidad solo se usa en la
   funcion leer_dimensiones(), por lo que no expresan una limitacion fisica
   de la libreria. */
#define MAX_ALTO  1024
#define MAX_ANCHO 1024

/********************** SECCION DE VARIABLES GLOBALES *********************
**************************************************************************/


/********************** SECCION DE RESUMEN FUNCIONES **********************
***************************************************************************
Nombre         Descripcion
======         ===========

 escribe_raw_uchar
 Escribe un fichero en formato raw a partir de la matriz img 

 lee_raw_uchar
 Lee un fichero en formato raw en la matriz img de unsigned char

 lee_raw_float
 Lee un fichero en formato raw en la matriz img de float  

 lee_pgm_uchar
 Lee un fichero en formato pgm en la matriz img de unsigned char

 lee_pgm_float
 Lee un fichero en formato pgm en la matriz img de float  

 escribe_pgm_float
 Escribe un fichero en formato pgm a partir de la matriz img de float 
 Se realiza una conversion de [-255.0 , 255.0] a [0 , 255]  

 escribe_pgm_float2
 Escribe un fichero en formato pgm a partir de la matriz img de float 
 Se realiza una conversion de [-255.0 , 255.0] a [0 , 255]  

 escribe_pgm_uchar
 Escribe un fichero en formato pgm a partir de la matriz de unsigned char 

 leer_dimensiones
 Lee el ancho, alto y nombre de fichero desde el teclado, comprueba que
 las dimensiones pertenecen al rango indicado por MAX_ALTO,MAX_ANCHO.  
 En caso de no pertenecer al rango el se termina todo el programa      

 */






/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_raw_uchar                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato raw a partir de la matriz img */
void escribe_raw_uchar (int alto, int ancho, char *nombre, unsigned char *img);

/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_raw_uchar                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato raw en la matriz img de unsigned char*/
void lee_raw_uchar(int alto, int ancho, char *nombre, unsigned char *img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_raw_float                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato raw en la matriz img de float */ 
void lee_raw_float (int alto, int ancho, char *nombre, float *img);

/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_pgm_uchar                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato pgm en la matriz img de unsigned char*/
void lee_pgm_uchar(int *alto, int *ancho, char *nombre, unsigned char **img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: lee_pgm_float                                                    *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre                                          *
*SALIDA: img                                                              *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Lee un fichero en formato pgm en la matriz img de float */ 
void lee_pgm_float (int *alto, int *ancho, char *nombre, float **img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_float                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de la matriz img de float */
/* Se realiza una conversion de [-255.0 , 255.0] a [0 , 255] */ 
void escribe_pgm_float(int alto, int ancho, char *nombre, float *img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_float2                                               *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de la matriz img de float */
/* Muestra datos positivos y negativos en imagenes distintas */ 
void escribe_pgm_float2(int alto, int ancho, char *nombre, float *img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: escribe_pgm_uchar                                                *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS: alto, ancho, nombre, img                                     *
*SALIDA:                                                                  *
*VAR.GLOBALES:                                                            *
*                                                                         *
**************************************************************************/
/* Escribe un fichero en formato pgm a partir de la matriz de unsigned char */
void escribe_pgm_uchar(int alto, int ancho, char *nombre, unsigned char *img);


/******************* Cabecera de Funcion **********************************
*NOMBRE: leer_dimensiones                                                 *
***************************************************************************
*DESCRIPCION:                                                             *
*PARAMETROS:                                                              *
*SALIDA: alto, ancho, nombre                                              *
*VAR.GLOBALES: MAX_ALTO,MAX_ANCHO                                         *
*                                                                         *
**************************************************************************/
/* Lee el ancho, alto y nombre de fichero desde el teclado, comprueba que*/
/* las dimensiones pertenecen al rango indicado por MAX_ALTO,MAX_ANCHO.  */
/* En caso de no pertenecer al rango el se termina todo el programa      */
void leer_dimensiones (int *ancho, int *alto, char *nombre);




















