/****************************************************************
 *  			Proceso carga.c				*
 *								*
 *  Proceso encargado de la informacion del sistema. Obtiene la *
 *  informacin local y la de los demas nodos			*
 *  								*
 *  Version: 1.0               Fecha:  6 - 9 - 2000		*
 *  Autor: Diego Chaparro Gonzalez				*
 *								*
 ****************************************************************/

#include "../include/bdd.h"

#define WORKTAG		1
#define DIETAG		2
	// Periodo de tiempo para actualizar informacin sobre noods
#define TIEMPOINF	10 

// Numero total de firmas que tiene el nodo actual para procesar
long total_firmas;

// Parmetros para el inicializar el MPI
MPI_Comm MPI_COMM_CARGA;
int sizeCARGA, rankCARGA, size, rank;


// Para guardar en la tabla, los nodos y sus velocidades dinmicas
struct reg {
  int nodo;
  float velocini; // velocidad inicial del nodo
  float tareas; // N de tareas medio
  float porcentaje; // Porcentaje realizado por el nodo
  time_t tultact; // Para saber cuando se actualiz por ltima vez
  int  ttotal; // Para saber cuanto tiemmpo ha transcurrido
  long nfirmas; // Para saber n firmas, puede cambiar
};
struct reg tabla[30];
struct reg tablaaux[30];


/****************************************************************************
 *                          FUNCION Numero_Tareas:                          *
 * ------------------------------------------------------------------------ *
 *    Manda la velocidad inicial local a los dems, y recibe la de los      *
 *   otros. Despus manda el nmero de tareas, y recibe la de los dems.    *
 *   Despus introduce los datos correctos en la tabla.                     *
 ****************************************************************************/
int Numero_Tareas (void) {
        
        FILE *fd;
	char cadnum1[10], cadnum2[10], cadnum3[10], cadnum[10];
	char *aux;
	int numero;
	
	fd = fopen ("/proc/loadavg", "r");

	fscanf (fd, "%s %s %s %s", cadnum1, cadnum2, cadnum3, cadnum);
	
	aux = strchr (cadnum, '/');
	*aux='\0';

	numero = atoi (cadnum);

	fclose(fd);
	
	return (numero);
}

/****************************************************************************
 *                       FUNCIN Mi_Velocidad:                              *
 * ------------------------------------------------------------------------ *
 * Devuelve la velocidad incial de cada nodo almacenada en el fichero veloc *
 ****************************************************************************/
float Mi_Velocidad (void) {
        
        FILE *fd;
	char cadnum[10];
	float numero;

	fd = fopen ("/var/tmp/distribuidoDiego/exe/veloc", "r");
	
	fscanf (fd, "%s", cadnum);

	numero = atof (cadnum);
	
	return (numero);
}


/****************************************************************************
 *                    PROCEDIMIENTO Inicializar_Tabla:                      *
 * ------------------------------------------------------------------------ *
 *    Manda la velocidad inicial local a los dems, y recibe la de los      *
 *   otros. Despus manda el nmero de tareas, y recibe la de los dems.    *
 *   Despus introduce los datos correctos en la tabla.
 ****************************************************************************/
void Inicializar_Tabla (time_t tult) {
        
        int env_ntareas, rec_ntareas;
	float env_veloc, rec_veloc;
	int i;
	MPI_Status status;
	
	// Mandar n de tareas y velocidad inicial local a todos los nodos
	env_ntareas = Numero_Tareas();
	env_veloc = Mi_Velocidad(); 
	for (i=0; i<sizeCARGA; i++){
	        if (i != rankCARGA ) {
	              MPI_Send (&env_ntareas, 1, MPI_INT, i, 0, MPI_COMM_CARGA);
		      MPI_Send (&env_veloc, 1, MPI_FLOAT, i, 1, MPI_COMM_CARGA);
		}
	          
	}

	// Recibir n de tareas y velocidad inicial de los dems
	for (i=0;i<sizeCARGA; i++) {
	        if (i!= rankCARGA) {
	              	MPI_Recv (&rec_ntareas, 1, MPI_INT, i, 0, MPI_COMM_CARGA, &status);
		      	MPI_Recv (&rec_veloc, 1, MPI_FLOAT, i, 1, MPI_COMM_CARGA, &status);
			
			tabla[i].nodo = i;
			tabla[i].velocini = rec_veloc;
			tabla[i].tareas = rec_ntareas;
			tabla[i].porcentaje = 0;
			tabla[i].tultact = tult;
			tabla[i].ttotal = 1;
			tabla[i].nfirmas = total_firmas;
		}
	}
	
	// Introduzco en la tabla los valores del nodo local
	tabla[rankCARGA].nodo = rankCARGA;
	tabla[rankCARGA].velocini = env_veloc;
	tabla[rankCARGA].tareas = env_ntareas;
	tabla[rankCARGA].porcentaje = 0;
	tabla[rankCARGA].tultact = tult;
	tabla[rankCARGA].ttotal = 1;
	tabla[rankCARGA].nfirmas = total_firmas;
		
}



/****************************************************************************
 *                        FUNCION Numero_Firmas:                            *
 * ------------------------------------------------------------------------ *
 *          Recibe como parmetro un fichero y devuelve el nmero de        *
 *                           lneas del mismo                               *
 ****************************************************************************/
long Numero_Firmas (char fichero[100]) {
        long num;
	char comando[200]= "wc -l >/var/tmp/distribuidoDiego/exe/numlin ";
	char cadaux[10];
	int cod;

	FILE *fd;
	
        strcat (comando, fichero);

	cod = system (comando);
	if (cod==127 || cod==-1)
	  printf ("ERROR: Al ejecutar system");
	fd = fopen ("/var/tmp/distribuidoDiego/exe/numlin", "r");
        fscanf (fd, "%s", cadaux);
	
	num = atol (cadaux);
	fclose (fd);

	return (num);
	
}



/****************************************************************************
 *                       FUNCIN Inicializar_MPI:                           *
 * ------------------------------------------------------------------------ *
 *   Inicializa MPI, crea el comunicador, halla el rango y el tamao, ...   *
 ****************************************************************************/
void Inicializar_MPI (int argc, char **argv) {

        int  color;

	MPI_Init (&argc, &argv);

	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Comm_rank (MPI_COMM_WORLD, &rank);

	// Creo el grupo para los demonios de carga
	color=2;
	MPI_Comm_split (MPI_COMM_WORLD, color, rank, &MPI_COMM_CARGA);
	MPI_Comm_rank (MPI_COMM_CARGA, &rankCARGA);
	MPI_Comm_size (MPI_COMM_CARGA, &sizeCARGA);

}



/***************************************************************************
 * 			FUNCIN Enviar_Tareas:				   *
 * ----------------------------------------------------------------------- *
 * 	Manda el nmero de tareas local a los dems nodos		   *
 ***************************************************************************/
void Enviar_Tareas () {
	int env_ntareas, i;

	// Mandar n de tareas local
	env_ntareas = Numero_Tareas();
	for (i=0; i<sizeCARGA; i++){
		if (i != rankCARGA )
			MPI_Send (&env_ntareas, 1, MPI_INT, i, 0, MPI_COMM_CARGA);
	}
}


	
/***************************************************************************
 * 			PROCEDIMIENTO Actualizar_Tabla;			   *
 * ----------------------------------------------------------------------- *
 *    		Actualiza los datos de un nodo en la tabla		   *
 ***************************************************************************/
void Actualizar_Tabla (int nodo, int ntar, time_t tult) {
	int indice;
	int dift;
	
	indice = 0;
	while (indice<sizeCARGA) {
		if (tabla[indice].nodo == nodo) {
			dift = difftime (tult, tabla[indice].tultact);
			tabla[indice].tareas = ((tabla[indice].tareas * tabla[indice].ttotal) + (ntar*dift))/(tabla[indice].ttotal+dift);
			tabla[indice].ttotal = tabla[indice].ttotal + dift;
			tabla[indice].tultact = tult;
			tabla[indice].porcentaje = ((tabla[indice].velocini * tabla[indice].ttotal / tabla[indice].nfirmas) * 100)/tabla[indice].tareas;
			indice = sizeCARGA;
		}
		else
			indice++;
			
	}
}


/***************************************************************************
 *                   MOSTRAR TABLA                                         *
 ***************************************************************************/

 void Mostrar_Tabla (void) {
 	int i;
	
	printf ("%dCARGA: ********  TABLA  **********\n", rankCARGA);
	printf ("NODO VELOCINI   TAREAS      PORC   TTOTAL   NFIRMAS\n");
	for (i=0; i<sizeCARGA; i++) 
		printf ("%d  %f  %f  %f    %d       %ld\n", tabla[i].nodo, tabla[i].velocini, tabla[i].tareas, tabla[i].porcentaje, tabla[i].ttotal, tabla[i].nfirmas);
		
}


/**************************************************************************
 * 			FUNCION Ordenar_Tabla				  *
 * ---------------------------------------------------------------------- *
 * 	Ordena la tabla y devuelve el nodo asociado al actual 		  *
 **************************************************************************/
void Ordenar_Tabla (void) {
	
	int orden;
	int i,j;
	struct reg aux;
	int salir, nuevonodo, nodotrans;
	MPI_Status status;
	int buffer;

	// Empleo el metodo de la burbuja
	for (i=1; i<(sizeCARGA-1); i++)
		for (j=0; j<(sizeCARGA-i); j++)
			if ((tabla[j].nfirmas-(tabla[j].porcentaje * tabla[j].nfirmas))>(tabla[j+1].nfirmas - (tabla[j+1].porcentaje*tabla[j+1].nfirmas))){
				
				//Mostrar_Tabla();
				aux.nodo = tabla[j].nodo;
				aux.velocini = tabla[j].velocini;
				aux.tareas = tabla[j].tareas;
				aux.porcentaje = tabla[j].porcentaje;
				aux.tultact = tabla[j].tultact;
				aux.ttotal = tabla[j].ttotal;
				aux.nfirmas = tabla[j].nfirmas;
				
				tabla[j].nodo = tabla[j+1].nodo;
				tabla[j].velocini = tabla[j+1].velocini;
				tabla[j].tareas = tabla[j+1].tareas;
				tabla[j].porcentaje = tabla[j+1].porcentaje;
				tabla[j].tultact = tabla[j+1].tultact;
				tabla[j].ttotal = tabla[j+1].ttotal;
				tabla[j].nfirmas = tabla[j+1].nfirmas;
				
				tabla[j+1].nodo = aux.nodo;
				tabla[j+1].velocini = aux.velocini;
				tabla[j+1].tareas = aux.tareas;
				tabla[j+1].porcentaje = aux.porcentaje;
				tabla[j+1].tultact = aux.tultact;
				tabla[j+1].ttotal = aux.ttotal;
				tabla[j+1].nfirmas = aux.nfirmas;
			}
	
	for (i=0; i<sizeCARGA; i++)
		if (tabla[i].nodo == (rankCARGA)) 
			orden = i;
			
	// Obtengo el nodo simetrico al nodo actual
	nodotrans = tabla[sizeCARGA-1-orden].nodo;

	// Mando numero de nodo al proceso de distribucion
	MPI_Send (&nodotrans, 1, MPI_INT, rank-sizeCARGA, 4, MPI_COMM_WORLD);

	// Se siguen enviando numero de nodos mientras que el proceso de distribucion
	// siga pidiendolos
	nuevonodo = sizeCARGA-1-orden;
	orden = nuevonodo;
	salir = 0;
	while (!salir) {
		MPI_Recv (&buffer, 1, MPI_LONG, rank-sizeCARGA, 1, MPI_COMM_WORLD, &status);
		if (buffer != 0) {
			if (nuevonodo >= orden && nuevonodo < sizeCARGA)
				nuevonodo ++;
			else
				if (nuevonodo == sizeCARGA)
					nuevonodo = orden-1;
				else
					nuevonodo --;
			nodotrans = tabla[nuevonodo].nodo;
			MPI_Send (&nodotrans, 1, MPI_INT, rank-sizeCARGA, 4, MPI_COMM_WORLD);
		} else
			salir = 1;
	}
	
}	



/**************************************************************************
 * 			PROCEDIMIENTO Resto_NFirmas 			  *
 * ---------------------------------------------------------------------- *
 * 	Modifica el nmero de firmas total en la tabla de nodo actual,	  *
 * 	porque ha pasado carga a otro nodo				  *
 **************************************************************************/
long  Resto_NFirmas (long firm){
	int i;
	long resul;
	
	i=0;
	while (i<sizeCARGA) {
		if (tabla[i].nodo == rankCARGA){
			tabla[i].nfirmas = tabla[i].nfirmas - firm;
			resul = tabla[i].nfirmas;
			i=sizeCARGA;
		} else
			i++;
	}
	total_firmas = total_firmas - firm;

	return (resul);
}


/**************************************************************************
 * 			PROCEDIMIENTO Informar_NFirmas 			  *
 * ---------------------------------------------------------------------- *
 * 	El nodo actual informa a los dems de su nuevo n de firmas,	  *
 * 	consecuencia de haber pasado carga a otro nodo			  *
 **************************************************************************/
void Informar_NFirmas (long firm) {
	
	int i;

	for (i=0; i<sizeCARGA; i++) 
		if (i!=rankCARGA)
			MPI_Send (&firm, 1, MPI_LONG, i, 5, MPI_COMM_CARGA);
}


/**************************************************************************
 * 			PROCEDIMIENTO Nuevo_NFirmas			  *
 * ---------------------------------------------------------------------- *
 * 	Asigna en la tabla el nuevo n de firmas del nodo actual,	  *
 * 	debido a la carga que le ha pasado otro nodo 			  *
 **************************************************************************/
void Nuevo_NFirmas (long firm) {
	int i;

	i=0;
	
	while (i<sizeCARGA){
		if (tabla[i].nodo == rankCARGA){
			tabla[i].nfirmas = firm;
			tabla[i].porcentaje = 0;
			tabla[i].ttotal = 1;
			i = sizeCARGA;
		} else
			i++;
	}
	total_firmas = firm;
}


/**************************************************************************
 * 			PROCEDIMIENTO Informar_Nuevo_NFirmas 		  *
 * ---------------------------------------------------------------------- *
 * 	Informa a los dems nodos de un nuevo nmero de firmas porque     *
 * 	otro nodo le ha pasado carga 					  *
 **************************************************************************/
void Informar_Nuevo_NFirmas (long firm) {

	int i;

	for (i=0; i<sizeCARGA; i++)
		if (i!=rankCARGA)
			MPI_Send (&firm, 1, MPI_LONG, i, 6, MPI_COMM_CARGA);
}


/**************************************************************************
 * 			PROCEDIMIENTO Actualizar_NFirmas 		  *
 * ---------------------------------------------------------------------- *
 * 	Actualiza el n firmas de un nodo que ha pasado carga a otro	  *
 **************************************************************************/
void Actualizar_NFirmas (int nodo, long firm){

	int i;

	i=0;
	while (i<sizeCARGA)
		if (tabla[i].nodo==nodo) {
			tabla[i].nfirmas = tabla[i].nfirmas-firm;
			i = sizeCARGA;
		} else
			i++;
}


/*************************************************************************
 * 			PROCEDIMIENTO Actualizar_Nuevo_NFirmas		 *
 * --------------------------------------------------------------------- *
 * 	Actualiza el nuevo n firmas de un nodo, ya que otro nodo le	 *
 * 	ha pasado carga 						 *
 *************************************************************************/
void Actualizar_Nuevo_NFirmas (int nodo, long firm){

	int i;

	i=0;
	while (i<sizeCARGA)
		if (tabla[i].nodo==nodo) {
			tabla[i].nfirmas = firm;
			tabla[i].ttotal = 1;
			tabla[i].porcentaje = 0;
			i = sizeCARGA;
		} else
			i++;
}


/**************************************************************************
 *		PROCEDIMIENTO Informar_No_Hay_Nodos			  *
 * ---------------------------------------------------------------------- *
 * 	Informa a los demas demonios de carga que no hay nodos 		  *
 * 	disponibles para realizar una transferencia de carga 		  *
 **************************************************************************/
void Informar_No_Hay_Nodos (void){
	
	int i;
	long buffer;
	
	buffer = 0;
	for (i=0; i<sizeCARGA; i++)
		if (i!=rankCARGA)
			MPI_Send (&buffer, 1, MPI_LONG, i, 7, MPI_COMM_CARGA);
}
			


/**************************************************************************
 *                   PROGRAMA PRINCIPAL  				  *
 **************************************************************************/


int main (argc, argv)
int argc;
char **argv;
{
        time_t tiempoini, tiempoult, tiempoact;
	int recibo, terminar;
	MPI_Status status, status2;
	long buffer;
	int error;
	
        Inicializar_MPI (argc, argv);
	
	// Calcular el tiempo inicial
	tiempoini = time (NULL);
	tiempoult = tiempoini;
	
	total_firmas = Numero_Firmas ("/var/tmp/distribuidoDiego/exe/firmas.en2");
	
	Inicializar_Tabla( time (NULL));

	// Permanece en este bucle, hasta que haya terminado el slave y no haya
	// nodos disponibles para realizar transferencias de carga
	terminar = 0;
	while (!terminar) {
		tiempoact = time (NULL);
		// Cuando pasa un periodo de tiempo TIEMPOINF actualizo informacion local
		if (tiempoact >tiempoult+TIEMPOINF) { 
			Enviar_Tareas ();
			tiempoult = tiempoact;
			Actualizar_Tabla (rankCARGA, Numero_Tareas(), tiempoact);
		}
		else {
			// Compruebo si recibo mensajes de MPI_COMM_CARGA
			MPI_Iprobe (MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_CARGA, &recibo, &status);
			if (recibo) {
				MPI_Recv (&buffer, 1, MPI_LONG, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_CARGA, &status2);
				switch (status2.MPI_TAG) {
					case 0: // Informacin sobre tareas
						Actualizar_Tabla (status2.MPI_SOURCE, buffer, tiempoact);
						break;
						
					case 5: // otro carga informa de n de firmas por paso a otro
                                                Actualizar_NFirmas (status2.MPI_SOURCE, buffer);
                                                break;

					case 6: // otro carga informa de nuevo n firmas porque otro nodo le ha pasado carga
						Actualizar_Nuevo_NFirmas (status2.MPI_SOURCE, buffer);
						break;
					case 7: // otro carga informa de que no hay nodos disponibles

						terminar = 1;
						break;
					default:
					        printf ("CARGA%d: RECIBO TAG INVALIDO", rankCARGA);
					        exit(1);
						
				}
				recibo = 0;
			}

			// Compruebo si recibo mensajes de MPI_COMM_WORLD
			MPI_Iprobe (MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &recibo, &status);
			if (!terminar && recibo) {
				MPI_Recv (&buffer, 1, MPI_LONG, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_WORLD, &status2);
				switch (status2.MPI_TAG) {
					case 1: // distrib ha terminado, y pide n nodo para conseguir carga
						Ordenar_Tabla();
						break;
					case 2: // distrib informa n firmas que ha pasado a otro nodo
						Informar_NFirmas (buffer);
						buffer = Resto_NFirmas (buffer);
						break;
					case 3: // no hay nodos para transferir carga
						Informar_No_Hay_Nodos();
						terminar = 1;
						break;
					case 4: // distrib informa n firmas que ha obtenido de otro
						Nuevo_NFirmas (buffer);
						Informar_Nuevo_NFirmas (buffer);
						break;
					default:
						printf ("CARGA%d: RECIBO TAG INVALIDO", rankCARGA);
						exit(1);
				}	
						
				recibo = 0;
				
			}
		}
	}
	
	terminar = 0;
	
	// El demonio de carga ha terminado su trabajo. Espera en el siguiente bucle
	// hasta que recibe un mensaje del master para acabar
	while (!terminar) {
		MPI_Iprobe (MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &recibo, &status);
		if (recibo) {
			MPI_Recv (&buffer, 1, MPI_LONG, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_WORLD, &status2);
			// recibo peticion de numero de nodo para transferir. Mando -1 para
			// indicar que no hay nodos disponibles
			if (status2.MPI_TAG==1){
				buffer = -1;
				MPI_Send (&buffer, 1, MPI_INT, status2.MPI_SOURCE, 4, MPI_COMM_WORLD);
			}
			// Si recibo mensaje del master finalizo
			if (status2.MPI_SOURCE==0) terminar = 1;
		}

		recibo =0;
		MPI_Iprobe (MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_CARGA, &recibo, &status);
		if (recibo){
				MPI_Recv(&buffer, 1, MPI_LONG, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_CARGA, &status2);
			}

		recibo = 0;

	}	
	
	MPI_Finalize ();
	exit (0);
}
