/***************** Cabecera de Mdulo *************************************
*MODULO: MODULO DE RUTINAS COMUNES A 2D Y 3D                              *
*PERTENECE AL PROYECTO:                                                   *
*DIRECTORIO: /home/acct/orobles/proyecto/onditas/vision/src/              *
*HISTORIA DE VERSIONES:                                                   *
*     (VERSION   FECHA, AUTOR                                             *
*          1.0    7-98  Oscar David Robles Sanchez                        *
*      .......   .....  ..........................                        *
**************************************************************************/

/********************** SECCION DESCRIPCION MODULO ************************
 Este modulo proporciona las operaciones comunes a 2D y 3D, que en realidad
se realizan sobre vectores, independientemente de la procedencia de los
datos.
**************************************************************************/

/********************** SECCION DE INCLUDES *******************************
**************************************************************************/
#include "../include/comun.h"

/********************** SECCION DE MACROS *********************************
**************************************************************************/
#define SWAP(a,b) temp.val=(a.val);temp.x=(a.x);temp.y=(a.y);(a.val)=(b.val);(a.x)=(b.x);(a.y)=(b.y);(b.val)=temp.val;(b.x)=temp.x;(b.y)=temp.y;

/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/



/*******
           ESTA ES PROVISIONAL
*******/
int Celdacompare (Celda *i, Celda *j)
{
  if (i->val < j->val)
    return (1);
  if (i->val > j->val)
    return (-1);
  return (0);
}


/******************* Cabecera de Funcion **********************************
*NOMBRE: SelectMenores                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Obtiene los k elementos mas pequenos de un vector.          *
*PARAMETROS:                                                              *
*   k:        Numero de elementos a encontrar.                            *
*   n:	      Tamano del vector en el que estan los datos.                *
*   arr:      Vector que contiene los datos.                              *
*SALIDA:                                                                  *
*   arr: Vector con los k elementos menores ordenados en las k primeras   *
*       posiciones y el resto de los datos en las demas posiciones.       *
*   Valor con la posicion que ocupa el mayor de los valores encontrados.  *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Extraido del NUMERICAL RECIPES IN C.                 
**************************************************************************/
 int SelectMenores(int k, unsigned long n, Celda *arr[])
{
  unsigned long i,ir,j,l,mid;
  Celda a,temp;

  l=0;
  ir=n-1;
  for(;;)
  {
    if (ir <= l+1)
    /* Active partition contains 1 or 2 elements */
    {
      if (ir == l+1 && (*arr)[ir].val < (*arr)[l].val) /* Case of 2 elements */
      {
        SWAP((*arr)[l],(*arr)[ir]);
      }
      return (k-1);
    }
    else
    /* Choose median of left, center, and right elements as partitioning
       element a. Also rearrange so that (*arr)[l].val <= (*arr)[l+1].val
       (*arr)[ir].val >= (*arr)[l+1].val
    */
    { 
      mid = (l+ir) >> 1;
      SWAP((*arr)[mid],(*arr)[l+1]);
      if ((*arr)[l].val > (*arr)[ir].val)
      {
        SWAP((*arr)[l],(*arr)[ir]);
      }
      if ((*arr)[l+1].val > (*arr)[ir].val)
      {
        SWAP((*arr)[l+1],(*arr)[ir]);
      }
      if ((*arr)[l].val > (*arr)[l+1].val)
      {
        SWAP((*arr)[l],(*arr)[l+1]);
      }
      /* Initializa pointers por partitioning */
      i=l+1;
      j=ir;
      /* Partitioning element */
      a.val=(*arr)[l+1].val;
      a.x=(*arr)[l+1].x;
      a.y=(*arr)[l+1].y;
      for(;;)
      /* Beggining of innermost loop */
      {
        /* Scan up to find element > a */
        do i++; while ((*arr)[i].val < a.val);
        /*Scan down to find element < a */
        do j--; while ((*arr)[j].val > a.val);
        if (j < i)
        /* Pointers crossed. Partitioning complete */
          break;

        SWAP((*arr)[i],(*arr)[j]);
      } /* End of innermost loop */
      /* Insert partitioning element */
      (*arr)[l+1].val=(*arr)[j].val;
      (*arr)[l+1].x=(*arr)[j].x;
      (*arr)[l+1].y=(*arr)[j].y;
    	
      (*arr)[j].val=a.val;
      (*arr)[j].x=a.x;
      (*arr)[j].y=a.y;
  
      /* Keep active the partition that contains the kth element */
      if (j >= k) ir=j-1;
      if (j <= k) l=i;
    }
  } /* for */
} /* SelectMenores */


/******************* Cabecera de Funcion **********************************
*NOMBRE: SelectMayores                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Obtiene los k elementos mayores de un vector.               *
*PARAMETROS:                                                              *
*   k:        Numero de elementos a encontrar.                            *
*   n:	      Tamano del vector en el que estan los datos.                *
*   arr:      Vector que contiene los datos.                              *
*SALIDA:                                                                  *
*   arr: Vector con los k elementos mayores ordenados en las k primeras   *
*       posiciones y el resto de los datos en las demas posiciones.       *
*   Valor con la posicion que ocupa el menor de los valores encontrados.  *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Extraido del NUMERICAL RECIPES IN C.                 
**************************************************************************/
 int SelectMayores(int k, unsigned long n, Celda *arr[])
{
  unsigned long i,ir,j,l,mid;
  Celda a,temp;

  l=0;
  ir=n-1;
  for(;;)
  {
    if (ir <= l+1)
    /* Active partition contains 1 or 2 elements */
    {
      if (ir == l+1 && (*arr)[ir].val > (*arr)[l].val) /* Case of 2 elements */
      {
        SWAP((*arr)[l],(*arr)[ir]);
      }
      return (k-1);
    }
    else
    /* Choose median of left, center, and right elements as partitioning
       element a. Also rearrange so that (*arr)[l].val >= (*arr)[l+1].val
       (*arr)[ir].val <= (*arr)[l+1].val
    */
    { 
      mid = (l+ir) >> 1;
      SWAP((*arr)[mid],(*arr)[l+1]);
      if ((*arr)[l].val < (*arr)[ir].val)
      {
        SWAP((*arr)[l],(*arr)[ir]);
      }
      if ((*arr)[l+1].val < (*arr)[ir].val)
      {
        SWAP((*arr)[l+1],(*arr)[ir]);
      }
      if ((*arr)[l].val < (*arr)[l+1].val)
      {
        SWAP((*arr)[l],(*arr)[l+1]);
      }
      /* Initializa pointers por partitioning */
      i=l+1;
      j=ir;
      /* Partitioning element */
      a.val=(*arr)[l+1].val;
      a.x=(*arr)[l+1].x;
      a.y=(*arr)[l+1].y;
      for(;;)
      /* Beggining of innermost loop */
      {
        /* Scan up to find element < a */
        do i++; while ((*arr)[i].val > a.val);
        /*Scan down to find element > a */
        do j--; while ((*arr)[j].val < a.val);
        /* Pointers crossed. Partitioning complete */
          break;

        SWAP((*arr)[i],(*arr)[j]);
      } /* End of innermost loop */
      /* Insert partitioning element */
      (*arr)[l+1].val=(*arr)[j].val;
      (*arr)[l+1].x=(*arr)[j].x;
      (*arr)[l+1].y=(*arr)[j].y;
    
      (*arr)[j].val=a.val;
      (*arr)[j].x=a.x;
      (*arr)[j].y=a.y;
  
      /* Keep active the partition that contains the kth element */
      if (j >= k) ir=j-1;
      if (j <= k) l=i;
    }
  } /* for */
} /* SelectMayores */
