/****************************************************************
 *  Proceso distrib.c						*
 *  Proceso encargado de realizar las labores de distribucion   *
 *  								*
 *  Version: 1.0                Fecha:  6 - 9 - 2000		*
 *  Autor: Diego Chaparro Gonzalez				*
 *								*
 ****************************************************************/			

#include "../include/bdd.h"
#include <errno.h>

#define WORKTAG		1
#define DIETAG		2

// Longitud de las lineas que indican el path de las imagenes
#define LONGLINEA	68

// Numero de firmas total a procesar por el nodo actual
long total_firmas;

// Parmetros para el inicializar el MPI
MPI_Comm MPI_COMM_DIST;
int sizeDIST, rankDIST, size, rank;


/****************************************************************************
 *                          FUNCION Numero_Tareas_Medio:                    *
 * ------------------------------------------------------------------------ *
 *	Devuelve el numero de tareas medio en el ultimo minuto.		    *
 * 	Este dato se obtiene del fichero /proc/loadavg			    *
 ****************************************************************************/

float Numero_Tareas_Medio (void) {
        
        FILE *fd;
	char cadnum1[10], cadnum2[10], cadnum3[10], cadnum[10];
	float numero;
	
	fd = fopen ("/proc/loadavg", "r");

	fscanf (fd, "%s %s %s %s", cadnum1, cadnum2, cadnum3, cadnum);
	
	numero = atof (cadnum1);

	fclose(fd);
	
	return (numero);
}

/****************************************************************************
 *                       FUNCION Mi_Velocidad:                              *
 * ------------------------------------------------------------------------ *
 * Devuelve la velocidad incial de cada nodo almacenada en el fichero veloc *
 ****************************************************************************/

float Mi_Velocidad (void) {
        
        FILE *fd;
	char cadnum[10];
	float numero;

	fd = fopen ("/var/tmp/distribuidoDiego/exe/veloc", "r");
	
	fscanf (fd, "%s", cadnum);

	numero = atof (cadnum);
	
	return (numero);
}

/****************************************************************************
 *                        FUNCION Numero_Firmas:                             *
 * ------------------------------------------------------------------------ *
 *          Recibe como parmetro un fichero y devuelve el nmero de        *
 *                           lneas del mismo                               *
 ****************************************************************************/
long Numero_Firmas (char fichero[100]) {
        long num;
	char comando[200]= "wc -l >/var/tmp/distribuidoDiego/exe/numlin ";
	char cadaux[10];
	int cod;

	FILE *fd;
	
        strcat (comando, fichero);

	cod = system (comando);
	if (cod==127 || cod==-1)
	  printf ("ERROR: Al ejecutar system");
	fd = fopen ("/var/tmp/distribuidoDiego/exe/numlin", "r");

        fscanf (fd, "%s", cadaux);
	
	num = atol (cadaux);
	fclose (fd);

	return (num);
	
}

/****************************************************************************
 *                       FUNCION Inicializar_MPI:                           *
 * ------------------------------------------------------------------------ *
 *   Inicializa MPI, crea el comunicador, halla el rango y el tamao, ...   *
 ****************************************************************************/
void Inicializar_MPI (void) {

        int  color;

	
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Comm_rank (MPI_COMM_WORLD, &rank);

	// Crea el comunicador MPI_COMM_DIST
	color=1;
	MPI_Comm_split (MPI_COMM_WORLD, color, rank, &MPI_COMM_DIST);
	MPI_Comm_rank (MPI_COMM_DIST, &rankDIST);
	MPI_Comm_size (MPI_COMM_DIST, &sizeDIST);

}

/****************************************************************************
 * 			PROCEDIMIENTO Pedir_Carga 		 	    *
 * ------------------------------------------------------------------------ *
 *   Cuando el slave ha terminado, este procedimiento pide la informacin   *
 *   sobre el nodo adecuado para realizar la transferencia, y realiza los   *
 *   pasos necesarios para llevarla a cabo.				    *
 ****************************************************************************/
int Pedir_Carga (void) {
	int nodotrans; // Indica el nmero de nodo con el que se realizar la transferencia
	MPI_Status status;
	int buffer;
	long bufferlong;
	long nfirmas; // n de firmas a recibir
	char *firmas; // todas las firmas recibidas
	int fd;
	long tamano;
	int iteracion;
	FILE *fd2;
	int salir, test;
	int velocidad;

	buffer=0;

	bufferlong = 0;

	iteracion = 0;
	nfirmas = 0;
	
	velocidad = (int) (Mi_Velocidad() / Numero_Tareas_Medio());
	
	// Permanece en el siguiente bucle hasta que obtiene un nodo adecuado para
	// realizar una transferencia, o hasta que comprueba que ninguno de los 
	// nodos posee carga para ser transferida
	while (nfirmas<1 && iteracion<sizeDIST) {

		// Envia peticion al demonio de carga de numero de nodo para transferencia
		MPI_Send (&iteracion, 1, MPI_LONG, (rank + sizeDIST), 1, MPI_COMM_WORLD);

		// Recibe el nodo para la transferencia
		MPI_Recv (&nodotrans, 1, MPI_INT, (rank + sizeDIST), 4,  MPI_COMM_WORLD, &status);
	
		if (nodotrans!=rankDIST)
			if (nodotrans==-1) // No hay nodos disponibles para transferencia
				iteracion = sizeDIST;
			else{
				// Envia al nodo seleccionado peticion de carga
				MPI_Send (&velocidad, 1, MPI_INT, nodotrans +(sizeDIST+1), 1, MPI_COMM_WORLD);
				salir = 0;
				while (!salir) {
					// Comprueba si recibe la respuesta del nodo
					MPI_Iprobe (nodotrans, 2, MPI_COMM_DIST, &test, &status);
					if (test)
						salir = 1;
					else {
						// Si recibe peticion de otro nodo le responde que no tiene carga.
						// Esto se hace para evitar inter bloqueos
						MPI_Iprobe (MPI_ANY_SOURCE, 1, MPI_COMM_WORLD, &test, &status);
						if (test) {
							MPI_Recv (&buffer, 1, MPI_INT, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_WORLD, &status);
							bufferlong = 0;
							MPI_Send (&bufferlong, 1, MPI_LONG, status.MPI_SOURCE-sizeDIST-1, 2, MPI_COMM_DIST);
						}
					}
				}
				
				// Recibo el numero de firmas que puede pasarme el nodo seleccionado
				MPI_Recv (&nfirmas, 1, MPI_LONG, nodotrans, 2, MPI_COMM_DIST, &status);
				if (nfirmas>0) iteracion = 1;
			}
		iteracion ++;
	}
		
	// Si he recorrido toda la lista o no hay nodos disponibles, el numero de firmas es 0
	if (nodotrans==-1 || iteracion>=sizeDIST)
		nfirmas = 0;
		
	iteracion = 0;
	// Le aviso al demonio de carga de que finaliza el proceso de eleccion de nodo
	MPI_Send (&iteracion, 1, MPI_LONG, rank+sizeDIST, 1, MPI_COMM_WORLD);
			
	// Si otro nodo tiene firmas para pasarme 
	if (nfirmas>0) {
		// Recibo el tamano del paquete que voy a recibir
		MPI_Recv (&tamano, 1, MPI_LONG, nodotrans, 5, MPI_COMM_DIST, &status);
		
		// Reservo memoria para el paquete que voy a recibir
		firmas = (char*) malloc (tamano);

		// Recibo las firmas para procesar
		MPI_Recv (firmas, tamano, MPI_CHAR, nodotrans, 3, MPI_COMM_DIST, &status);

		// Guardo las firmas en el fichero para el solver
		fd = open ("/var/tmp/distribuidoDiego/exe/firmas.en2", O_TRUNC | O_CREAT | O_WRONLY);
		write (fd, firmas, tamano);
		close(fd);

		// Calculo el nuevo numero de firmas, y lo meto en el fichero totalfirmas
		total_firmas = nfirmas;
		fd2 = fopen ("/var/tmp/distribuidoDiego/exe/totalfirmas", "w");
		fprintf (fd2, "%ld", total_firmas);
		fclose (fd2);
		fd2 = fopen ("/var/tmp/distribuidoDiego/exe/firmasreali", "w");
		bufferlong = 0;
		fprintf (fd2, "%ld", bufferlong);
		fclose (fd2);

		// Mando mensaje al slave para que vuelva a ejecutar
		MPI_Send (&buffer, 1, MPI_INT, (rank - sizeDIST), 0, MPI_COMM_WORLD);
		// Mando mensaje al demonio de carga con el nuevo numero de firmas
		MPI_Send (&nfirmas, 1, MPI_LONG, (rank + sizeDIST), 4, MPI_COMM_WORLD);
		return (0);
	} else 
		return (1);
}

 
/****************************************************************************
 * 			PROCEDIMIENTO Mandar_Carga			    *
 * ------------------------------------------------------------------------ *
 * 	Procedimiento que se ejecuta cuando se recibe una peticin de 	    *
 * 	carga. Se calcula el porcentaje realizado, y si es posible, se      *
 * 	manda el trabajo al nodo destino				    *
 ****************************************************************************/
void Mandar_Carga (int nododest, int veloc) {

	long firmasreali; // Nmero de firmas realizadas hasta el momento
	double porcreali; // Porcentaje realizado hasta el momento
	long nfirmas;
	char *firmas;
	char comando[200], cadaux[15];
	int fd;
	int cod;
	long leidos;
	FILE *fd2;
	FILE *fd3;
	int velocidad;
	long mifirmasordenar, susfirmasordenar;
	
	// Calculo el rango del nodo que solicita carga en MPI_COMM_DIST
	nododest = nododest - (sizeDIST) -1;
	
	// Obtengo el numero de firmas procesadas hasta el momento
	fd3 = fopen ("/var/tmp/distribuidoDiego/exe/firmasreali", "r");
	fscanf (fd3, "%ld", &firmasreali);
	fclose (fd3);
	
	// Hallo la velocidad relativa del nodo actual
	velocidad = (int) (Mi_Velocidad() / Numero_Tareas_Medio());
	
	// Calculo el porcentaje realizado en el nodo actual
	porcreali = ((float)firmasreali / (float)total_firmas)*100;
	
	// Calculo el numero de firmas que puedo pasar al nodo
	nfirmas = (int) (((float)veloc / (float)(veloc+velocidad))*(total_firmas-firmasreali));
	
	// Calculo el tiempo de ordenacion
	mifirmasordenar = (int) ( (((float) (total_firmas-nfirmas))/10000) * (float)velocidad);
	susfirmasordenar = (int)((((float)(nfirmas+mifirmasordenar))/10000)*veloc);

	// Obtengo las firmas despues de procesar el tiempo de ordenacion
	if (nfirmas>2000)
		nfirmas = nfirmas+mifirmasordenar-susfirmasordenar;

	if ((total_firmas-nfirmas)<firmasreali)
		nfirmas = total_firmas-firmasreali;

	// Si el numero de firmas es mayor de un umbral
	if (nfirmas>2000) {

		// Mando al nodo origen el numero de firmas a transferir
		MPI_Send (&nfirmas, 1, MPI_LONG, nododest, 2, MPI_COMM_DIST);

		// Reservo espacio para guardar las firmas a mandar
		firmas = (char*) malloc ( nfirmas * LONGLINEA );
		
		// Meto en un fichero las firmas a transferir
		strcpy (comando, "sed </var/tmp/distribuidoDiego/exe/firmas.en2 >/var/tmp/distribuidoDiego/exe/firmas.en2tmp -n ' ");
		sprintf (cadaux, "%ld", (total_firmas-nfirmas));
		strcat (comando, cadaux);
		strcpy (cadaux, ",");
		strcat (comando, cadaux);
		sprintf (cadaux, "%ld", total_firmas);
		strcat (comando, cadaux);
		strcpy (cadaux, "p'");
		strcat (comando, cadaux);
		cod = system (comando);
		total_firmas= total_firmas-nfirmas;
		if (cod==127 || cod==-1)
			printf ("ERROR: Al ejecutar system (tail)");
	
		// Abro el fichero y extraigo las firmas a transferir
		if ((fd = open ("/var/tmp/distribuidoDiego/exe/firmas.en2tmp", O_RDONLY))==-1)
			perror ("En el open");
		if ((leidos = read (fd, firmas, nfirmas * LONGLINEA))==-1)
			perror ("En el read: ");
		
		// Mando el tamano del paquete que se va a mandar
		MPI_Send (&leidos, 1, MPI_LONG, nododest, 5, MPI_COMM_DIST);
		
		// Mando las firmas a transferir
		MPI_Send (firmas, leidos, MPI_CHAR, nododest, 3, MPI_COMM_DIST);
		
		// Informo al demonio de carga de la transferencia de firmas
		MPI_Send (&nfirmas, 1, MPI_LONG, (rank + sizeDIST), 2, MPI_COMM_WORLD); 

		fd2 = fopen ("/var/tmp/distribuidoDiego/exe/totalfirmas", "w");
		fprintf (fd2, "%ld", total_firmas);
		fclose (fd2);
	} else {
		// Mando al nodo origen que no hay firmas para transferir
		nfirmas = 0;
		MPI_Send (&nfirmas, 1, MPI_INT, nododest, 2, MPI_COMM_DIST);
	}

}


/****************************************************************************
 *			PROGRAMA PRINCIPAL 				    *
 ****************************************************************************/


int main (int argc, char *argv[])
{
  	int buffer, buffer2;
	int terminar;
        MPI_Status status;
	
	
	MPI_Init (&argc, &argv);
        Inicializar_MPI ();
	
	total_firmas = Numero_Firmas ("/var/tmp/distribuidoDiego/exe/firmas.en2");
	terminar = 0;

	// Permanece en el bucle hasta que no haya nodos para transferir
	while (!terminar) {
		
		// Espero hasta que recibo un mensaje
		MPI_Recv (&buffer, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);

		switch (status.MPI_TAG) {
			case 0: // SLAVE ha terminado y tengo que pedir carga
				terminar = Pedir_Carga ();
				break;
			case 1: // recibo peticin de carga
				Mandar_Carga (status.MPI_SOURCE, buffer);
				break;
			default:
				printf ("%dDISTRIB: RECIBO TAG INVALIDO\n", rankDIST);
				exit (1);
		}

	}

	//Cuando el slave ha terminado pero no hay nodos para darme carga

	
	// Mando mensaje al slave para que termine
	MPI_Send (&buffer2, 1, MPI_INT, ( rank - sizeDIST ), 1, MPI_COMM_WORLD);
	
	// Mando mensaje a carga para decirle que no hay nodo para transferencia y espere a terminar
	MPI_Send (&buffer, 1, MPI_INT, ( rank + sizeDIST ), 3, MPI_COMM_WORLD);
	
	// Permanece en el bucle hasta que recibe mensaje del master para terminar
	terminar = 0;
	buffer = 0;
	while (!terminar) {

		MPI_Recv (&buffer, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);

		if (status.MPI_SOURCE==0) // Recibo del master y termino
			terminar = 1;
		else if (status.MPI_TAG==1) { // Recibo peticin de carga y respondo que no hay firmas
			buffer = 0;
			MPI_Send (&buffer, 1, MPI_INT, (status.MPI_SOURCE)-sizeDIST-1, 2, MPI_COMM_DIST);
		}
	}
	
	MPI_Finalize ();
	exit (0);
}
