/*
 *  Proceso master.c
 *  Implementacion de las funciones de gestion distribuida de la
 *  aplicacion de busqueda en basas de datOs por contenido.
 *  
 *  Version 1.0.	Fecha: 19 - 01 - 00
 */

#include "../include/bdd.h"
#include "../include/firmas.h"
#include <errno.h>

#define WORKTAG		1
#define DIETAG		2
#define NUM_WORK_REQS	200

int sizeMS;

MPI_Comm MPI_COMM_MS;

int main (argc, argv)
int	argc;
char	*argv[];

{
	parametros_t	param;
	file_t		file;
	int		myrank, size;
	tiempos_t	tiempos;
	tiempos_aux_t	taux;
	clock_t		tiniexe, tendexe;
	struct tms	t, tini, tend;
	int		i, buffer;
	resultados_t	resultados, res_def;
	int 		color, myrankMS;
	int 		numresul;
	//MPI_Op		selec;

	tiniexe = times (&tini);
/* 
 * Iniciar los parametros de ejecucion del solver.
 * Los valores se pasan en la linea de comandos junto con el nombre de la imagen.
 */
	if (argc < 4)
	{
		printf ("SINTAXIS: master <nombre_imagen> <en_analisis> <distancia>\n");
		exit (-1);
	} 

	sprintf (param.flag_ent[0], argv[2]);
	sprintf (param.flag_ent[1], argv[3]);

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
	MPI_Comm_size (MPI_COMM_WORLD, &size);

	printf ("MI rango Comm_World es: %d\n ",myrank);
	printf ("EL tamao de Comm_World es: %d\n ", size);

	color=0;
	MPI_Comm_split (MPI_COMM_WORLD, color, myrank, &MPI_COMM_MS);
	MPI_Comm_rank (MPI_COMM_MS, &myrankMS);
	MPI_Comm_size (MPI_COMM_MS, &sizeMS);
	printf ("MI rango en mi grupo es: %d\n ",myrankMS);
	printf ("EL tamao de mi grupo es: %d\n ", sizeMS);

	taux.fin2 = (clock_t *) malloc (size * sizeof (clock_t));
	
	Leer_parametros (&param);

	Calcular_firma (argv[1], "firma.dat");
	Leer_firma (&file);

	taux.ini1 = times (&t);
       	Enviar_parametros (param);

	Enviar_firma (file);

/* Recibir los resultados y seleccionar los que nos interesan */

        numresul = Recibir_resultados (&resultados, param, size, taux.fin2); 
/*
printf ("Resultados:\n ");
for (i=0; i< param.n_resul * (size-1); i++)
  {
    printf ("%s# %f\n", resultados.firmas[i], resultados.semejanzas[i]);
}
printf  ("\n");
*/
	Seleccionar_mejores (resultados, param, size, &res_def, numresul);
/*
printf ("Resultados definitivos:\n ");
for (i=0; i< param.n_resul; i++)
  {
    printf ("%s %f\n", res_def.firmas[i], res_def.semejanzas[i]);
}
printf  ("\n");
*/
	tendexe = times (&tend);
	tiempos.exetotal = (double) (tendexe - tiniexe)/CLK_TCK;
	tiempos.systotal = (double) (tend.tms_stime - tini.tms_stime)/CLK_TCK;
	tiempos.cputotal = (double) (tend.tms_utime - tini.tms_utime)/CLK_TCK;

	Recibir_tiempos (&tiempos, taux, size); 

	printf ("FIN\n");

	buffer=0;
	for (i=sizeMS; i<=(sizeMS-1)*3; i++)
		MPI_Send (&buffer, 1, MPI_INT, i, 0, MPI_COMM_WORLD);

	
	MPI_Finalize();
	return(0);
}

void Leer_parametros (param)
parametros_t *param;
{
  /*  printf ("Nombre del solver: ");
  scanf ("%s", param->nombre_solver);
  printf ("Numero de resultados: ");
  scanf ("%d",&param->n_resul);*/

	sprintf (param->nombre_solver, "/var/tmp/distribuidoDiego/exe/scoreq2D_s");
	param->n_resul = 100;
}

void Leer_firma (file)
file_t *file;
{
	int	error;
	FILE	*fd;
	struct stat status;

	/*	printf ("Nombre del fichero:  ");
		scanf ("%s", file->name);*/
	sprintf (file->name, "firma.dat");

        if ((fd = fopen (file->name, "r")) == NULL)
        {
		printf ("Error: no se puede abrir el fichero: %s\n", file->name);
      	       	MPI_Finalize ();
                exit (-1);
        }

	error = stat (file->name, &status);
	if (error!=0)
	{
		perror ("Error al leer el fichero");
		MPI_Finalize ();
		exit (-1);
	}

        file->size = status.st_size;

        file->contenido = (char *) malloc (file->size * sizeof (char));
        fread (file->contenido, sizeof (char), file->size, fd);

	fclose (fd);
}

void Enviar_parametros (param)
parametros_t param;
{
	unsigned int	msgsize, maxsize;
	int		pos;
	char		*buffer;
	//struct tms	t;

	MPI_Pack_size (50, MPI_CHAR, MPI_COMM_MS, &msgsize);
	maxsize = msgsize;
	MPI_Pack_size (1, MPI_INT, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	MPI_Pack_size (16, MPI_CHAR, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	MPI_Pack_size (16, MPI_CHAR, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	buffer = (char *) malloc (maxsize);
	pos = 0;

       	MPI_Pack (param.nombre_solver, 50, MPI_CHAR, buffer, maxsize, &pos, MPI_COMM_MS);	
	MPI_Pack (&param.n_resul, 1, MPI_INT, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&(param.flag_ent[0][0]), 16, MPI_CHAR, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&(param.flag_ent[1][0]), 16, MPI_CHAR, buffer, maxsize, &pos, MPI_COMM_MS);

	MPI_Bcast (buffer, pos, MPI_PACKED, 0, MPI_COMM_MS);
}

void Enviar_firma (file)
file_t file;
{
  /*
   * Hya dos alterntivas: enviar un dos Bcast uno con el tamao del fichero 
   * y otro con el propio fichero. Esto es necesario para inicializar el buffer.
   * O la otra opcion en enviar Send a todos los procesos slaves con un bucle.
   * Voy a probar primero la primera forma.
   */
	
	MPI_Bcast (&file.size, 1, MPI_INT, 0, MPI_COMM_MS);

	MPI_Bcast (file.contenido, file.size, MPI_CHAR, 0, MPI_COMM_MS);
}

void Esc_fich (file)
file_t file;
{
	FILE *fd;
	
	fd = fopen ("salida", "w");
	fwrite (file.contenido, sizeof (char), file.size, fd);
	fclose (fd);
}

int Recibir_resultados (res, p, size, fin2)
resultados_t *res;
parametros_t p;
int size;
clock_t *fin2;
{
	MPI_Datatype	MPI_firma;
       	//firma_t		env1 = "";
	//float		env2 = 0.0;
	int		n;
	MPI_Status	status, status1;
	struct tms	t;
	//int		trec;
	int 		recibo, buffer;
 	int 		terminados;	
	buffer = 0;
	
	MPI_Type_contiguous (70, MPI_CHAR, &MPI_firma); 
	MPI_Type_commit (&MPI_firma);
	
	// No se cuanto espacio tengo que reservar, porque no se el nmero de paquetes que
	// voy a recibir. Como el tamao no va a ser muy grande, le doy espacio de sobra.
	res->firmas = (firma_t *) malloc ((p.n_resul+1)  * size*3 * sizeof (firma_t));
        res->semejanzas = (float *) malloc (p.n_resul * size*3 *  sizeof (float));
	//taux.fin2 = (clock_t *) malloc (size * sizeof (clock_t));

	/*
	 * La solucion con MPI_Gather tiene un problema. Tambin recibe datos del propio proceso raiz, 
	 * que en este caso es el master y por lo tanto no nos interesa recibirlos. 
	 * Para que no halla problemas e incluido las variables env1 y env2, que estn vacias y
	 * por lo tanto los primeros valores almacenados en resultados no se deben utilizar. 
	 * EL primer valor real enviado por un slave est almacenado a partir de la posicin param.n_resul-1.
	 * Tambin hay que tener cuidado con la reserva de memoria: en el caso con Gather el malloc debe 
	 * hacerse con size. En el caso con Recv el mallo se hace de size-1.
	 */

	/*	MPI_Gather (&env1,p.n_resul, MPI_firma, res->firmas, p.n_resul, MPI_firma, 0, MPI_COMM_MS);
        	MPI_Gather (&env2,p.n_resul, MPI_FLOAT, res->semejanzas, p.n_resul, MPI_FLOAT, 0, MPI_COMM_MS);
	*/
/*
 * Recepcin de resultados con un bucle y MPI_Recv.
 * Funciona correctamente. Se pueden hacer medidas de rendimiento, frente a una solucin basada en 
 * operaciones globales de comunicacon.
 */
	terminados = 0;
	n = 0; // iteracin del bucle, es decir, nmero de paquete recibido
	while (terminados<(sizeMS-1)) 
	//for (n=0; n < (sizeMS-1); n++)
	  {
		  
	    MPI_Recv (&buffer, 1, MPI_INT, MPI_ANY_SOURCE, 100, MPI_COMM_MS, &status);
	    //printf ("MASTER: RECIBO MENSAJE DE SLAVE: %d\n", status.MPI_SOURCE);
	    
	    if (buffer !=1) {
 		MPI_Recv (&(res->firmas[n*p.n_resul]), p.n_resul, MPI_firma, status.MPI_SOURCE, 200, MPI_COMM_MS, &status);
		//printf ("MASTER: Primer resultado de %d\n", status.MPI_SOURCE);
	    	MPI_Recv (&(res->semejanzas[n*p.n_resul]), p.n_resul, MPI_FLOAT, status.MPI_SOURCE, 210, MPI_COMM_MS, &status1);
		//printf ("MASTER: Segundo resultado de %d\n", status1.MPI_SOURCE);
	   	n++; 
		printf ("MASTER: RECIBO RESULTADOS DE: %d\n", status1.MPI_SOURCE);
	    
	    } else {
		    
		    fin2[status.MPI_SOURCE] = times(&t);
		    terminados++;
		    printf ("MASTER: EL NODO %d HA TERMINADO \n", status.MPI_SOURCE);
	    }
			    
	  }
	return (n);
}

void Seleccionar_mejores (resultados, param, size, res_def, resul)
resultados_t resultados;
parametros_t param;
int size;
resultados_t *res_def;
int resul;
{
	float	aux;
	int	i, k, auxind;

	res_def->firmas = (firma_t *) malloc (param.n_resul * sizeof (firma_t));
        res_def->semejanzas = (float *) malloc (param.n_resul *  sizeof (float));

	for (k=0; k < param.n_resul; k++)
	  {
	    aux = resultados.semejanzas[0];
	    auxind = 0;

	    for (i=0; i < param.n_resul*resul; i++)
	      {
		if (aux > resultados.semejanzas[i])
		  {
		    aux = resultados.semejanzas[i];
		    auxind = i;
		  }
	      }
	    res_def->semejanzas[k] = resultados.semejanzas[auxind];
	    strcpy (&(res_def->firmas[k]), &resultados.firmas[auxind]);
	    resultados.semejanzas[auxind] = 100000;
	  }
}

void Recibir_tiempos (tiempos, taux, size)
tiempos_t *tiempos;
tiempos_aux_t taux;
int size;
{
	unsigned int	msgsize = 40;
	int		pos, n, nodo;
	char		*buffer;
	clock_t		tcom;
	MPI_Status	status;
	FILE		*fdtiempos;

	tiempos->comtotal = 0;
	buffer = (char *) malloc (msgsize);
	
	if ((fdtiempos = fopen ("tiempos", "w")) == NULL)
        {
		printf ("Error: no se puede abrir el fichero: tiempos\n");
      	       	MPI_Finalize ();
                exit (-1);
        }

	fprintf (fdtiempos, "RESULTADOS DE TIEMPOS DE EJECUCION Y COMUNICACION\n");
	fprintf (fdtiempos, "--------------------------------------------------\n");
	for (n=1; n < sizeMS; n++)
	  {
	    pos = 0;

	    MPI_Recv (buffer, msgsize, MPI_PACKED, MPI_ANY_SOURCE, 300, MPI_COMM_MS, &status);
	    nodo = status.MPI_SOURCE;

	    MPI_Unpack (buffer, msgsize, &pos, &tiempos->exenodo, 1, MPI_DOUBLE, MPI_COMM_MS);
	    MPI_Unpack (buffer, msgsize, &pos, &tiempos->sysnodo, 1, MPI_DOUBLE, MPI_COMM_MS);
	    MPI_Unpack (buffer, msgsize, &pos, &tiempos->cpunodo, 1, MPI_DOUBLE, MPI_COMM_MS);

	    MPI_Unpack (buffer, msgsize, &pos, &taux.fin1, 1, MPI_INT, MPI_COMM_MS);
	    MPI_Unpack (buffer, msgsize, &pos, &taux.ini2, 1, MPI_INT, MPI_COMM_MS);

	    tcom = abs (taux.fin1 + taux.fin2[nodo] - taux.ini1 - taux.ini2);
	    tiempos->comnodo = (double) tcom/CLK_TCK;
	    tiempos->comtotal += tiempos->comnodo;
	   
	    fprintf (fdtiempos, "NODO: %d\n", nodo);
	    fprintf (fdtiempos, "Ejecucin, cpu y systema:\t%.4f\t%.4f\t%.4f\n", tiempos->exenodo, tiempos->cpunodo, tiempos->sysnodo);
	    fprintf (fdtiempos, "Comunicacion:\t\t\t%.4f\n", tiempos->comnodo);
	    fprintf (fdtiempos, "-------------------------------------------------------------------------------\n");
	  }
	  fprintf (fdtiempos, "\n\nComunicacion Total:\t\t%.4f\n\n", tiempos->comtotal);
	  fprintf (fdtiempos, "Ejecucin, cpu y systema total:\t%.4f\t%.4f\t%.4f\n", tiempos->exetotal, tiempos->cputotal, tiempos->systotal);
	  fclose (fdtiempos);
}

