/*
 *  Proceso slave.c
 *  Implementacion de las funciones de gestion distribuida de la
 *  aplicacion de busqueda en basas de datOs por contenido.
 *  
 *  Version 1.0.	Fecha: 19 - 01 - 00
 */

#include "../include/bdd.h"
#include <errno.h>

#define WORKTAG		1
#define DIETAG		2

MPI_Comm MPI_COMM_MS;
int main (argc, argv)
int argc;
char **argv;
{
	int             size, rank;
	parametros_t	param;
	file_t		file;
	double		texenodo, tsysnodo, tcpunodo;
	clock_t		tinicom, tendcom;
	resultados_t	resultados;
	clock_t 	tiniexe, tendexe;
	struct tms 	tini, tend;
	FILE 		*fd;
	int  sizeMS, rankMS, color;
	int terminar;
	int bufaux, bufaux2;
	MPI_Status status;
	
	tiniexe = times (&tini);

	MPI_Init (&argc, &argv);
	
	
	MPI_Comm_size (MPI_COMM_WORLD, &size);
	MPI_Comm_rank (MPI_COMM_WORLD, &rank);
//	printf ("S: Mi rango en COmmWorld es: %d\n", rank);
//	printf ("S: El tamao de CommWOrld es: %d\n", size);

	color=0;
	MPI_Comm_split (MPI_COMM_WORLD, color, rank, &MPI_COMM_MS);
	MPI_Comm_rank (MPI_COMM_MS, &rankMS);
	MPI_Comm_size (MPI_COMM_MS, &sizeMS);
//	printf ("S: El rango en mi grupo es: %d\n", rankMS);
//	printf ("S: El tamao de mi grupo es: %d\n", sizeMS);
	
	Recibir_parametros (&param);

	Recibir_firma (&file, &tendcom);

	terminar = 0;
	bufaux = 0;	
	while (!terminar) {

	   	Ejecutar_solver (param.nombre_solver, param);
		printf ("%dSLAVE: He terminado de ejecutar el solver\n ", rankMS);
		/* Enviar los resultados parciales al master */
		Leer_resultados (&resultados, param);

		Enviar_resultados (resultados, param, &tinicom);

		MPI_Send (&bufaux, 1, MPI_INT, ( rankMS + ( sizeMS - 1 )), 0, MPI_COMM_WORLD);
		printf ("%dSLAVE: Ya se lo he dicho a distrib(%d)\n ", rankMS, (rank+(sizeMS-1)));

		MPI_Recv (&bufaux2, 1, MPI_INT,  rank+sizeMS- 1, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
		if (status.MPI_TAG==0){
			printf ("%dSLAVE: Distrib me ha dicho que ejecute otra vez \n", rankMS);
			terminar=0;
		} else {
			printf ("%dSLAVE: DIstrib me ha dicho que acabe \n", rankMS);
			terminar = 1;
		}
	}

	bufaux = 1;
	//printf ("%dSLAVE: Salgo de bucle y mando mensaje a MASTER\n", rankMS);
	MPI_Send (&bufaux, 1, MPI_INT, 0, 100, MPI_COMM_MS);
	
	tendexe = times (&tend);
	texenodo = (double) (tendexe - tiniexe) / CLK_TCK;
/*	tsysnodo = (double) (tend.tms_stime - tini.tms_stime) / CLK_TCK;
	tcpunodo = (double) (tend.tms_utime - tini.tms_utime) / CLK_TCK;*/

	if ((fd = fopen ("/var/tmp/distribuidoDiego/exe/tiempos", "r")) == NULL)
	{
		printf ("Error: no puedo leer el fichero **tiempos**\n");
		exit (-1);
	}
	fscanf (fd, "%lf %lf\n", &tcpunodo, &tsysnodo);
	fclose (fd);
	printf ("Tiempos nodo[%d]: %.4f %.4f %.4f \n", rank, texenodo, tcpunodo, tsysnodo);
	Enviar_tiempos (texenodo, tsysnodo, tcpunodo, tendcom, tinicom);
	
	MPI_Finalize ();
	exit (0);
}

void Recibir_parametros (param)
parametros_t *param;
{
       	unsigned int    msgsize = 100;
	int		pos;
	char		*buffer;

	pos = 0;

	/*
	 * El valor msgsize que hace referencia al tamao del mensaje empaquetado que se
	 * recibe est calculado para los parmetros actuales. Esto permite ahorrar un envio 
	 * desde el master (que es el MPI_Recv comentado que aparece a continuacin. 
	 * En caso de cambiar los parmetros hay que modificar tambin este valor o bien 
	 * utilizar la funcin de envio recepcion.
	 */
	//MPI_Recv (&maxsize, 1, MPI_INT, 0, 105, MPI_COMM_WORLD, &status);
	
        buffer = (char *) malloc (msgsize);

       	MPI_Bcast (buffer, msgsize, MPI_PACKED, 0, MPI_COMM_MS);

       	MPI_Unpack (buffer, msgsize, &pos, &param->nombre_solver, 50, MPI_CHAR, MPI_COMM_MS);
	MPI_Unpack (buffer, msgsize, &pos, &param->n_resul, 1, MPI_INT, MPI_COMM_MS);
	MPI_Unpack (buffer, msgsize, &pos, &param->flag_ent[0], 16, MPI_CHAR, MPI_COMM_MS);
	MPI_Unpack (buffer, msgsize, &pos, &param->flag_ent[1], 16, MPI_CHAR, MPI_COMM_MS);
}

void Recibir_firma (file, tendcom)
file_t *file;
clock_t *tendcom;
{
	FILE		*fd;
	struct tms	t;

/* Recibo el tamao del ficheo para iniciar el buffer */
	MPI_Bcast (&file->size, 1, MPI_INT, 0, MPI_COMM_MS);

/* Reservo memoria en funcion del tamao del fichero de firma. */
	file->contenido = (char *) malloc (file->size);

/* Recibo el contenido del fichero del firma. */
	MPI_Bcast (file->contenido, file->size, MPI_CHAR, 0, MPI_COMM_MS);

	/* fin de las funciones de comunicacion. */
	(*tendcom) = times (&t);

/* Escribo la firma en el disco */
	if ((fd = fopen ("/var/tmp/distribuidoDiego/exe/firma.dat", "w")) == NULL)
	{
	    printf ("Error: no puedo abrir el fichero firma\n");
	    MPI_Finalize ();
	    exit (-1);
	}
	fwrite (file->contenido, sizeof (char), file->size, fd);
	fclose (fd);
}

void Ejecutar_solver (solver, param)
char *solver;
parametros_t	param;
{
	int pid, status;

	if ((pid = fork ()) != 0)
	{
		wait (&status);

	}
	else
	{
		execl (solver, solver, param.flag_ent[0], param.flag_ent[1], (char *)0, (char*)0);

	}

	if (status != 0)
	{
		perror ("Error en Execl:");
	}
}

void Esc_param (p)
parametros_t p;
{
	printf ("Nombre solver: %s\n", p.nombre_solver);
	printf ("Numero de resultados: %d\n", p.n_resul);
}

void Leer_resultados (res, param)
resultados_t *res;
parametros_t param;
{
	FILE *fd;
	int r;

	res->firmas = (firma_t *) malloc (param.n_resul * sizeof (firma_t));
	res->semejanzas = (float *) malloc (param.n_resul * sizeof (float));

/* Leo el fichero de resultados. */
	if ((fd = fopen ("/var/tmp/distribuidoDiego/exe/solver.ord", "r")) == NULL)
	{
	    printf ("Error: no puedo abrir el fichero solver.ord\n");
	    MPI_Finalize ();
	    exit (-1);
	}

	for (r=0; r < param.n_resul; r++)
	{	    fscanf (fd, "%s %f", &(res->firmas[r]), &(res->semejanzas[r]));
//		    printf ("***prueba: %s %f\n", res->firmas[r], res->semejanzas[r]);
	}
	fclose (fd);
}
	
void Enviar_resultados (resultados, param, tinicom)
resultados_t resultados;
parametros_t param;
clock_t *tinicom;
{
	MPI_Datatype	MPI_firma;
	//firma_t		rec1;
	//float		rec2;
	struct tms	t;
	int 		buffer;
	
	MPI_Type_contiguous (64, MPI_CHAR, &MPI_firma); 
	MPI_Type_commit (&MPI_firma);
       
	*tinicom = times (&t);

	/*
	 	MPI_Gather (resultados.firmas, param.n_resul, MPI_firma, rec1, 0, MPI_firma, 0, MPI_COMM_MS);
	 MPI_Gather (resultados.semejanzas, param.n_resul, MPI_FLOAT, 0, &rec2, MPI_FLOAT, 0, MPI_COMM_MS);
	*/	
	
	buffer=0;
	MPI_Send (&buffer, 1, MPI_INT, 0, 100, MPI_COMM_MS);
	
	MPI_Send (resultados.firmas, param.n_resul, MPI_firma, 0, 200, MPI_COMM_MS);

       	MPI_Send (resultados.semejanzas, param.n_resul, MPI_FLOAT, 0, 210, MPI_COMM_MS);
}

void Enviar_tiempos (texenodo, tsysnodo, tcpunodo, tendcom, tinicom)
double texenodo, tsysnodo, tcpunodo;
clock_t tendcom, tinicom;
{
	unsigned int	msgsize, maxsize;
	int		pos;
	char		*buffer;

	MPI_Pack_size (1, MPI_DOUBLE, MPI_COMM_MS, &msgsize);
	maxsize = msgsize;
	MPI_Pack_size (1, MPI_DOUBLE, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	MPI_Pack_size (1, MPI_DOUBLE, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	MPI_Pack_size (1, MPI_INT, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;
	MPI_Pack_size (1, MPI_INT, MPI_COMM_MS, &msgsize);
	maxsize += msgsize;

	buffer = (char *) malloc (maxsize);
	pos = 0;

	MPI_Pack (&texenodo, 1, MPI_DOUBLE, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&tsysnodo, 1, MPI_DOUBLE, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&tcpunodo, 1, MPI_DOUBLE, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&tendcom, 1, MPI_INT, buffer, maxsize, &pos, MPI_COMM_MS);
	MPI_Pack (&tinicom, 1, MPI_INT, buffer, maxsize, &pos, MPI_COMM_MS);

	MPI_Send (buffer, pos, MPI_PACKED, 0, 300, MPI_COMM_MS);
}
