/***************** Cabecera de Mdulo *************************************
*MODULO: MODULO DE RUTINAS PARA EXTRAER LA FIRMA Y COMPARAR IMAGENES 2D   *
*PERTENECE AL PROYECTO:                                                   *
*DIRECTORIO: /home/acct/orobles/proyecto/onditas/vision/src/              *
*HISTORIA DE VERSIONES:                                                   *
*     (VERSION   FECHA, AUTOR                                             *
*          1.0    7-98  Oscar David Robles Sanchez                        *
*      .......   .....  ..........................                        *
**************************************************************************/

/********************** SECCION DESCRIPCION MODULO ************************
 En este modulo se encuentran las rutinas necesarias para extraer la firma
a imagenes 2D y para, posteriormente, compararlas.
**************************************************************************/

/********************** SECCION DE INCLUDES *******************************
**************************************************************************/
#include "../include/querying2D.h"

/********************** SECCION DE MACROS *********************************
**************************************************************************/
#define SWAP(a,b) temp.val=(a.val);temp.x=(a.x);temp.y=(a.y);(a.val)=(b.val);(a.x)=(b.x);(a.y)=(b.y);(b.val)=temp.val;(b.x)=temp.x;(b.y)=temp.y;
#ifndef NORMALIZAR
#define NORMALIZAR
#endif


/********************** SECCION DE DEFINICION FUNCIONES *******************
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: Convertir2D                                                      *
***************************************************************************
*                                                                         *
*DESCRIPCION: Adapta los datos de una imagen almacenada en un vector de   *
*            tipo TIPO a un vector de tipo Celda.                        *
*PARAMETROS:                                                              *
*   in:       Imagen de entrada que se quiere cambiar de formato.         *
*   out:      Imagen de salida, ya cambiada al nuevo formato.             *
*   tamx:     Numero de filas de la imagen.                               *
*   tamy:     Numero de columnas de la imagen.                            *
*SALIDA:                                                                  *
*    out: Vector con los datos de la imagen adaptados al tipo de          *
*        informacion que se almacena en Celda.                            *
*VAR.GLOBALES: Ninguna.                                                   *
*ALGORITMO: Se recorre de forma lineal el vector de datos para pasar cada *
*   elemento al nuevo tipo de datos.                                      *
**************************************************************************/
void Convertir2D (TIPO *in, Celda **out, int tam_x, int tam_y)
{
  int j,k;
 
  *out = (Celda *)malloc(tam_x*tam_y*sizeof(Celda));
  if (!(*out))
  {
    printf("\nNo hay memoria suficiente\n");
    exit(1);
  }
  for (j=0; j<tam_x; j++)
  {
    for (k=0; k<tam_y; k++)
    {
      ((*out)[k*tam_y+j]).val=in[k*tam_y+j];
      ((*out)[k*tam_y+j]).x=j;
      ((*out)[k*tam_y+j]).y=k;
    }
  }
} /* Convertir2D */


/******************* Cabecera de Funcion **********************************
*NOMBRE: CortarYConvertir2D                                               *
***************************************************************************
*                                                                         *
*DESCRIPCION: Adapta los datos de una imagen almacenada en un vector de   *
*            tipo TIPO a un vector de tipo Celda.                         *
*PARAMETROS:                                                              *
*   in:       Imagen de entrada que se quiere cambiar de formato.         *
*   out:      Imagen de salida, ya cambiada al nuevo formato.             *
*   res:      Maximo nivel de resolucion.                                 *
*SALIDA:                                                                  *
*    out: Vector con los datos de la imagen adaptados al tipo de          *
*        informacion que se almacena en Celda.                            *
*VAR.GLOBALES: Ninguna.                                                   *
*ALGORITMO: Se recorre de forma lineal el vector de datos para pasar cada *
*   elemento al nuevo tipo de datos. Ademas, se queda con los res menores *
*   niveles de resolucion.                                                *
**************************************************************************/
void CortarYConvertir2D (TIPO *in, Celda **out, int tam_final)
{
  int j,k;

  *out = (Celda *)malloc(tam_final*tam_final*sizeof(Celda));
  if (!(*out))
  {
    printf("\nNo hay memoria suficiente\n");
    exit(1);
  }
  for (j=0; j<tam_final; j++)
  {
    for (k=0; k<tam_final; k++)
    {
      ((*out)[k*tam_final+j]).val=in[k*tam_final+j];
      ((*out)[k*tam_final+j]).x=j;
      ((*out)[k*tam_final+j]).y=k;
    }
  }
} /* CortarYConvertir2D */



/******************* Cabecera de Funcion **********************************
*NOMBRE: Lee_pnm_TIPO                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Lee una imagen tipo TIPO y la almacena en un vector TIPO  *
*PARAMETROS:                                                              *
*   alto:     Numero de filas de la imagen.                               *
*   ancho:    Numero de columnas de la imagen.                            *
*   nombre:   Ruta al fichero donde se encuentra la imagen a leer.        *
*   img1,img2, img3: En cada img se almacena uno de los planos de color   *
*             de la imagen.                                               *
*SALIDA:                                                                  *
*    img1, img2, img3: se obtiene cada plano de color por separado, para  *
*        su tratamiento individual.                                       *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se recorre de forma lineal el fichero origen y se almacena en *
*   el vector, teniendo en cuenta que es un vector unidimensional.        *
**************************************************************************/
void Lee_pnm_TIPO (int *alto, int *ancho, char *nombre,
		    TIPO **img1, TIPO **img2, TIPO **img3)
{
  int i,j, contador=0;
  int retonno;
  FILE *ffent;
  unsigned char c,aux,buf[100];

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* lectura del ancho y alto. Salto de la cabecera. */
  fgets((unsigned char *)buf, 100, ffent);

  /* Saltamos los comentarios PGM, marcados con '#' */
  while ((buf[0]=getc(ffent)) == '#')
    {
       /* se ha leido un comentario... saltamos toda la linea */
       fgets((unsigned char *)buf, 100, ffent);
    }
  /* devolvemos el caracter leido */
  ungetc(buf[0], ffent);

  /* leemos el ancho y el alto */
  fscanf(ffent, "%d %d\n", ancho, alto);

  /* leemos el maximo nivel de gris" */
  fscanf(ffent, "%d", &i);
  /* Lectura del \n que va despues de ese numero */
  c=getc(ffent);
  if (c != '\n')
  {
    printf("\nNO SE HA LEIDO BIEN LA CABECERA\n");
    exit(1);
  }

  /* reserva de la memoria */
  *img1 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));
  *img2 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));
  *img3 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));

  /* Lectura de la imagen formato RAW */
  for (j=0; j< *alto; j++  )
  {
    for (i=0; i< *ancho; i++  )
    {
      retonno = fread(&aux, sizeof(unsigned char), 1, ffent);
      if ((retonno != 1) && (ferror(ffent)) && (feof(ffent)))
      {
	printf("Error en la lectura de %s i: %d j: %d, PLANO 1\n",nombre,i,j);
	printf("El contador se quedo en %d\n", contador);
	printf("Error: %d  %s\n",errno,strerror(errno));
	exit(1);
      }
      else
      {
	(*img1)[*ancho * j +i]= aux;
	retonno = fread(&aux, sizeof(unsigned char), 1, ffent);
	if ((retonno != 1) && (ferror(ffent)) && (feof(ffent)))
        {
	  printf("Error en la lectura de %s i: %d j: %d, "
		 "PLANO 2\n",nombre,i,j);
	  printf("El contador se quedo en %d\n", contador);
	  printf("Error: %d  %s\n",errno,strerror(errno));
	  exit(1);
	}
	else
        {
	  (*img2)[*ancho * j +i]= aux;
	  retonno = fread(&aux, sizeof(unsigned char), 1, ffent);
	  if ((retonno != 1) && (ferror(ffent)) && (feof(ffent)))
          {
	      printf("Error en la lectura de %s i: %d j: %d, "
		     "PLANO 3\n",nombre,i,j);
	      printf("El contador se quedo en %d\n", contador);
	      printf("Error: %d  %s\n",errno,strerror(errno));
	      exit(1);
	  }
	  else
          {
	    contador++;
	    (*img3)[*ancho * j +i]= aux;
	  } /* else del tercer fread */
	} /* else del segundo fread */
      } /* else del primer fread */
    }  /* for i*/
  } /* for j */
  if (contador != ((*ancho)*(*alto)))
  {
    printf("\nGUARNING: NO SE HA LEIDO EL NUMERO CORRECTO DE BYTES\n");
    printf("Leyendo la imagen %s\n\n", nombre);
    free(*img1);*img1=NULL;
    free(*img2);*img2=NULL;
    free(*img3);*img3=NULL;
  }
  fclose(ffent);
} /* Lee_pnm_TIPO */


/******************* Cabecera de Funcion **********************************
*NOMBRE: Lee_pgm_TIPO                                                     *
***************************************************************************
*                                                                         *
*DESCRIPCION: Lee una imagen tipo TIPO y la almacena en un vector TIPO    *
*PARAMETROS:                                                              *
*   alto:     Numero de filas de la imagen.                               *
*   ancho:    Numero de columnas de la imagen.                            *
*   nombre:   Ruta al fichero donde se encuentra la imagen a leer.        *
*   img1,img2, img3: En cada img se almacena uno de los planos de color   *
*             de la imagen.                                               *
*SALIDA:                                                                  *
*    img1, img2, img3: se obtiene cada plano de color por separado, para  *
*        su tratamiento individual.                                       *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se recorre de forma lineal el fichero origen y se almacena en *
*   el vector, teniendo en cuenta que es un vector unidimensional.        *
**************************************************************************/
void Lee_pgm_TIPO (int *alto, int *ancho, char *nombre,
		    TIPO **img1, TIPO **img2, TIPO **img3)
{
  int i,j, contador=0;
  int retonno;
  FILE *ffent;
  unsigned char c,aux,buf[100];

  /* Abrimos el fichero con entrada formateada */
  ffent = fopen (nombre, "rb");
  if (ffent == NULL) 
    { 
      printf("Error en la apertura de %s con formato\n",nombre);
      printf("Error: %d %s \n",errno,strerror(errno));
      exit(1);
    }

  /* lectura del ancho y alto. Salto de la cabecera. */
  fgets((unsigned char *)buf, 100, ffent);

  /* Saltamos los comentarios PGM, marcados con '#' */
  while ((buf[0]=getc(ffent)) == '#')
    {
       /* se ha leido un comentario... saltamos toda la linea */
       fgets((unsigned char *)buf, 100, ffent);
    }
  /* devolvemos el caracter leido */
  ungetc(buf[0], ffent);

  /* leemos el ancho y el alto */
  fscanf(ffent, "%d %d\n", ancho, alto);

  /* leemos el maximo nivel de gris" */
  fscanf(ffent, "%d", &i);
  /* Lectura del \n que va despues de ese numero */
  c=getc(ffent);
  if (c != '\n')
  {
    printf("\nNO SE HA LEIDO BIEN LA CABECERA\n");
    exit(1);
  }

  /* reserva de la memoria */
  *img1 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));
  *img2 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));
  *img3 = (TIPO *)malloc(*ancho * *alto * sizeof(TIPO));

  /* Lectura de la imagen formato RAW */
  for (j=0; j< *alto; j++  )
  {
    for (i=0; i< *ancho; i++  )
    {
      retonno = fread(&aux, sizeof(unsigned char), 1, ffent);
      if ((retonno != 1) && (ferror(ffent)) && (feof(ffent)))
      {
	printf("Error en la lectura de %s i: %d j: %d, PLANO 1\n",nombre,i,j);
	printf("El contador se quedo en %d\n", contador);
	printf("Error: %d  %s\n",errno,strerror(errno));
	exit(1);
      }
      else
      {
	(*img1)[*ancho * j +i]= aux;
	(*img2)[*ancho * j +i]= aux;
	contador++;
	(*img3)[*ancho * j +i]= aux;
      } /* else del primer fread */
    }  /* for i*/
  } /* for j */
  if (contador != ((*ancho)*(*alto)))
  {
    printf("\nGUARNING: NO SE HA LEIDO EL NUMERO CORRECTO DE BYTES\n");
    printf("Leyendo la imagen %s\n\n", nombre);
    free(*img1);*img1=NULL;
    free(*img2);*img2=NULL;
    free(*img3);*img3=NULL;
  }
  fclose(ffent);
} /* Lee_pgm_TIPO */



/* Paso de RGB a YIQ */
void ToYIQ(int tam_x, int tam_y, TIPO **img, TIPO **y, TIPO **i, TIPO **q)
{
  int j;

  *y = (TIPO *)malloc(tam_y *tam_x * sizeof(TIPO));
  *i = (TIPO *)malloc(tam_y *tam_x * sizeof(TIPO));
  *q = (TIPO *)malloc(tam_y *tam_x * sizeof(TIPO));

  for (j=0; j < (tam_y*tam_x); j++)
  {
    (*y)[j]=(0.299*(img[0][j]))+(0.587*(img[1][j]))+(0.114*(img[2][j]));
    (*i)[j]=(0.596*(img[0][j]))-(0.275*(img[1][j]))-(0.321*(img[2][j]));
    (*q)[j]=(0.212*(img[0][j]))-(0.528*(img[1][j]))+(0.311*(img[2][j]));
  }
} /* ToYIQ */







/******************* Cabecera de Funcion **********************************
*NOMBRE: DeterminarTamano                                                 *
***************************************************************************
*                                                                         *
*DESCRIPCION: Devuelve el tamao potencia de 2 inmediatamente superior *
*            al tamao que tiene la imagen original.                   *
*PARAMETROS:                                                              *
*   x:        Numero de filas de la imagen.                               *
*   y:        Numero de columnas de la imagen                             *
*SALIDA:                                                                  *
*   0 si el tamao de la imagen ya es potencia de 2.                   *
*   El tamao al que quedara la imagen si originalmente no era potencia*
*  de 2; la imagen quedara a tamao nxn, es decir, se hara que tenga el*
*  mismo numero de filas y de columnas.                                   *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se recorre de forma lineal el vector de datos para pasar cada *
*   elemento al nuevo tipo de datos.                                      *
**************************************************************************/
int DeterminarTamano(int x, int y)
{
  TIPO div_x, div_y;
  int int_x, int_y;
  int valex, valey, mayor;


  div_x = (log((TIPO)x) / log(2.0));
  int_x = (int) div_x;
  div_y = (log((TIPO)y) / log(2.0));
  int_y = (int) div_y;

  valex = (div_x == (TIPO)int_x);
  valey = (div_y == (TIPO)int_y);
 
  if (valex && valey)
    return (0);
  else
  {
    mayor = max(int_x,int_y);
    if (mayor == int_x)
    {
      if (valex)
	return(pow(2,mayor));
      else
	return(pow(2,mayor+1));
    }
    else /* if (mayor == int_y)*/
    {
      if (valey)
	return(pow(2,mayor));
      else
	return(pow(2,mayor+1));
    }
  }
} /* DeterminarTamano */



/******************* Cabecera de Funcion **********************************
*NOMBRE: Cambiar_Tamano                                                   *
***************************************************************************
*DESCRIPCION: Convierte una imagen cualquiera en otra cuadrada.           *
*PARAMETROS:                                                              *
*   alto:    numero de filas de la imagen.                                *
*   ancho:   numero de columnas de la imagen.                             *
*   img:     imagen que se quiere convertir.                              *
*   tanano_final: tamao al que debe quedar la imagen.                    *
*   img_salida: imagen al nuevo tamao.                                   *
*SALIDA:     img_salida                                                   *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
void Cambiar_Tamano(int ancho,int alto, TIPO *img,
		    int tamano_final, TIPO **img_salida)
{
  int i, j;
  long int fila, fila_final;
  int min_ancho, min_alto;

  *img_salida = (TIPO *) calloc(tamano_final*tamano_final,
				 sizeof(TIPO));
  if (*img_salida == NULL )
  {
    perror("Cambiar_Tamano");
    exit(1);
  }
  min_alto = min(alto, tamano_final);
  min_ancho= min(ancho, tamano_final);
  for (j = 0; j < min_alto; j++)
  {
    fila = j * ancho;
    fila_final = j * tamano_final;
    for (i = 0; i < min_ancho; i++)
    {
      (*img_salida)[fila_final + i] = img[fila + i];
    }
  }
  
} /* Cambiar_Tamano*/



/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirTres                                                     *
***************************************************************************
*DESCRIPCION: Guarda en fichero los tres planos de color. Esta pensado    *
*           para guardar en fichero la firma. Se guarda en modo binario.  *
*PARAMETROS:                                                              *
*   tres:    puntero a los tres planos que vamos a guardar en fichero.    *
*   tam:     tamao de los planos (los tres tienen el mismo).             *
*   ffsal:   puntero al fichero donde se va a guardar la imagen.          *
*   nombre:  rutal al fichero.                                            *
*SALIDA:                                                                  *
*   0 si la operacion se realiza correctamente.                           *
*   -1 si hubo algun problema.
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
/* Escribir tres vectores en fichero. Lo traslado aqui para no emborronar
en el otro lado
*/
int EscribirTres (Celda **tres, int tam, FILE *ffsal, char *nombre)
{
  int i;
  for (i=0; i<tam; i++)
  {
    if (fwrite(&tres[0][i],sizeof(Celda),1,ffsal) != 1)
    { 
      printf("Error en la escritura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      return(-1);
    }
    if (fwrite(&tres[1][i],sizeof(Celda),1,ffsal) != 1)
    { 
      printf("Error en la escritura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      return(-1);
    }
    if (fwrite(&tres[2][i],sizeof(Celda),1,ffsal) != 1)
    { 
      printf("Error en la escritura de %s\n",nombre);
      printf("Error: %d  %s\n",errno,strerror(errno));
      return(-1);
    }
  }
  return(0);
} /* EscribirTres */

/* Leer tres vectores de fichero. Lo traslado aqui para no emborronar
en el otro lado
*/
int LeerTres (Celda *tres[], int tam, FILE *ffent, char *nombre)
{
  int i;
  
  tres[0] = (Celda *) malloc (tam*sizeof(Celda));
  tres[1] = (Celda *) malloc (tam*sizeof(Celda));
  tres[2] = (Celda *) malloc (tam*sizeof(Celda));

  if ((!tres[0]) || (!tres[1]) || (!tres[2]))
  {
     printf("\nNo se pudo reservar memoria para imgbase\n");
     perror("");
     return(-1);
  }
  else
    for (i=0; i<tam; i++)
    {
      if (fread(&tres[0][i],sizeof(Celda),1,ffent) != 1)
      {
	printf("Error en la lectura de %s\n",nombre);
	printf("Error: %d  %s\n",errno,strerror(errno));
	return(-1);
      }
      if (fread(&tres[1][i],sizeof(Celda),1,ffent) != 1)
      {
	printf("Error en la lectura de %s\n",nombre);
	printf("Error: %d  %s\n",errno,strerror(errno));
	return(-1);
      }
      if (fread(&tres[2][i],sizeof(Celda),1,ffent) != 1)
      {
	printf("Error en la lectura de %s\n",nombre);
	printf("Error: %d  %s\n",errno,strerror(errno));
	return(-1);
      }
    } /* for */
    return(0);
} /* LeerTres */


/* Descomponer */
void DescomponerTres (TIPO **tres,int filas,int col,int e_filas,int e_col)
{
  int f_pasos;

  /*
  int c_pasos;
  */

  TIPO pasos_f;

  /*
   TIPO pasos_c;
  */
 
  pasos_f=(log((TIPO)filas)/log(2.0));
  /*
  pasos_c=(log((TIPO)col)/log(2.0));
  */

  if (e_filas > 0)
  {
    if (e_filas > pasos_f)
    {
      printf("\nERROR: el numero de pasos en que quiere descomponer no es "
	     "valido.\n");
      exit(1);
    }
    else
      f_pasos=e_filas;
  }
  else
    f_pasos=pasos_f;

  /*
  if (e_col > 0)
  {
    if (e_col > pasos_c)
    {
      printf("\nERROR: el numero de pasos en que quiere descomponer no es "
	     "valido.\n");
      exit(1);
    }
    else
      c_pasos=e_col;
  }
  else
    c_pasos=pasos_c;
  
  */

  /* OJO: FILAS=COLUMNAS */
  if (filas != col)
  {
    printf("\nHa intentado descomponer una imagen no cuadrada.\n"
	   "Esto no es posible.\n");
    exit(1);
  }
  NonstandardDecomposition2  (tres[0],filas, f_pasos);
  NonstandardDecomposition2  (tres[1],filas, f_pasos);
  NonstandardDecomposition2  (tres[2],filas, f_pasos);
}




/***************************PARTE DE ENERGIAS *****************************
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularEnergias                                                 *
***************************************************************************
*DESCRIPCION: Calcula las energias en de una imagen bidimensional. Es     *
*           IMPRESCINDIBLE que se haya calculado la transformada en       *
*           onditas hasta el maximo nivel                                 *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el clculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   Vector con las energias. Es un doble puntero en el que para cada      *
*nivel de resolucion se almacena en la posicion 0 E1, en la 1 E2 y en la  *
*2 E3.                                                                    *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO **CalcularEnergias(TIPO *plano, int filas, int columnas)
{
  TIPO **e=NULL,tonta;
  int resolucion,i,j,seguir,x,y,res;
  
  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  e=(TIPO **)calloc(resolucion,sizeof(TIPO *));
  if (e)
  {
    seguir=1;
    for (i=0; (i<resolucion) && (seguir); i++)
    {
      e[i]=(TIPO *)calloc(3,sizeof(TIPO));
      seguir=(e[i] != NULL);
    }
    if (seguir)
    {
      y=filas;
      x=columnas;

      for (res=resolucion-1; res >= 0; res--,x=x/2,y=y/2)
      {
	for (j=x/2; j<x; j++)
	{
	  for(i=0;i<=y/2-1; i++)
	    e[res][0] += plano[columnas*i+j]*plano[columnas*i+j];
	  
	  for(i=y/2; i<y; i++)
	    e[res][1] += plano[columnas*i+j]*plano[columnas*i+j];
	}
	for (j=0; j<=x/2-1; j++)
	  for(i=y/2; i<y; i++)
	    e[res][2] += plano[columnas*i+j]*plano[columnas*i+j];

      } /* for res */
    } /* if seguir */
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "energias\n");
  } /* if (e) */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular energias\n");

  /*
  for (res=0; res<=resolucion-1; res++)
    for (i=0; i<3; i++)
      printf("e[%d][%d]=%f\n",res,i,e[res][i]);
  */

  return(e);
} /* CalcularEnergias */


/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularCovarianzas                                              *
***************************************************************************
*DESCRIPCION: Calcula los coeficientes de covarianza de una imagen        *
*bidimensional. Es IMPRESCINDIBLE que se haya calculado la transformada   *
*en onditas hasta el maximo nivel de resolucion.                          *
*PARAMETROS:                                                              *
*   plano1,plano2,plano3: punteros a los planos de color de la imagen.    *
*   filas:                numero de filas de la imagen (tamano en Y).     *
*   columnas:             numero de columnas de la imagen (tamano en X).  *
*SALIDA:                                                                  *
*   Vector con las covarianzas. Es un triple puntero en el que para cada  *
*nivel de resolucion se almacena, en la posicion que corresponderia a la  *
*energia Ei, los coeficientes de covarianza correspondientes a comparar   * 
*los planos 12, 13 y 23 y en ese orden.                                   *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO ***CalcularCovarianzas(TIPO *plano1, TIPO *plano2, TIPO *plano3,
			    int filas, int columnas)
{
  TIPO ***cov=NULL,tonta;
  int resolucion,i,j,seguir,x,y,res;

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;
  cov=(TIPO ***)calloc(resolucion, sizeof(TIPO **));

  if (cov)
  {
    seguir=1;
    for (i=0; (i<resolucion) && (seguir); i++)
    {
      cov[i]=(TIPO **)calloc(3,sizeof(TIPO*));
      seguir=(cov[i] != NULL);
      for (j=0; (j<3) && (seguir); j++)
      {
	cov[i][j]=(TIPO *)calloc(3,sizeof(TIPO));
	seguir=(cov[i][j] != NULL);
      }
    } /* for i */
    if (seguir)
    {
      y=filas;
      x=columnas;

      for (res=resolucion-1; res >= 0; res--,x=x/2,y=y/2)
      {
	for (j=x/2; j<x; j++)
	{
	  for(i=0;i<=y/2-1; i++)
	  {
	     /* Planos 1 y 2 */
	    cov[res][0][0] += plano1[columnas*i+j]*plano2[columnas*i+j];

	    /* Planos 1 y 3 */
	    cov[res][0][1] += plano1[columnas*i+j]*plano3[columnas*i+j];

	    /* Planos 2 y 3 */
	    cov[res][0][2] += plano2[columnas*i+j]*plano3[columnas*i+j];
	  }

	  for(i=y/2; i<y; i++)
	  {
	    /* Planos 1 y 2 */
	    cov[res][1][0] += plano1[columnas*i+j]*plano2[columnas*i+j];

	    /* Planos 1 y 3 */
	    cov[res][1][1] += plano1[columnas*i+j]*plano3[columnas*i+j];

	    /* Planos 2 y 3 */
	    cov[res][1][2] += plano2[columnas*i+j]*plano3[columnas*i+j];
	  }       
	} /* for j=x/2 */

	for (j=0; j<=x/2-1; j++)
	  for(i=y/2; i<y; i++)
	  {
	    /* Planos 1 y 2 */
	    cov[res][2][0] += plano1[columnas*i+j]*plano2[columnas*i+j];

	    /* Planos 1 y 3 */
	    cov[res][2][1] += plano1[columnas*i+j]*plano3[columnas*i+j];

	    /* Planos 2 y 3 */
	    cov[res][2][2] += plano2[columnas*i+j]*plano3[columnas*i+j];
	  }
      } /* for res */
    } /* if seguir */
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "covarianzas\n");
  } /* if (cov) */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "covarianzas\n");

  /*
  for (res=0; res<=resolucion-1; res++)
    for (i=0; i<3; i++)
      for (j=0; j<3; j++)
	printf("cov[%d][%d][%d]=%f\n",res,i,j,cov[res][i][j]);
  */
  
  return(cov);
} /* CalcularCovarianzas */


/* 11, 12, 13, 22, 23, 33 */
TIPO ***NormalizarCov(TIPO **e1, TIPO **e2, TIPO **e3,
                       TIPO ****cov, int filas, int columnas)
{
  int i,j,seguir,tamano;
  TIPO ***c=*cov, ***corr=NULL,tonta;

  tonta=(log((TIPO)filas)/log(2.0));
  tamano=tonta;
  corr=(TIPO ***)calloc(tamano, sizeof(TIPO **));
  if (corr)
  {
    seguir=1;
    for (i=0; (i<tamano) && (seguir); i++)
    {
      corr[i]=(TIPO **)calloc(3, sizeof(TIPO *));
      seguir = (corr[i] != NULL);
      for (j=0; (j<3) && (seguir); j++)
      {
	corr[i][j]=(TIPO *)calloc(6, sizeof(TIPO));
	seguir = (corr[i][j] != NULL);
	if (seguir)
	{
	  corr[i][j][0]=e1[i][j];                        /* Plano 1*/

	  if ((e1[i][j]==0) || (e2[i][j]==0))
	    corr[i][j][1]=0;
	  else
	    corr[i][j][1]=c[i][j][0]/(e1[i][j]*e2[i][j]);  /* Planos 1 y 2 */
	  
	  if ((e1[i][j]==0) || (e3[i][j]==0))
	    corr[i][j][2]=0;
	  else
	    corr[i][j][2]=c[i][j][1]/(e1[i][j]*e3[i][j]);  /* Planos 1 y 3 */

	  corr[i][j][3]=e2[i][j];                        /* Plano 2 */

	  if ((e2[i][j]==0) || (e3[i][j]==0))
	    corr[i][j][4]=0;
	  else
	    corr[i][j][4]=c[i][j][2]/(e2[i][j]*e3[i][j]);  /* Planos 2 y 3 */
	  
	  corr[i][j][5]=e3[i][j];                        /* Plano 3 */
	} /* if seguir */
      } /* for j */
    } /* for i */
    if (seguir)
    {
      for (i=0; i<tamano; i++)
      {
	for (j=0; j<3; j++)
	{
	  free(c[i][j]);
	  c[i][j]=NULL;
	}
	free(c[i]);
	c[i]=NULL;
      }
      free(c);
      c=NULL;
    }
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "correlaciones\n");
  } /* if corr */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "correlaciones\n");
  
  /*
  for (res=0; res<=tamano-1; res++)
    for (i=0; i<3; i++)
      for (j=0; j<6; j++)
	printf("corr[%d][%d][%d]=%f\n",res,i,j,corr[res][i][j]);
  */


  return(corr);
} /* NormalizarCov */


int EscribirCorrelaciones (TIPO ***corr, int filas, int columnas, 
			   char *nombre)
{
  int i, j, k, tamano, retonno=0, items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano = tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano) &&(!retonno); i++)
    {
      for (j=0; (j<3) && (!retonno); j++)
      {
	for (k=0; (k<6) && (!retonno); k++)
        {
	  items=fwrite(&corr[i][j][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    retonno=tamano-1;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirCorrelaciones */

TIPO ***LeerCorrelaciones (char *nombre, int tamano)
{
  int i, j, k, items, seguir;
  FILE *fent;
  TIPO ***corr;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    corr=(TIPO ***)calloc(tamano, sizeof(TIPO **));
    seguir=(corr != NULL);
    for (i=0; (i<tamano) &&(seguir); i++)
    {
      corr[i]=(TIPO **)calloc(3, sizeof(TIPO *));
      seguir = (corr[i] != NULL);
      for (j=0; (j<3) && (seguir); j++)
      {
	corr[i][j]=(TIPO *)calloc(6, sizeof(TIPO));
	seguir = (corr[i][j] != NULL);
	for (k=0; (k<6) && (seguir); k++)
	{
	  items=fread(&corr[i][j][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para las"
	     "correlaciones.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(corr);
  } /* else */
} /* LeerCorrelaciones */

void Liberar (TIPO ****correlaciones, int tamano)
{
  TIPO ***corr=*correlaciones;
  int i, j;

  for (i=0; i<tamano; i++)
  {
    for (j=0; j<3; j++)
    {
      free(corr[i][j]);
      corr[i][j]=NULL;
    }
    free(corr[i]);
    corr[i]=NULL;
  }
  free(corr);
  corr=NULL;
}


/*******************PARTE DE UMBRALIZACION MAS MOMENTOS *******************
**************************************************************************/

/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularEnergiaTotal                                             *
***************************************************************************
*DESCRIPCION: Calcula las energia de una imagen bidimensional.            *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   energ:    variable donde se deja la energia calculada.                *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

void CalcularEnergiaTotal(TIPO *plano, int filas, int columnas, 
			  TIPO *energ)
{
  int i,j;
  
  *energ=0;
  for (i=0; i<filas; i++)
    for (j=0; j<columnas; j++)
      *energ += plano[columnas*i+j]*plano[columnas*i+j];

  /* 
  printf("\nEnergia total: %f\n", *energ);
  */
} /* CalcularEnergiaTotal */


/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularE_Umbral                                                 *
***************************************************************************
*DESCRIPCION: Calcula la energia media de un nivel de resolucion,         *
*   que se determina en base a los rangos en los que nos moveremos.       *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   rango_y:  limite del nivel en Y.                                      *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_Umbral(TIPO *plano,int rango_x, int rango_y, 
			int columnas, int tipo_umbral, TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,tmp;
  int i,j,num_pixels=0;

  for (j=rango_x/2; j<rango_x; j++)
  {
    /* SOLO ES NECESARIO EL CUADRANTE DIAGONAL
    for(i=0;i<=rango_y/2-1; i++)
    {
      e_media+=(plano[columnas*i+j]*plano[columnas*i+j]);
      num_pixels++;
    }
    */

    for(i=rango_y/2; i<rango_y; i++)
    {
      e_media+=(plano[columnas*i+j]*plano[columnas*i+j]);
      num_pixels++;
    }
  }

  /* SOLO ES NECESARIO EL CUADRANTE DIAGONAL 
  for (j=0; j<=rango_x/2-1; j++)
    for(i=rango_y/2; i<rango_y; i++)
    {
      e_media+=(plano[columnas*i+j]*plano[columnas*i+j]);
      num_pixels++;
    }
  */


  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (j=rango_x/2; j<rango_x; j++)
    {
      for(i=rango_y/2; i<rango_y; i++)
      {
        tmp = plano[columnas*i+j] * plano[columnas*i+j];
        tmp = tmp-e_media;
        varianza += tmp * tmp;
        /*
	varianza+=pow(pow(plano[columnas*i+j],2)-e_media,2);
        */
      }
    } /* for j */

    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    valor = e_media + (varianza*factor);
    return(valor);

  } /* else */
} /* CalcularE_Umbral */


/******************* Cabecera de Funcion **********************************
*NOMBRE: Escribe_pgm                                                      *
***************************************************************************
*                                                                         *
*DESCRIPCION: Lee una imagen tipo float y la almacena en un vector float  *
*PARAMETROS:                                                              *
*   alto:     Numero de filas de la imagen.                               *
*   ancho:    Numero de columnas de la imagen.                            *
*   nombre:   Ruta al fichero donde se encuentra la imagen a leer.        *
*   img1:     En cada img se almacena uno de los planos de color          *
*             de la imagen.                                               *
*SALIDA:                                                                  *
*    img1, img2, img3: se obtiene cada plano de color por separado, para  *
*        su tratamiento individual.                                       *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se recorre de forma lineal el fichero origen y se almacena en *
*   el vector, teniendo en cuenta que es un vector unidimensional.        *
**************************************************************************/
void Escribe_pgm (int alto, int ancho, char *nombre, TIPO *img, int conv)
{
  int i,j,contador=0;
  int retonno;
  FILE *fsal;
  unsigned char aux;

  /* Abrimos el fichero con entrada formateada */
  fsal = fopen (nombre, "wb");
  if (fsal == NULL) 
  { 
    printf("Error en la apertura de %s con formato\n",nombre);
    printf("Error: %d %s \n",errno,strerror(errno));
    exit(1);
  }
  
  /* Escritura de cabecera, ancho y alto. */

  fprintf(fsal, "P5\n%d %d\n255\n", ancho, alto);

  /* Escritura de la imagen formato RAW */
  for (j=0; j< alto; j++  )
  {
    for (i=0; i< ancho; i++  )
    {


      /* OJO: CONVERSION */
      if (conv)
	aux=(127.0 + img[ancho*j+i] / 2.0); 
      else
	aux=img[ancho*j+i];

      retonno = fwrite(&aux, sizeof(unsigned char), 1, fsal);
      if (retonno != 1)
      {
	printf("Error en la escritura de %s i: %d j: %d\n",nombre,i,j);
	printf("Error: %d  %s\n",errno,strerror(errno));
	exit(1);
      }

      contador++;
    }
  }
  /*
  printf("\nESCRITURA\ncontador= %d\ti=%d\tj=%d\n", contador,i,j);
  */
  fclose(fsal);
} /* Escribe_pgm */



/************************* Cabecera de Funcion ****************************
*NOMBRE: Umbralizar                                                       *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

void Umbralizar (TIPO **plano, int filas, int columnas, int tipo_umbral)
{
  int i,j,x,y,resolucion,res;
  TIPO *plano_aux=*plano, e_umbral, tonta;

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  y=filas;
  x=columnas;


  /* Para depurar  */
  /*
  for (i=0; i<filas; i++)
    for(j=0; j<filas; j++)
	e_umbral+=plano_aux[columnas*i+j]*plano_aux[columnas*i+j];

  printf("Energia total: %.12f\n", e_umbral);
  exit(0);
  */

  for (res=resolucion-1; res >= 0; res--,x=x/2,y=y/2)
  {
    e_umbral=CalcularE_Umbral(plano_aux,x,y,columnas,tipo_umbral,1.0);
    /*
    printf("\nEnergia umbral: %.12f\n", e_umbral);
    */
    for (j=x/2; j<x; j++)
    {
      for(i=0;i<=y/2-1; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j]) >= e_umbral);
      
      for(i=y/2; i<y; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j])>= e_umbral);
    }
    for (j=0; j<=x/2-1; j++)
      for(i=y/2; i<y; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j]) >= e_umbral);
  
  } /* for res */


  /*
  for (i=0; i<filas*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
  */


} /* Umbralizar */


/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarUser                                                   *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   umbral:   umbral que se exige.                                        *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

void UmbralizarUser (TIPO **plano, int filas, int columnas, TIPO factor,
		     int tipo_umbral)
{
  int i,j,x,y,resolucion,res;
  TIPO *plano_aux=*plano, tonta, e_umbral;

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  y=filas;
  x=columnas;

  for (res=resolucion; res >= 0; res--,x=x/2,y=y/2)
  {
    e_umbral=CalcularE_Umbral(plano_aux,x,y,columnas,tipo_umbral,factor);
    for (j=x/2; j<x; j++)
    {
      for(i=0;i<=y/2-1; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j]) >= e_umbral);
      
      for(i=y/2; i<y; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j])>= e_umbral);
    }
    for (j=0; j<=x/2-1; j++)
      for(i=y/2; i<y; i++)
	plano_aux[columnas*i+j]=((plano_aux[columnas*i+j]*
				  plano_aux[columnas*i+j]) >= e_umbral);
  
  } /* for res */

  /*
  for (i=0; i<filas*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
  */
  
} /* UmbralizarUser */








void CalcularMu2 (TIPO *plano, int tamano, int x, int y, int cuadrante, 
		 TIPO *mu11, TIPO *mu20, TIPO *mu02)
{
   TIPO mom10=0, mom00=0, mom01=0, mu00=0, central_x,central_y;
   int inf_x, inf_y, sup_x, sup_y, i,j;
   int inf_x_m, inf_y_m, sup_x_m, sup_y_m;

   TIPO mu01=0, mu10=0;
   /* 
     Correcciones para cambiar de dominio a (-1,1) Se sigue una fo'rmula del
     tipo (x-resta)/denominador 
   */
   TIPO x_resta,x_denominador,y_resta,y_denominador;
   TIPO x_resta_m, x_deno_m, y_resta_m, y_deno_m;
   TIPO tmp;

   /*
      Esto es para pasar al dominio [-1,1] y seleccionar los limites del
    bucle
   */
   if (cuadrante == V)
   {
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)((x-x/2.0)/2.0);
     y_resta=(TIPO)((y/2.0-1.0)/2.0);
     y_denominador=(TIPO)((y/2.0-1.0)/2.0);
    
     inf_x=x/2;
     sup_x=x;
     inf_y=0;
     sup_y=y/2;
   }
   else if (cuadrante == D)
   {
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)((x-x/2.0)/2.0);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)((y-y/2.0)/2.0);

     inf_x=x/2;
     sup_x=x;
     inf_y=y/2;
     sup_y=y;
   }
   else if (cuadrante == H)
   {
     x_resta=(TIPO)((x/2.0-1.0)/2.0);
     x_denominador=(TIPO)((x/2.0-1.0)/2.0);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)((y-y/2.0)/2.0);

     inf_x=0;
     sup_x=x/2;
     inf_y=y/2;
     sup_y=y;
   }
   else if (cuadrante == U)
   {
     /* Primero el cuadrante vertical */
     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)((x+2.0*x)/2.0);
     x_denominador=(TIPO)((2.0*x-x)/2.0);
     y_resta=(TIPO)((y-1.0)/2.0);
     y_denominador=(TIPO)((y-1.0)/2.0);

     inf_x=x;
     sup_x=2*x;
     inf_y=0;
     sup_y=y;

     /* Despus el cuadrante horizontalal */
     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta_m=(TIPO)((x-1.0)/2.0);
     x_deno_m=(TIPO)((x-1.0)/2.0);
     y_resta_m=(TIPO)((y+2.0*y)/2.0);
     y_deno_m=(TIPO)((2.0*y-y)/2.0);

     inf_x_m=0;
     sup_x_m=x;
     inf_y_m=y;
     sup_y_m=2*y;
   } /* else if (cuadrante == U) */

   else  /* if (cuadrante == A) || (cuadrante == M) */
   {
     x_resta=(TIPO)(x/2.0);
     x_denominador=(TIPO)(x/2.0);
     y_resta=(TIPO)(y/2.0);
     y_denominador=(TIPO)(y/2.0);

     inf_x=0;
     sup_x=x;
     inf_y=0;
     sup_y=y;

     if (cuadrante == M)
     {
       x_resta_m=(TIPO)((x+2*x)/2.0);
       x_deno_m=(TIPO)((2*x-x)/2.0);
       y_resta_m=(TIPO)((y+2*y)/2.0);
       y_deno_m=(TIPO)((2*y-y)/2.0);

       inf_x_m=x;
       sup_x_m=2*x;
       inf_y_m=y;
       sup_y_m=2*y;
     }
   }

     
   /* Primero se calculan los momentos centrales central_x y central_y */
   for (j=inf_x; j<sup_x; j++)
   {
     for(i=inf_y;i<sup_y; i++)
     {
       mom00 += fabs(plano[tamano*i+j]);
       mom10 += (fabs(plano[tamano*i+j])*(j-x_resta)/x_denominador);
       mom01 += (fabs(plano[tamano*i+j])*(i-y_resta)/y_denominador);
       
     }
   }
   if ((cuadrante == M) || (cuadrante == U))
   {
     for (j=inf_x_m; j<sup_x_m; j++)
     {
       for(i=inf_y_m;i<sup_y_m; i++)
       {
	 mom00 += fabs(plano[tamano*i+j]);
	 mom10 += (fabs(plano[tamano*i+j])*(j-x_resta_m)/x_deno_m);
	 mom01 += (fabs(plano[tamano*i+j])*(i-y_resta_m)/y_deno_m);	 
       }
     }
   } /* if cuadrante == M */
	  
   if (mom00 == 0)
   {
     central_x=0;
     central_y=0;
   }
   else
   {
     central_x=mom10/mom00;
     central_y=mom01/mom00;
   }

   for (j=inf_x; j<sup_x; j++)
   {
     for(i=inf_y;i<sup_y; i++)
     {
 
       *mu11 += ((((j-x_resta)/x_denominador)-central_x)*
		 (((i-y_resta)/y_denominador)-central_y)*
		 fabs(plano[tamano*i+j]));
       
       *mu20 += ((((j-x_resta)/x_denominador)-central_x)*
		 (((j-x_resta)/x_denominador)-central_x)*
		 fabs(plano[tamano*i+j]));
       
       *mu02 += ((((i-y_resta)/y_denominador)-central_y)*
		 (((i-y_resta)/y_denominador)-central_y)*
		 fabs(plano[tamano*i+j]));
       
       mu10 += (((j-x_resta)/x_denominador-central_x)*fabs(plano[tamano*i+j]));
       mu01 += (((i-y_resta)/y_denominador-central_y)*
		 fabs(plano[tamano*i+j]));
     }
   }

   if ((cuadrante == M) || (cuadrante == U))
   {
     for (j=inf_x_m; j<sup_x_m; j++)
     {
       for(i=inf_y_m;i<sup_y_m; i++)
       {
 
	 *mu11 += ((((j-x_resta_m)/x_deno_m)-central_x)*
		   (((i-y_resta_m)/y_deno_m)-central_y)*
		   fabs(plano[tamano*i+j]));
       
	 *mu20 += ((((j-x_resta_m)/x_deno_m)-central_x)*
		   (((j-x_resta_m)/x_deno_m)-central_x)*
		   fabs(plano[tamano*i+j]));
       
	 *mu02 += ((((i-y_resta_m)/y_deno_m)-central_y)*
		   (((i-y_resta_m)/y_deno_m)-central_y)*
		   fabs(plano[tamano*i+j]));
       
	 mu10 += (((j-x_resta_m)/x_deno_m-central_x)*fabs(plano[tamano*i+j]));
	 mu01 += (((i-y_resta_m)/y_deno_m-central_y)*
		  fabs(plano[tamano*i+j]));
       }
     }
   } /* if cuadrante == m */
      
   mu00=mom00;
   /* NORMALIZACION DE LOS MU */
   if (mu00)
   {
     tmp = (mu00*mu00);
     *mu11=*mu11/tmp;
     *mu20=*mu20/tmp;
     *mu02=*mu02/tmp;
     
     /*
     *mu11=*mu11/pow(mu00,(1+1+2)/2);
     *mu20=*mu20/pow(mu00,(2+0+2)/2);
     *mu02=*mu02/pow(mu00,(0+2+2)/2);
     */

   }
   else
   {
     *mu11=0;
     *mu20=0;
     *mu02=0;
   }
   
   /*
   printf("mom10=%.12f\tmom01=%.12f\n", mom10,mom01);
   printf("mu00=%.12f\tmu10=%.12f\tmu01=%.12f\n", mu00,mu10,mu01);
   printf("mu20=%.12f\tmu02=%.12f\n", *mu20, *mu02);
   */

} /* CalcularMu2 */


void LiberarMomentos (TIPO ****momentos, int tamano)
{
  TIPO ***m;
  int i, j,k;

  for (k=0; k<3; k++)
  {
    m=momentos[k];
    for (i=0; i<tamano-1; i++)
    {
      for (j=0; j<3; j++)
      {
        free(m[i][j]);
        m[i][j]=NULL;
      }
      free(m[i]);
      m[i]=NULL;
    }
    free(m);
    m=NULL;
  } /* for k */
} /* LiberarMomentos */



/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentos                                                 *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentos (TIPO ***m, int filas, int columnas, char *nombre)
{
  int i, k, tamano, retonno=0, items,num_mom;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (num_mom=0; num_mom<3; num_mom++)
    {
      for (i=0; (i<tamano-1) &&(!retonno); i++)
      {
	for (k=0; (k<2) && (!retonno); k++)
        {
	  /*
	  printf("m[%d][%d][%d]=%.15f\n",num_mom,i,k,m[num_mom][i][k]);
	  */

	  items=fwrite(&m[num_mom][i][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for i */
    } /*for num_mom */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentos */

TIPO ***LeerMomentos (char *nombre, int tamano)
{
  int i, k, items, seguir,num_mom;
  FILE *fent;
  TIPO ***m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ***)calloc(3,sizeof(TIPO **));
    seguir=(m != NULL);
    if (seguir)
    {
      for (num_mom=0; num_mom<3; num_mom++)
      {
	m[num_mom]=(TIPO **)calloc(tamano-1, sizeof(TIPO *));
	seguir=(m[num_mom] != NULL);
	for (i=0; (i<tamano-1) &&(seguir); i++)
        {
	  m[num_mom][i]=(TIPO *)calloc(2, sizeof(TIPO));
	  seguir = (m[num_mom][i] != NULL);
	  for (k=0; (k<2) && (seguir); k++)
	  {
	    items=fread(&m[num_mom][i][k], sizeof(TIPO), 1, fent);
	    if (items != 1)
	    {
	      printf("\nError de lectura en %s\n", nombre);
	      perror(" ");
	      return(NULL);
	    }
	  } /* for k */
	} /* for i */
      } /* for num_mom */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
    {
      /*
      for (num_mom=0; num_mom<3; num_mom++)
	for (i=0; (i<tamano-1) &&(seguir); i++)
	  for (k=0; (k<2) && (seguir); k++)
            printf("m[%d][%d][%d]=%.12f\n", num_mom,i,k,m[num_mom][i][k]);
      */

      return(m);
    }
  } /* else */
} /* LeerMomentos */


/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMomentosTres                                             *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 2 de imagen bidimensional a   *
*   que se ha aplicado la transformada en onditas y se ha umbralizado.    *
*   Se calculan sobre todos los coeficientes de detalle del nivel de      *
*   resolucion a la vez.                                                  *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   Vector con los momentos. Es un doble  puntero en el que para cada     *
* nivel de resolucion se almacenan los momentos (los dos momentos de      *
* orden 2: fi1 y fi2, y los cinco que vienen en el Hain para orden 3).    *
* Ademas se transforman las coordenadas de [0,511] al dominio [-1,1].     *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO **CalcularMomentosTres(TIPO *plano, int filas, int columnas)
{
  TIPO **m=NULL,mu20,mu02,mu11,mom00,mom10,mom01,mu30,mu03,mu12,mu21,
        central_x,central_y;
  int resolucion,i,j,seguir,x,y,res;
  TIPO tonta, tmp;
                           
  /* 
     Correcciones para cambiar de dominio a [-1,1] o el que se desee.  
  Se sigue una fo'rmula del tipo (x-resta)/denominador */
  TIPO x_resta_alto,x_denominador_alto,y_resta_alto,y_denominador_alto,
         x_resta_diagonal,x_denominador_diagonal,y_resta_diagonal,
         y_denominador_diagonal,x_resta_bajo,x_denominador_bajo,
         y_resta_bajo,y_denominador_bajo;

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

  */

  /* 
     Como tratamos a la vez los tres cuadrantes, en resolucion 0 habra 3
     coeficientes, asi que si tendra sentido calcular sus momentos
  */
  m=(TIPO **)calloc(resolucion,sizeof(TIPO *));
  if (m)
  {
    seguir=1;
    for (i=0; (i<resolucion) && (seguir); i++)
    {
      /* Se consideran los 7 momentos para todo el nivel de resolucion */
      m[i]=(TIPO *)calloc(7,sizeof(TIPO));
      seguir=(m[i] != NULL);
    }

    /* Se ha reservado la memoria. Ahora vienen los calculos */

    if (seguir)
    {
      y=filas;
      x=columnas;

      for (res=resolucion-1; res >= 0; res--,x=x/2,y=y/2)
      {
	mu20=0;mu02=0;
	mu11=0;
	mu12=0;mu21=0;
	mu03=0;mu30=0;
	mom10=0;
	mom00=0;
	mom01=0;

	/* Esto es para pasar al dominio [-1,1]: cada cuadrante por
	 separado pasa al [-1,1]*/
	/* ######## ANULADO
	x_resta_alto=(TIPO)((x/2.0+x)/2.0);
	x_denominador_alto=(TIPO)((x-x/2.0)/2.0);
	y_resta_alto=(TIPO)((y/2.0-1.0)/2.0);
	y_denominador_alto=(TIPO)((y/2.0-1.0)/2.0);

	x_resta_diagonal=(TIPO)((x/2.0+x)/2.0);
	x_denominador_diagonal=(TIPO)((x-x/2.0)/2.0);
	y_resta_diagonal=(TIPO)((y/2.0+y)/2.0);
	y_denominador_diagonal=(TIPO)((y-y/2.0)/2.0);
	
	x_resta_bajo=(TIPO)((x/2.0-1.0)/2.0);
	x_denominador_bajo=(TIPO)((x/2.0-1.0)/2.0);
	y_resta_bajo=(TIPO)((y/2.0+y)/2.0);
	y_denominador_bajo=(TIPO)((y-y/2.0)/2.0);
	######## FIN ANULADO*/

	/* TODOS LOS PIXELES PASAN AL DOMINIO [-1,1]. EN CONSECUENCIA,
	   este cuadrante pasa al [0,1] en columnas y [-1,0] en filas */

        /* 
	   Como tratamos a la vez los tres cuadrantes, en resolucion 0 habra 3
	   coeficientes, asi que si tendra sentido calcular sus momentos
	*/
	if (res == 0)
	{
	  x_resta_alto=0;
	  x_denominador_alto=1;
	  y_resta_alto=0;
	  y_denominador_alto=1;
	  x_resta_diagonal=0;
	  x_denominador_diagonal=1;
	  y_resta_diagonal=0;
	  y_denominador_diagonal=1;
	  x_resta_bajo=0;
	  x_denominador_bajo=1;
	  y_resta_bajo=0;
	  y_denominador_bajo=1;
	}
	else
	{
	  x_resta_alto=(TIPO)(x/2.0);
	  x_denominador_alto=(TIPO)((x-1.0)-(x/2.0));
	  y_resta_alto=(TIPO)(y/2.0 - 1.0);
	  y_denominador_alto=(TIPO)(y/2.0 - 1.0);
	  
	  /* TODOS LOS PIXELES PASAN AL DOMINIO [-1,1]. EN CONSECUENCIA,
	     este cuadrante pasa al [0,1] en columnas y [0,1] en filas */
	  x_resta_diagonal=(TIPO)(x/2.0);
	  x_denominador_diagonal=(TIPO)((x-1.0)-(x/2.0));
	  y_resta_diagonal=(TIPO)(y/2.0);
	  y_denominador_diagonal=(TIPO)((y-1.0)-(y/2.0));
	  
	  /* TODOS LOS PIXELES PASAN AL DOMINIO [-1,1]. EN CONSECUENCIA,
	     este cuadrante pasa al [-1,0] en columnas y [0,1] en filas */
	  x_resta_bajo=(TIPO)(x/2.0 - 1.0);
	  x_denominador_bajo=(TIPO)(x/2.0 - 1.0);
	  y_resta_bajo=(TIPO)(y/2.0);
	  y_denominador_bajo=(TIPO)((y-1.0)-(y/2.0));
	}

	/* Primero se calculan los momentos centrales central_x y central_y */
	/* Cuadrante paso alto */
	for (j=x/2; j<x; j++)
	{
	  for(i=0;i<=y/2-1; i++)
	  {
	    mom00 += plano[columnas*i+j];
	    mom10 += (plano[columnas*i+j]*((j-x_resta_alto)/
					   x_denominador_alto));
	    mom01 += (plano[columnas*i+j]*((i-y_resta_alto)/
					   y_denominador_alto));
	  }
	}

	/* Cuadrante diagonal */
	for(j=x/2; j<x; j++)
	{
	  for(i=y/2; i<y; i++)
	  {
	    mom00 += plano[columnas*i+j];
	    mom10 += (plano[columnas*i+j]*((j-x_resta_diagonal)/
					   x_denominador_diagonal));
	    mom01 += (plano[columnas*i+j]*((i-y_resta_diagonal)/
					   y_denominador_diagonal));
	  }
	}

	/* Cuadrante paso bajo */
	for (j=0; j<=x/2-1; j++)
	{
	  for(i=y/2; i<y; i++)
	  {
	    mom00 += plano[columnas*i+j];
	    mom10 += (plano[columnas*i+j]*((j-x_resta_bajo)/
					   x_denominador_bajo));
	    mom01 += (plano[columnas*i+j]*((i-y_resta_bajo)/
					   y_denominador_bajo));
	  }
	}

	if (mom00 == 0)
	{
	  central_x=0;
	  central_y=0;
	}
	else
        {
	  central_x=mom10/mom00;
	  central_y=mom01/mom00;
	}
	/*
	printf("\nmom00=%.12f\tmom10=%.12f\tmom01=%.12f\n",mom00,mom10,mom01);
	printf("media_x=%.12f\tmedia_y=%.12f\n",central_x,central_y);
	*/

	/* Ahora calculamos los mu */

	/* Cuadrante paso alto */
	for (j=x/2; j<x; j++)
	{
	  for(i=0;i<=y/2-1; i++)
	  {
	    mu11 += ((((j-x_resta_alto)/x_denominador_alto)-central_x)*
		    (((i-y_resta_alto)/y_denominador_alto)-central_y)*
	            plano[columnas*i+j]);
	    
	    mu20 += ((((j-x_resta_alto)/x_denominador_alto)-central_x)*
		     (((j-x_resta_alto)/x_denominador_alto)-central_x)*
		     plano[columnas*i+j]);
	    
	    mu02 += ((((i-y_resta_alto)/y_denominador_alto)-central_y)*
		     (((i-y_resta_alto)/y_denominador_alto)-central_y)*
		     plano[columnas*i+j]);
	   
            tmp = ((j-x_resta_alto)/x_denominador_alto)-central_x;
	    mu21 += ((tmp * tmp) * 
		     (((i-y_resta_alto)/y_denominador_alto)-central_y)*
		     plano[columnas*i+j]);
            
	    /*
	    mu21 += (pow(((j-x_resta_alto)/x_denominador_alto)-central_x,2)*
		     (((i-y_resta_alto)/y_denominador_alto)-central_y)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_alto)/y_denominador_alto)-central_y;
	    mu12 += ((((j-x_resta_alto)/x_denominador_alto)-central_x)*
		     (tmp * tmp) *
		     plano[columnas*i+j]);
	    /*
	    mu12 += ((((j-x_resta_alto)/x_denominador_alto)-central_x)*
		     pow(((i-y_resta_alto)/y_denominador_alto)-central_y,2)*
		     plano[columnas*i+j]);
	    */
	    
	    tmp = ((j-x_resta_alto)/x_denominador_alto)-central_x;
	    mu30 += (tmp * tmp * tmp) * plano[columnas*i+j];
	    
	    /*
	    mu30 += (pow(((j-x_resta_alto)/x_denominador_alto)-central_x,3)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_alto)/y_denominador_alto)-central_y;
	    mu03 += (tmp * tmp * tmp) * plano[columnas*i+j];
	    
	    /*
	    mu03 += (pow(((i-y_resta_alto)/y_denominador_alto)-central_y,3)*
		     plano[columnas*i+j]);
	    */
	  }
	}
	
	/* Cuadrante diagonal */
	for(j=x/2; j<x; j++)
	{
	  for(i=y/2; i<y; i++)
	  {
	    mu11 += ((((j-x_resta_diagonal)/x_denominador_diagonal)-central_x)*
		    (((i-y_resta_diagonal)/y_denominador_diagonal)-central_y)*
	            plano[columnas*i+j]);
	    
	    mu20 += ((((j-x_resta_diagonal)/x_denominador_diagonal)-central_x)*
		     (((j-x_resta_diagonal)/x_denominador_diagonal)-central_x)*
		     plano[columnas*i+j]);
	    
	    mu02 += ((((i-y_resta_diagonal)/y_denominador_diagonal)-central_y)*
		     (((i-y_resta_diagonal)/y_denominador_diagonal)-central_y)*
		     plano[columnas*i+j]);

	    tmp = ((j-x_resta_diagonal)/x_denominador_diagonal)-central_x;
	    mu21 += ((tmp * tmp) * 
		     (((i-y_resta_diagonal)/y_denominador_diagonal)-central_y)*
		     plano[columnas*i+j]);

	    /*
	    mu21 += (pow(((j-x_resta_diagonal)/x_denominador_diagonal)-
			 central_x,2)*
		     (((i-y_resta_diagonal)/y_denominador_diagonal)-central_y)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_diagonal)/y_denominador_diagonal)-central_y;
	    mu12 += ((((j-x_resta_diagonal)/x_denominador_diagonal)-central_x)*
		     (tmp * tmp) * plano[columnas*i+j]);

	    /*
	    mu12 += ((((j-x_resta_diagonal)/x_denominador_diagonal)-central_x)*
		     pow(((i-y_resta_diagonal)/y_denominador_diagonal)-
			 central_y,2)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((j-x_resta_diagonal)/x_denominador_diagonal)-central_x;
	    mu03 += (tmp * tmp * tmp) * plano[columnas*i+j];

	    /*
	    mu30 += (pow(((j-x_resta_diagonal)/x_denominador_diagonal)-
			 central_x,3)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_diagonal)/y_denominador_diagonal)-central_y;
	    mu03 += (tmp * tmp * tmp) * plano[columnas*i+j];
	    
	    /*
	    mu03 += (pow(((i-y_resta_diagonal)/y_denominador_diagonal)-
			 central_y,3)*
		     plano[columnas*i+j]);
	    */
	  }
	}

	/* Cuadrante paso bajo */
	for (j=0; j<=x/2-1; j++)
	{
	  for(i=y/2; i<y; i++)
	  {
	    mu11 += ((((j-x_resta_bajo)/x_denominador_bajo)-central_x)*
		    (((i-y_resta_bajo)/y_denominador_bajo)-central_y)*
	            plano[columnas*i+j]);
	    
	    mu20 += ((((j-x_resta_bajo)/x_denominador_bajo)-central_x)*
		     (((j-x_resta_bajo)/x_denominador_bajo)-central_x)*
		     plano[columnas*i+j]);
	    
	    mu02 += ((((i-y_resta_bajo)/y_denominador_bajo)-central_y)*
		     (((i-y_resta_bajo)/y_denominador_bajo)-central_y)*
		     plano[columnas*i+j]);

	    tmp = ((j-x_resta_bajo)/x_denominador_bajo)-central_x;
	    mu21 += ((tmp * tmp) *
		     (((i-y_resta_bajo)/y_denominador_bajo)-central_y)*
		     plano[columnas*i+j]);

	    /*
	    mu21 += (pow(((j-x_resta_bajo)/x_denominador_bajo)-central_x,2)*
		     (((i-y_resta_bajo)/y_denominador_bajo)-central_y)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_bajo)/y_denominador_bajo)-central_y;
	    mu12 += ((((j-x_resta_bajo)/x_denominador_bajo)-central_x)*
		     (tmp * tmp) * plano[columnas*i+j]);

	    /*
	    mu12 += ((((j-x_resta_bajo)/x_denominador_bajo)-central_x)*
		     pow(((i-y_resta_bajo)/y_denominador_bajo)-central_y,2)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((j-x_resta_bajo)/x_denominador_bajo)-central_x;
	    mu30 += (tmp * tmp * tmp) * plano[columnas*i+j];

	    /*
	    mu30 += (pow(((j-x_resta_bajo)/x_denominador_bajo)-central_x,3)*
		     plano[columnas*i+j]);
	    */

	    tmp = ((i-y_resta_bajo)/y_denominador_bajo)-central_y;
	    mu03 += (tmp * tmp * tmp) * plano[columnas*i+j];
	    
	    /*
	    mu03 += (pow(((i-y_resta_bajo)/y_denominador_bajo)-central_y,3)*
		     plano[columnas*i+j]);
	    */
	  }
	}

	/*
	printf("mu11=%.12f\nmu20=%.12f\nmu02=%.12f\nmu21=%.12f\nmu12=%.12f\n"
	       "mu30=%.12f\nmu03=%.12f\n",mu11,mu20,mu02,mu21,mu12,mu30,mu03);
	*/

	m[res][0]=mu20+mu02;

	m[res][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

	m[res][2]=((mu30-3*mu12)*(mu30-3*mu12))+((mu03-3*mu21)*(mu03-3*mu21));

	m[res][3]=((mu30+mu12)*(mu30+mu12))+((mu03+mu21)*(mu03+mu21));

	m[res][4]=(((mu30-3*mu12)*(mu30+mu12))*
		   (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		   (mu03-3*mu21)*(mu03+mu21)*
		   (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));

	m[res][5]=((mu20-mu02)*
		   (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		   4*mu11*(mu30+mu12)*(mu03+mu21));

	m[res][6]=((3*mu21-mu03)*(mu30+mu12)*
		   (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		   (mu30-3*mu21)*(mu21+mu03)*
		   (((mu03+mu21)*(mu03+mu21))-3*((mu30+mu12)*(mu30+mu12))));

	/*
	printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	       m[res][0],m[res][1],m[res][2],m[res][3],
	       m[res][4],m[res][5],m[res][6]);
	printf("\n---------------------\n");
        */

      } /* for res */
    } /* if seguir */
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "los momentos\n");
  } /* if (m) */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular energias\n");

  return(m);
} /* CalcularMomentosTres */


/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentosTres                                             *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentosTres (TIPO ***m, int filas, int columnas, char *nombre)
{
  int i, k, tamano, retonno=0, items,num_mom;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (num_mom=0; num_mom<3; num_mom++)
    {
      for (i=0; (i<tamano-1) &&(!retonno); i++)
      {
	for (k=0; (k<7) && (!retonno); k++)
        {
	  /*
	  printf("m[%d][%d][%d]=%f\n",num_mom,i,k,m[num_mom][i][k]);
	  */

	  items=fwrite(&m[num_mom][i][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for i */
    } /*for num_mom */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentosTres */

TIPO ***LeerMomentosTres (char *nombre, int tamano)
{
  int i, k, items, seguir,num_mom;
  FILE *fent;
  TIPO ***m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ***)calloc(3,sizeof(TIPO **));
    seguir=(m != NULL);
    if (seguir)
    {
      for (num_mom=0; num_mom<3; num_mom++)
      {
	m[num_mom]=(TIPO **)calloc(tamano-1, sizeof(TIPO *));
	seguir=(m[num_mom] != NULL);
	for (i=0; (i<tamano-1) &&(seguir); i++)
        {
	  m[num_mom][i]=(TIPO *)calloc(7, sizeof(TIPO));
	  seguir = (m[num_mom][i] != NULL);
	  for (k=0; (k<7) && (seguir); k++)
	  {
	    items=fread(&m[num_mom][i][k], sizeof(TIPO), 1, fent);
	    if (items != 1)
	    {
	      printf("\nError de lectura en %s\n", nombre);
	      perror(" ");
	      return(NULL);
	    }
	  } /* for k */
	} /* for i */
      } /* for num_mom */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosTres */

void LiberarMomentosTres (TIPO ***momentos, int tamano)
{
  TIPO **m;
  int i,k;

  for (k=0; k<3; k++)
  {
    m=momentos[k];
    for (i=0; i<tamano-1; i++)
    {
      free(m[i]);
      m[i]=NULL;
    }
    free(m);
    m=NULL;
  } /* for k */
} /* LiberarMomentosTres */


/*
***************************************************************************
CalcularMu3 MODERNO, preparado para considerar los coeficientes de 
analisis y de detalle como si fueran una unica region.
***************************************************************************
*/

void CalcularMu3 (TIPO *plano, int tamano, int x, int y, int cuadrante, 
		  TIPO *mu11, TIPO *mu20, TIPO *mu02, TIPO *mu21,
		  TIPO *mu12, TIPO *mu30, TIPO *mu03)
{
   TIPO mom10=0, mom00=0, mom01=0, mu00=0, central_x,central_y;
   int inf_x, inf_y, sup_x, sup_y, i,j;
   int inf_x_m, inf_y_m, sup_x_m, sup_y_m;

   TIPO mu01=0, mu10=0;
   /* 
     Correcciones para cambiar de dominio a [-1,1] Se sigue una fo'rmula del
     tipo (x-resta)/denominador 
   */
   TIPO x_resta,x_denominador,y_resta,y_denominador;
   TIPO x_resta_m, x_deno_m, y_resta_m, y_deno_m;
   TIPO tmp;

   /* 
      Primero hay que pasar las coordenadas al dominio [1-,1] y seleccionar
      los limites adecuados para el bucle.
      Se consigue trabajar en (-1,1) si corregimos para trabajar en [-1,1] 
      partiendo de las dimensiones (n-1,m+1) en lugar de (n,m)
   */

   if (cuadrante == V)
   {
     /* Para pasar las coordenadas al dominio (-1,1) 
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)(x+1-x_resta);
     y_resta=(TIPO)((y/2.0)/2.0);
     y_denominador=(TIPO)((y/2.0+1.0)-y_resta);
     */

     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)((x-x/2.0)/2.0);
     y_resta=(TIPO)((y/2.0-1.0)/2.0);
     y_denominador=(TIPO)((y/2.0-1.0)/2.0);


     inf_x=x/2; 
     sup_x=x;
     inf_y=0;
     sup_y=y/2;
   }
   else if (cuadrante == D)
   {
     /* Para pasar las coordenadas al dominio (-1,1) 
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)(x+1-x_resta);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)(y+1-y_resta);
     */

     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)((x/2.0+x)/2.0);
     x_denominador=(TIPO)((x-x/2.0)/2.0);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)((y-y/2.0)/2.0);

     inf_x=x/2;
     sup_x=x;
     inf_y=y/2;
     sup_y=y;
   }
   else if (cuadrante == H)
   {
     /* Para pasar las coordenadas al dominio (-1,1) 
     x_resta=(TIPO)((x/2.0)/2.0);
     x_denominador=(TIPO)((x/2.0+1.0)-x_resta);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)(y+1-y_resta);
     */

     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)((x/2.0-1.0)/2.0);
     x_denominador=(TIPO)((x/2.0-1.0)/2.0);
     y_resta=(TIPO)((y/2.0+y)/2.0);
     y_denominador=(TIPO)((y-y/2.0)/2.0);

     inf_x=0;
     sup_x=x/2;
     inf_y=y/2;
     sup_y=y;
   }
   else if (cuadrante == U)
   {
     /* Primero el cuadrante vertical */
     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)((x+2.0*x)/2.0);
     x_denominador=(TIPO)((2.0*x-x)/2.0);
     y_resta=(TIPO)((y-1.0)/2.0);
     y_denominador=(TIPO)((y-1.0)/2.0);

     inf_x=x;
     sup_x=2*x;
     inf_y=0;
     sup_y=y;

     /* Despus el cuadrante horizontalal */
     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta_m=(TIPO)((x-1.0)/2.0);
     x_deno_m=(TIPO)((x-1.0)/2.0);
     y_resta_m=(TIPO)((y+2.0*y)/2.0);
     y_deno_m=(TIPO)((2.0*y-y)/2.0);

     inf_x_m=0;
     sup_x_m=x;
     inf_y_m=y;
     sup_y_m=2*y;
   } /* else if (cuadrante == U) */

   else /* if (cuadrante == A) || (cuadrante == M) */
   {
     /* Para pasar las coordenadas al dominio (-1,1) 
     x_resta=(TIPO)(x/2.0);
     x_denominador=(TIPO)(x/2.0+1);
     y_resta=(TIPO)(y/2.0);
     y_denominador=(TIPO)(y/2.0+1);
     */

     /* Para pasar las coordenadas al dominio [-1,1] */
     x_resta=(TIPO)(x/2.0);
     x_denominador=(TIPO)(x/2.0);
     y_resta=(TIPO)(y/2.0);
     y_denominador=(TIPO)(y/2.0);

     inf_x=0;
     sup_x=x;
     inf_y=0;
     sup_y=y;
     
     if (cuadrante == M)
     {
      /* Para pasar las coordenadas al dominio (-1,1) 
       x_resta_m=(TIPO)((x+2*x)/2.0);
       x_deno_m=(TIPO)(2*x+1-x_resta);
       y_resta_m=(TIPO)((y+2*y)/2.0);
       y_deno_m=(TIPO)(2*y+1-y_resta);
      */

       /* Para pasar las coordenadas al dominio [-1,1] */
       x_resta_m=(TIPO)((x+2.0*x)/2.0);
       x_deno_m=(TIPO)((2.0*x-x)/2.0);
       y_resta_m=(TIPO)((y+2.0*y)/2.0);
       y_deno_m=(TIPO)((2.0*y-y)/2.0);

       inf_x_m=x;
       sup_x_m=2*x;
       inf_y_m=y;
       sup_y_m=2*y;
     }
   }
   /* Primero se calculan los momentos centrales central_x y central_y */
   for (j=inf_x; j<sup_x; j++)
   {
     for(i=inf_y;i<sup_y; i++)
     {
       mom00 += fabs(plano[tamano*i+j]);
       mom10 += (fabs(plano[tamano*i+j])*(j-x_resta)/x_denominador);
       mom01 += (fabs(plano[tamano*i+j])*(i-y_resta)/y_denominador);

     }
   }
   if ((cuadrante == M) || (cuadrante == U))
   {
     for (j=inf_x_m; j<sup_x_m; j++)
     {
       for(i=inf_y_m;i<sup_y_m; i++)
       {
         mom00 += fabs(plano[tamano*i+j]);
         mom10 += (fabs(plano[tamano*i+j])*(j-x_resta_m)/x_deno_m);
         mom01 += (fabs(plano[tamano*i+j])*(i-y_resta_m)/y_deno_m);      
       }
     }
   } /* if cuadrante == M */

   if (mom00 == 0)
   {
     central_x=0;
     central_y=0;
   }
   else
   {
     central_x=mom10/mom00;
     central_y=mom01/mom00;
   }

   for (j=inf_x; j<sup_x; j++)
   {
     for(i=inf_y;i<sup_y; i++)
     {
 
       *mu11 += ((((j-x_resta)/x_denominador)-central_x)*
		 (((i-y_resta)/y_denominador)-central_y)*
		 fabs(plano[tamano*i+j]));
       
       tmp = ((j-x_resta)/x_denominador)-central_x;
       *mu20 += (tmp * tmp) * fabs(plano[tamano*i+j]);

       /*
       *mu20 += (pow(((j-x_resta)/x_denominador)-central_x,2)*
		 fabs(plano[tamano*i+j]));
       */

       tmp = ((i-y_resta)/y_denominador)-central_y;
       *mu02 += (tmp * tmp) * fabs(plano[tamano*i+j]);

       /*
       *mu02 += (pow(((i-y_resta)/y_denominador)-central_y,2)*
		 fabs(plano[tamano*i+j]));
       */

       tmp = ((j-x_resta)/x_denominador)-central_x;
       *mu21 += ((tmp * tmp) * 
		 ((i-y_resta)/y_denominador)-central_y*
		 fabs(plano[tamano*i+j]));
       /* 
       *mu21 += (pow(((j-x_resta)/x_denominador)-central_x,2)*
		 (((i-y_resta)/y_denominador)-central_y)*
		 fabs(plano[tamano*i+j]));
       */

       tmp = ((i-y_resta)/y_denominador)-central_y;
       *mu12 += ((((j-x_resta)/x_denominador)-central_x)*
		 (tmp * tmp) * fabs(plano[tamano*i+j]));

       /*
       *mu12 += ((((j-x_resta)/x_denominador)-central_x)*
		 pow(((i-y_resta)/y_denominador)-central_y,2)*
		 fabs(plano[tamano*i+j]));
       */

       tmp = ((j-x_resta)/x_denominador)-central_x;
       *mu30 += (tmp * tmp * tmp) * fabs(plano[tamano*i+j]);

       /* 
       *mu30 += (pow(((j-x_resta)/x_denominador)-central_x,3)*
		 fabs(plano[tamano*i+j]));
       */

       tmp = ((i-y_resta)/y_denominador)-central_y;
       *mu03 += (tmp * tmp * tmp) * fabs(plano[tamano*i+j]);

       /*
       *mu03 += (pow(((i-y_resta)/y_denominador)-central_y,3)*
		 fabs(plano[tamano*i+j]));
       */

       
       mu10 += (((j-x_resta)/x_denominador-central_x)*fabs(plano[tamano*i+j]));
       mu01 += (((i-y_resta)/y_denominador-central_y)*
		 fabs(plano[tamano*i+j]));
     }
   }
   
   if ((cuadrante == M) || (cuadrante == U))
   {
     for (j=inf_x_m; j<sup_x_m; j++)
     {
       for(i=inf_y_m;i<sup_y_m; i++)
       {
 
         *mu11 += ((((j-x_resta_m)/x_deno_m)-central_x)*
                   (((i-y_resta_m)/y_deno_m)-central_y)*
                   fabs(plano[tamano*i+j]));
       
         *mu20 += ((((j-x_resta_m)/x_deno_m)-central_x)*
                   (((j-x_resta_m)/x_deno_m)-central_x)*
                   fabs(plano[tamano*i+j]));
       
         *mu02 += ((((i-y_resta_m)/y_deno_m)-central_y)*
                   (((i-y_resta_m)/y_deno_m)-central_y)*
                   fabs(plano[tamano*i+j]));
       
	 tmp = ((j-x_resta)/x_deno_m)-central_x;
	 *mu21 += ((tmp * tmp)*
		   (((i-y_resta)/y_deno_m)-central_y)*
		   fabs(plano[tamano*i+j]));

	 /*
	 *mu21 += (pow(((j-x_resta)/x_deno_m)-central_x,2)*
		   (((i-y_resta)/y_deno_m)-central_y)*
		   fabs(plano[tamano*i+j]));
	 */

	 tmp = ((i-y_resta)/y_deno_m)-central_y;
	 *mu12 += ((((j-x_resta)/x_deno_m)-central_x)*
		   (tmp * tmp) * fabs(plano[tamano*i+j]));
	 
	 /*
	 *mu12 += ((((j-x_resta)/x_deno_m)-central_x)*
		   pow(((i-y_resta)/y_deno_m)-central_y,2)*
		   fabs(plano[tamano*i+j]));
	 */

	 tmp = ((j-x_resta)/x_deno_m)-central_x;
	 *mu30 += (tmp * tmp * tmp) * fabs(plano[tamano*i+j]);

	 /*
	 *mu30 += (pow(((j-x_resta)/x_deno_m)-central_x,3)*
		   fabs(plano[tamano*i+j]));
	 */

	 tmp = ((i-y_resta)/y_deno_m)-central_y;
	 *mu03 += (tmp * tmp * tmp) * fabs(plano[tamano*i+j]);

	 /*
	 *mu03 += (pow(((i-y_resta)/y_deno_m)-central_y,3)*
		   fabs(plano[tamano*i+j]));
	 */
	 mu10 += (((j-x_resta_m)/x_deno_m-central_x)*fabs(plano[tamano*i+j]));
	 mu01 += (((i-y_resta_m)/y_deno_m-central_y)*
		  fabs(plano[tamano*i+j]));
       }
     }
   } /* if cuadrante == M */
  
   mu00=mom00;
   /* NORMALIZACION DE LOS MU */
   if (mu00)
   {
     *mu11=*mu11/(mu00*mu00);
     *mu20=*mu20/(mu00*mu00);
     *mu02=*mu02/(mu00*mu00);
     *mu21=*mu21/pow(mu00,2.5);
     *mu12=*mu12/pow(mu00,2.5);
     *mu30=*mu30/pow(mu00,2.5);
     *mu03=*mu03/pow(mu00,2.5);

     /*
     *mu11=*mu11/pow(mu00,(1+1+2)/2);
     *mu20=*mu20/pow(mu00,(2+0+2)/2);
     *mu02=*mu02/pow(mu00,(0+2+2)/2);
     *mu21=*mu21/pow(mu00,(2+1+2)/2);
     *mu12=*mu12/pow(mu00,(1+2+2)/2);
     *mu30=*mu30/pow(mu00,(3+0+2)/2);
     *mu03=*mu03/pow(mu00,(0+3+2)/2);
     */
   }
   else
   {
    *mu11=0;
    *mu20=0;
    *mu02=0;
    *mu21=0;
    *mu12=0;
    *mu30=0;
    *mu03=0;
   }
   /*
   printf("mom10=%.12f\tmom01=%.12f\n", mom10,mom01);
   printf("mu00=%.12f\tmu10=%.12f\tmu01=%.12f\n", mu00,mu10,mu01);
   printf("mu20=%.12f\tmu02=%.12f\n", *mu20, *mu02);
   printf("mu21=%.12f\tmu12=%.12f\n", *mu21, *mu12);
   printf("mu30=%.12f\tmu03=%.12f\n", *mu30, *mu03);
   */
} /* CalcularMu3 */





/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMTres_S                                                  *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 2 de imagen bidimensional a   *
*   que se ha aplicado la transformada en onditas y se ha umbralizado.    *
*   Se calculan de forma individual sobre cada cuadrante de coeficientes  *
*   de detalle.                                                           *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   Vector con los momentos. Es un triple puntero en el que para cada     *
* nivel de resolucion se almacenan los momentos de las tres regiones que  *
* surgen (los dos momentos de orden 2 para cada region: fi1 y fi2, y los  *
* cinco que vienen en el Hain para orden 3). Ademas se transforman las    *
* coordenadas de [0,511] al dominio [-1,1].                               *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO ***CalcularMTres_S(TIPO *plano, int filas, int columnas)
{
  TIPO ***m=NULL,mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int resolucion,i,j,seguir,x,y,res;
  TIPO tonta;
                           

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */
 
  /* 
     A resolucion 0, cada cuadrante tiene un unico elemento. No tiene sentido
     calcular sus momentos.
  */
  m=(TIPO ***)calloc(resolucion-1,sizeof(TIPO **));
  if (m)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      m[i]=(TIPO **)calloc(3,sizeof(TIPO *));
      seguir=(m[i] != NULL);
      if (seguir)
      {
	for (j=0; j<3; j++)
	{
	  m[i][j]=(TIPO *)calloc(7,sizeof(TIPO));
	  seguir=(m[i][j] != NULL);
	}
      } /* if seguir */
    }

    /* Se ha reservado la memoria. Ahora vienen los calculos */

    if (seguir)
    {
      y=filas;
      x=columnas;

      for (res=resolucion-2; res >= 0; res--,x=x/2,y=y/2)
      {
	
	printf("RESOLUCION %d\n", res);
	
	mu20=0;mu02=0;
	mu11=0;
	mu12=0;mu21=0;
	mu03=0;mu30=0;


	CalcularMu3(plano,columnas,x,y,V,&mu11,&mu20,&mu02,&mu21,&mu12,
		    &mu30,&mu03);

	m[res][0][0]=mu20+mu02;

	m[res][0][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

	m[res][0][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		      ((mu03-3*mu21)*(mu03-3*mu21)));

	m[res][0][3]=(((mu30+mu12)*(mu30+mu12))+
		      ((mu03+mu21)*(mu03+mu21)));

	m[res][0][4]=((mu30-3*mu12)*(mu30+mu12)*
		      (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		      (mu03-3*mu21)*(mu03+mu21)*
		      (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));

	m[res][0][5]=((mu20-mu02)*
		      (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		      4*mu11*(mu30+mu12)*(mu03+mu21));

	m[res][0][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
			  (((mu30+mu12)*(mu30+mu12))-
			   3*((mu21+mu03)*(mu21+mu03)))+
			  (mu30-3*mu21)*(mu21+mu03)*
			  (((mu03+mu21)*(mu03+mu21))-
			   3*((mu30+mu12)*(mu30+mu12))));

        /*
	printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	       m[res][0][0],m[res][0][1],m[res][0][2],m[res][0][3],
	       m[res][0][4],m[res][0][5],m[res][0][6]);
	printf("Y mu02=%.12f   mu20=%.12f\n", mu02, mu20);
	printf("\n---------------------\n");
	*/

	/* Siguiente cuadrado en la misma resolucion */
	mu20=0;mu02=0;
	mu11=0;
	mu12=0;mu21=0;
	mu03=0;mu30=0;

	CalcularMu3(plano,columnas,x,y,D,&mu11,&mu20,&mu02,&mu21,&mu12,
		    &mu30,&mu03);

	m[res][1][0]=mu20+mu02;

	m[res][1][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

	m[res][1][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		      ((mu03-3*mu21)*(mu03-3*mu21)));

	m[res][1][3]=(((mu30+mu12)*(mu30+mu12))+
		      ((mu03+mu21)*(mu03+mu21)));

	m[res][1][4]=((mu30-3*mu12)*(mu30+mu12)*
		      (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		      (mu03-3*mu21)*(mu03+mu21)*
		      (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));

	m[res][1][5]=((mu20-mu02)*
		      (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		      4*mu11*(mu30+mu12)*(mu03+mu21));

	m[res][1][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
			  (((mu30+mu12)*(mu30+mu12))-
			   3*((mu21+mu03)*(mu21+mu03)))+
			  (mu30-3*mu21)*(mu21+mu03)*
			  (((mu03+mu21)*(mu03+mu21))-
			   3*((mu30+mu12)*(mu30+mu12))));
	
        /*
	printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	       m[res][1][0],m[res][1][1],m[res][1][2],m[res][1][3],
	       m[res][1][4],m[res][1][5],m[res][1][6]);
	printf("Y mu02=%.12f   mu20=%.12f\n", mu02, mu20);
	printf("\n---------------------\n");
        */

	/* Siguiente cuadrado en la misma resolucion */
	mu20=0;mu02=0;
	mu11=0;
	mu12=0;mu21=0;
	mu03=0;mu30=0;

	CalcularMu3(plano,columnas,x,y,H,&mu11,&mu20,&mu02,&mu21,&mu12,
		    &mu30,&mu03);
	
	m[res][2][0]=mu20+mu02;

	m[res][2][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

	m[res][2][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		      ((mu03-3*mu21)*(mu03-3*mu21)));

	m[res][2][3]=(((mu30+mu12)*(mu30+mu12))+
		      ((mu03+mu21)*(mu03+mu21)));

	m[res][2][4]=((mu30-3*mu12)*(mu30+mu12)*
		      (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		      (mu03-3*mu21)*(mu03+mu21)*
		      (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));

	m[res][2][5]=((mu20-mu02)*
		      (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		      4*mu11*(mu30+mu12)*(mu03+mu21));

	m[res][2][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
			  (((mu30+mu12)*(mu30+mu12))-
			   3*((mu21+mu03)*(mu21+mu03)))+
			  (mu30-3*mu21)*(mu21+mu03)*
			  (((mu03+mu21)*(mu03+mu21))-
			   3*((mu30+mu12)*(mu30+mu12))));

	
        /*
	printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	       m[res][2][0],m[res][2][1],m[res][2][2],m[res][2][3],
	       m[res][2][4],m[res][2][5],m[res][2][6]);
	printf("Y mu02=%.12f   mu20=%.12f\n", mu02, mu20);
	printf("=====================================\n");
        */

      } /* for res */
    } /* if seguir */
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "los momentos\n");
  } /* if (m) */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular energias\n");

  return(m);
} /* CalcularMTres_S */


/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirTres_S                                                   *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMTres_S (TIPO ****m, int filas, int columnas, char *nombre)
{
  int i, j, k, tamano, retonno=0, items,num_mom;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (num_mom=0; num_mom<3; num_mom++)
    {
      for (i=0; (i<tamano-1) &&(!retonno); i++)
      {
	for (j=0; (j<3) && (!retonno); j++)
        {
	  for (k=0; (k<7) && (!retonno); k++)
          {
	    /*
	    printf("m[%d][%d][%d][%d]=%f\n",num_mom,i,j,k,m[num_mom][i][j][k]);
	    */

	    items=fwrite(&m[num_mom][i][j][k], sizeof(TIPO), 1, fsal);
	    if (items != 1)
	    {
	      printf("\nError de escritura en %s\n", nombre);
	      perror(" ");
	      retonno=-1;
	    }
	  } /* for k */
	} /* for j */
      } /* for i */
    } /*for num_mom */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMTres_S */

TIPO ****LeerMTres_S (char *nombre, int tamano)
{
  int i, j, k, items, seguir,num_mom;
  FILE *fent;
  TIPO ****m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ****)calloc(3,sizeof(TIPO ***));
    seguir=(m != NULL);
    if (seguir)
    {
      for (num_mom=0; num_mom<3; num_mom++)
      {
	m[num_mom]=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
	seguir=(m[num_mom] != NULL);
	for (i=0; (i<tamano-1) &&(seguir); i++)
        {
	  m[num_mom][i]=(TIPO **)calloc(3, sizeof(TIPO *));
	  seguir = (m[num_mom][i] != NULL);
	  for (j=0; (j<3) && (seguir); j++)
          {
	    m[num_mom][i][j]=(TIPO *)calloc(7, sizeof(TIPO));
	    seguir = (m[num_mom][i][j] != NULL);
	    for (k=0; (k<7) && (seguir); k++)
	    {
	      items=fread(&m[num_mom][i][j][k], sizeof(TIPO), 1, fent);
	      if (items != 1)
	      {
		printf("\nError de lectura en %s\n", nombre);
		perror(" ");
		return(NULL);
	      }
	    } /* for k */
	  } /* for j */
	} /* for i */
      } /* for num_mom */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMTres_S */     

/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirE_Total                                                  *
***************************************************************************
*DESCRIPCION: Escribe los tres valores de energia en fichero.             *
*PARAMETROS:                                                              *
*   m:           puntero con los tres vectores de momentos.               *
*   e1,e2,e3:    valores que vamos a escribir.                            *
*SALIDA:                                                                  *
*   -1 si hay algun problema.                                             *
*    0 en otro caso.                                                      *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirE_Total (TIPO e1, TIPO e2, TIPO e3, char *nombre)
{
  int retonno=0, items;
  FILE *fsal;
  
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    items=fwrite(&e1, sizeof(TIPO), 1, fsal);
    if (items != 1)
    {
      printf("\nError de escritura en %s\n", nombre);
      perror(" ");
      retonno=-1;
    }
    else
    {
      items=fwrite(&e2, sizeof(TIPO), 1, fsal);
      if (items != 1)
      {
	printf("\nError de escritura en %s\n", nombre);
	perror(" ");
	retonno=-1;
      }
      else
      {
	items=fwrite(&e3, sizeof(TIPO), 1, fsal);
	if (items != 1)
        {
	  printf("\nError de escritura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* else => items e2 */
    } /* else => items e1 */
  } /* else => if (!fsal) */
  fclose(fsal);
  return(retonno);
} /*  EscribirE_Total */

int LeerE_Total (char *nombre, TIPO *e1, TIPO *e2, TIPO *e3)
{
  int items, retonno=0;
  FILE *fent;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    items=fread(e1, sizeof(TIPO), 1, fent);
    if (items != 1)
    {
      printf("\nError de lectura en %s\n", nombre);
      perror(" ");
      retonno=-1;
    }
    else
    {
      items=fread(e2, sizeof(TIPO), 1, fent);
      if (items != 1)
      {
	printf("\nError de lectura en %s\n", nombre);
	perror(" ");
	retonno=-1;
      }
      else
      {
	items=fread(e3, sizeof(TIPO), 1, fent);
	if (items != 1)
        {
	  printf("\nError de lectura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* else => items e2 */
    } /* else => items e1 */
  } /* else => if (!fent) */   
  fclose(fent);
  return(retonno);  
} /* LeerE_Total */



/**************************** DEBUG *******************************/


/* DescomponerDebug */
void DescomponerDebug (TIPO *tres,int filas,int col,int e_filas,int e_col)
{
  int f_pasos,c_pasos;
  TIPO pasos_f,pasos_c;
 
  pasos_f=(log((TIPO)filas)/log(2.0));
  pasos_c=(log((TIPO)col)/log(2.0));
  
  if (e_filas > 0)
  {
    if (e_filas > pasos_f)
    {
      printf("\nERROR: el numero de pasos en que quiere descomponer no es "
	     "valido.\n");
      exit(1);
    }
    else
      f_pasos=e_filas;
  }
  else
    f_pasos=pasos_f;

  if (e_col > 0)
  {
    if (e_col > pasos_c)
    {
      printf("\nERROR: el numero de pasos en que quiere descomponer no es "
	     "valido.\n");
      exit(1);
    }
    else
      c_pasos=e_col;
  }
  else
    c_pasos=pasos_c;

  /* OJO: FILAS=COLUMNAS */
  if (filas != col)
  {
    printf("\nHa intentado descomponer una imagen no cuadrada.\n"
	   "Esto no es posible.\n");
    exit(1);
  }

  /*
  j=filas;
  for (i=1; i<=f_pasos; i++)
  {
    NonstandardDecomposition2  (tres,j,1);
    j/= 2;
  }
  */
  NonstandardDecomposition (tres,filas);

  Escribe_pgm(filas,col,"NonStdDesc_1.pgm",tres,1);
} /* DescomponerDebug */


/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentosDebug                                            *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentosDebug (TIPO ***m, int filas, int columnas, char *nombre)
{
  int i, j, k, tamano, retonno=0, items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano-1) &&(!retonno); i++)
    {
      for (j=0; (j<3) && (!retonno); j++)
      {
	for (k=0; (k<7) && (!retonno); k++)
        {
	  /*
	  printf("m[%d][%d][%d][%d]=%f\n",num_mom,i,j,k,m[num_mom][i][j][k]);
	  */

	  items=fwrite(&m[i][j][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentosDebug */

TIPO ***LeerMomentosDebug (char *nombre, int tamano)
{
  int i, j, k, items, seguir;
  FILE *fent;
  TIPO ***m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
    seguir=(m != NULL);
    for (i=0; (i<tamano-1) &&(seguir); i++)
    {
      m[i]=(TIPO **)calloc(3, sizeof(TIPO *));
      seguir = (m[i] != NULL);
      for (j=0; (j<3) && (seguir); j++)
      {
	m[i][j]=(TIPO *)calloc(7, sizeof(TIPO));
	seguir = (m[i][j] != NULL);
	for (k=0; (k<7) && (seguir); k++)
	{
	  items=fread(&m[i][j][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for j */
    } /* for i */
  
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosDebug */



/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
** Calculo de momentos utilizando los valores los coeficientes de        **
** analisis, en vez de los coeficientes de detalle.                      **
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMomAnal                                                  *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional.  *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMomAnal(TIPO *plano, int tam_reg, TIPO **momento)
{
  TIPO mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int x,y;
  TIPO *m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;


  CalcularMu3(plano,tam_reg,x,y,A,&mu11,&mu20,&mu02,&mu21,&mu12,
	      &mu30,&mu03);
  

  m_nivel[0]=mu20+mu02;
  
  m_nivel[1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

  m_nivel[2]=(((mu30-3*mu12)*(mu30-3*mu12))+
	      ((mu03-3*mu21)*(mu03-3*mu21)));

  m_nivel[3]=(((mu30+mu12)*(mu30+mu12))+
	      ((mu03+mu21)*(mu03+mu21)));
  
  m_nivel[4]=((mu30-3*mu12)*(mu30+mu12)*
	      (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
	      (mu03-3*mu21)*(mu03+mu21)*
	      (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
  m_nivel[5]=((mu20-mu02)*
	      (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
	      4*mu11*(mu30+mu12)*(mu03+mu21));
  
  m_nivel[6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		  (((mu30+mu12)*(mu30+mu12))-
		   3*((mu21+mu03)*(mu21+mu03)))+
		  (mu30-3*mu21)*(mu21+mu03)*
		  (((mu03+mu21)*(mu03+mu21))-
		   3*((mu30+mu12)*(mu30+mu12))));

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
  m_nivel[0],m_nivel[1],m_nivel[2],m_nivel[3],m_nivel[4],m_nivel[5],
  m_nivel[6]);
  printf("\n---------------------\n");
  */

  return(0);
} /* CalcularMomAnal */

/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularE_UmbralAnal                                             *
***************************************************************************
*DESCRIPCION: Calcula la energia media de los coeficientes de analisis de *
*   un nivel de resolucion, que quedara determinado en base a los rangos  *
*   en los que nos moveremos.                                             *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   tam_img:  tamano de la imagen.                                        *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_UmbralAnal(TIPO *plano,int rango_x, int tam_img, 
			  int tipo_umbral,TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,tmp;
  int i,j,num_pixels=0;

  for (i=0; i<rango_x; i++)
  {
    for(j=0;j<rango_x; j++)
    {
      e_media+=(plano[tam_img*i+j]*plano[tam_img*i+j]);
      num_pixels++;
    }
  }
  
  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (i=0; i<rango_x; i++)
    {
      for(j=0;j<rango_x; j++)
      {
	tmp = plano[tam_img*i+j] * plano[tam_img*i+j];
	tmp = tmp - e_media;
	varianza += tmp * tmp;
	
	/*
	varianza+=pow(pow(plano[tam_img*i+j],2)-e_media,2);
	*/
      }
    }
    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    
    valor = e_media + (varianza * factor);
    return(valor);
   
  }/* else */
} /* CalcularE_UmbralAnal */

/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarAnal                                                   *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarAnal (TIPO *plano,int tam_reg,int tam_img,int tipo_umbral)
{
  int i,j;
  TIPO e_umbral, *umb=NULL;
  
  umb=(TIPO *)calloc(tam_reg*tam_reg,sizeof(TIPO));

  if (umb)
  {
    e_umbral=CalcularE_UmbralAnal(plano,tam_reg,tam_img,tipo_umbral,1.0);
    for (i=0; i<tam_reg; i++)
      for(j=0; j<tam_reg; j++)
	umb[tam_reg*i+j]=((plano[tam_img*i+j]*
			 plano[tam_img*i+j]) >= e_umbral);
      
 
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }
  /*
  for (i=0; i<tam_reg*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
	*/
  return(umb);
} /* UmbralizarAnal */


/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarUserAnal                                               *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   umbral:   umbral que se exige.                                        *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarUserAnal (TIPO *plano, int tam_reg, int tam_img,
			  TIPO factor, int tipo_umbral)
{
  int i,j;
  TIPO e_umbral, *umb=NULL;

  umb=(TIPO *)calloc(tam_reg*tam_reg,sizeof(TIPO));

  if (umb)
  {
    e_umbral=CalcularE_UmbralAnal(plano,tam_reg,tam_img,tipo_umbral,1.0);

    for (i=0; i<tam_reg; i++)
      for(j=0; j<tam_reg; j++)
      {
	umb[tam_reg*i+j]=((plano[tam_img*i+j]*
			 plano[tam_img*i+j]) >= e_umbral);
	/*
	printf("%.15f\n",umb[tam_reg*i+j]);
	*/
      }
 
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }
  /*
  for (i=0; i<tam_reg*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
	*/
  return(umb);

} /* UmbralizarUserAnal */



/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom                                                         *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada.                                                         *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO  **DescYMom (TIPO *c, int tamano, int hay_umbral, TIPO umbral_user) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i, resolucion,seguir;
  TIPO **mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO **)calloc(resolucion-1,sizeof(TIPO *));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
	/* Se consideran los 7 momentos para el nivel de 
	   resolucion */
	mom[i]=(TIPO *)calloc(7,sizeof(TIPO));
	seguir=(mom[i] != NULL);
    } /* for i */
  } /* if mom */

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
/****** DESACTIVAMOS ESTA ORMALIZACION PARA QUE EQUIVALGA A MATLAB ***
#ifdef NORMALIZAR
  {
    int cont;
    for (cont=0; cont < tamano*tamano; cont++)
      c[cont] /= (TIPO)tamano;
  }
#endif
****/
  while ((g >= 2) && (resolucion >= 0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    
    
   if (hay_umbral)
     umb=UmbralizarUserAnal(c,g,tamano,umbral_user,VARIANZA);
   else
     umb=UmbralizarAnal(c,g,tamano,VARIANZA);
   
   CalcularMomAnal(umb,g,&mom[resolucion]);
   free(umb);
   umb=NULL;

   resolucion--;
  } /* while */
  return (mom);
} /* DescYMom */


/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
** Calculo de energias utilizando los valores de los coeficientes de     **
** analisis, en vez de los coeficientes de detalle.                      **
**************************************************************************/

/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularEnergAnal                                                *
***************************************************************************
*DESCRIPCION: Calcula las energia de una region de una imagen.            *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   tam_reg:  numero de filas y columnas de la region.                    *
*   tam_img:  numero de columnas de la imagen.                            *
*SALIDA:                                                                  *
*   La energia calculada                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO CalcularEnergAnal(TIPO *plano, int tam_reg, int tam_img)
{
  int i,j;
  TIPO energ=0;
  
  for (i=0; i<tam_reg; i++)
    for (j=0; j<tam_reg; j++)
      energ += plano[tam_img*i+j]*plano[tam_img*i+j];

  return(energ);
  
} /* CalcularEnergAnal */

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularCovAnal                                                  *
***************************************************************************
*DESCRIPCION: Calcula los coeficientes de covarianza de una imagen        *
*bidimensional. Es IMPRESCINDIBLE que se haya calculado la transformada   *
*en onditas hasta el maximo nivel de resolucion.                          *
*PARAMETROS:                                                              *
*   plano1,plano2,plano3: punteros a los planos de color de la imagen.    *
*   filas:                numero de filas y columnas de la region         *
*   tam_img:              tamano de la imagen.                            *
*SALIDA:                                                                  *
*   Vector con las covarianzas. Es un doble  puntero en el que para cada  *
*nivel de resolucion se almacena, en la posicion que corresponderia a la  *
*energia Ei, los coeficientes de covarianza correspondientes a comparar   * 
*los planos 12, 13 y 23 y en ese orden.                                   *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

void CalcularCovAnal(TIPO *plano1, TIPO *plano2, TIPO *plano3,
		     int tam_reg, int tam_img, TIPO **covarianzas)
{
  TIPO *cov=*covarianzas;
  int i,j;

  for(i=0;i<tam_reg; i++)
  {
    for (j=0; j<tam_reg; j++);
    {
      /* Planos 1 y 2 */
      cov[0] += plano1[tam_img*i+j]*plano2[tam_img*i+j];

      /* Planos 1 y 3 */
      cov[1] += plano1[tam_img*i+j]*plano3[tam_img*i+j];

      /* Planos 2 y 3 */
      cov[2] += plano2[tam_img*i+j]*plano3[tam_img*i+j];
    }
  }
	  
} /* CalcularCovarianzas */


/* 11, 12, 13, 22, 23, 33 */
TIPO **NormalizarCovAnal(TIPO **e, TIPO ***cov, int tamano)
{
  int i,seguir;
  TIPO **c=*cov, **corr=NULL;

  corr=(TIPO **)calloc(tamano, sizeof(TIPO *));
  if (corr)
  {
    seguir=1;
    for (i=0; (i<tamano) && (seguir); i++)
    {
      corr[i]=(TIPO *)calloc(6, sizeof(TIPO));
      seguir = (corr[i] != NULL);
      if (seguir)
      {
	  corr[i][0]=e[0][i];                        /* Plano 1*/

	  if ((e[0][i]==0) || (e[1][i]==0))
	    corr[i][1]=0;
	  else
	    corr[i][1]=c[i][0]/(e[0][i]*e[1][i]);  /* Planos 1 y 2 */
	  
	  if ((e[0][i]==0) || (e[2][i]==0))
	    corr[i][2]=0;
	  else
	    corr[i][2]=c[i][1]/(e[0][i]*e[2][i]);  /* Planos 1 y 3 */

	  corr[i][3]=e[1][i];                        /* Plano 2 */

	  if ((e[1][i]==0) || (e[2][i]==0))
	    corr[i][4]=0;
	  else
	    corr[i][4]=c[i][2]/(e[1][i]*e[2][i]);  /* Planos 2 y 3 */
	  
	  corr[i][5]=e[2][i];                        /* Plano 3 */
      } /* if seguir */
    } /* for i */
    if (seguir)
    {
      for (i=0; i<tamano; i++)
      {
	free(c[i]);
	c[i]=NULL;
      }
      free(c);
      c=NULL;
    }
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "correlaciones\n");
  } /* if corr */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "correlaciones\n");
  
  /*
  for (res=0; res<=tamano-2; res++)
    for (i=0; i<3; i++)
      for (j=0; j<6; j++)
	printf("corr[%d][%d][%d]=%f\n",res,i,j,corr[res][i]);
  */


  return(corr);
} /* NormalizarCovAnal */



/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYEnerg                                                       *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de la    *
*   energia de los coeficientes de analisis de cada paso de la            *
*   transformada.                                                         *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
**************************************************************************/
TIPO  **DescYEnerg (TIPO *c, TIPO *c2, TIPO *c3, int tamano, 
		      TIPO ***covs) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux, *columnaAux2, *columnaAux3, **covarianzas=*covs;

  int resolucion;
  TIPO **energia=NULL,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
 
  /* Reserva de memoria para las energias */
  energia=(TIPO **)calloc(3,sizeof(TIPO *));
  if (!energia)
  {
    perror("No se pudo reservar memoria para calcular energias");
    return(NULL);
  }
  energia[0]=(TIPO *)calloc(resolucion,sizeof(TIPO ));
  if (!energia[0])
  {
    perror("No se pudo reservar memoria para calcular energias");
    return(NULL);
  }
  energia[1]=(TIPO *)calloc(resolucion,sizeof(TIPO ));
  if (!energia[1])
  {
    perror("No se pudo reservar memoria para calcular energias");
    return(NULL);
  }
  energia[2]=(TIPO *)calloc(resolucion,sizeof(TIPO ));
  if (!energia[2])
  {
    perror("No se pudo reservar memoria para calcular energias");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=1;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  columnaAux2 = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux2)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  columnaAux3 = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux3)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }

/****** DESACTIVAMOS ESTA ORMALIZACION PARA QUE EQUIVALGA A MATLAB ***
#ifdef NORMALIZAR
  {
    int cont;
    for (cont=0; cont < tamano*tamano; cont++)
      c[cont] /= (TIPO)tamano;
  }
#endif
****/
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++){
      DecompositionStep (c + cont_filas*tamano, g);
      DecompositionStep (c2 + cont_filas*tamano, g);
      DecompositionStep (c3 + cont_filas*tamano, g);
    }
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++){
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
	columnaAux2[cont_filas] = c2 [cont_filas*tamano+cont_colum];
	columnaAux3[cont_filas] = c3 [cont_filas*tamano+cont_colum];
      }
      DecompositionStep (columnaAux, g);
      DecompositionStep (columnaAux2, g);
      DecompositionStep (columnaAux3, g);
      for (cont_filas=0; cont_filas < g; cont_filas++){
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
        c2 [cont_filas*tamano+cont_colum] = columnaAux2[cont_filas];
        c3 [cont_filas*tamano+cont_colum] = columnaAux3[cont_filas];
      }
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    
    energia[0][resolucion]=CalcularEnergAnal(c,g,tamano);
    energia[1][resolucion]=CalcularEnergAnal(c2,g,tamano);
    energia[2][resolucion]=CalcularEnergAnal(c3,g,tamano);

    CalcularCovAnal(c, c2, c3, g, tamano,&covarianzas[resolucion]);

    resolucion--;
  } /* while */
  return (energia);
} /* DescYEnerg */


int EscribirCorrAnal (TIPO **corr, int tamano, char *nombre)
{
  int i, k, retonno=0, items;
  FILE *fsal;

  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano) &&(!retonno); i++)
    {
      for (k=0; (k<6) && (!retonno); k++)
      {
	items=fwrite(&corr[i][k], sizeof(TIPO), 1, fsal);
	if (items != 1)
	{
	  printf("\nError de escritura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* for k */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirCorrelaciones */

TIPO **LeerCorrAnal (char *nombre, int tamano)
{
  int i, k, items, seguir;
  FILE *fent;
  TIPO **corr;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    corr=(TIPO **)calloc(tamano, sizeof(TIPO *));
    seguir=(corr != NULL);
    for (i=0; (i<tamano) &&(seguir); i++)
    {
      corr[i]=(TIPO *)calloc(6, sizeof(TIPO));
      seguir = (corr[i] != NULL);
      for (k=0; (k<6) && (seguir); k++)
      {
	items=fread(&corr[i][k], sizeof(TIPO), 1, fent);
	if (items != 1)
	{
	  printf("\nError de lectura en %s\n", nombre);
	  perror(" ");
	  return(NULL);
	}
      } /* for k */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para las"
	     "correlaciones.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(corr);
  } /* else */
} /* LeerCorrAnal */

void LiberarCorrAnal (TIPO ***correlaciones, int tamano)
{
  TIPO **corr=*correlaciones;
  int i;

  for (i=0; i<tamano; i++)
  {
    free(corr[i]);
    corr[i]=NULL;
  }
  free(corr);
  corr=NULL;
} /* LiberarCorrAnal */



/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
** Calculo de energias utilizando los valores de los coeficientes de     **
** analisis y los coeficientes de detalle del cuadrante diagonal.        **
**************************************************************************/

/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularEnergA_2                                                 *
***************************************************************************
*DESCRIPCION: Calcula las energia de los cuadrantes de coeficientes de    *
*   analisis y diagonal de una imagen.                                    *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   tam_reg:  numero de filas y columnas de la region.                    *
*   tam_img:  numero de columnas de la imagen.                            *
*SALIDA:                                                                  *
*   La energia calculada                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO *CalcularEnergA_2(TIPO *plano, int tam_reg, int tam_img)
{
  int i,j;
  TIPO *energ=NULL;
  

  energ=(TIPO *)calloc(2,sizeof(TIPO));
  if (!energ)
  {
    perror("Sin memoria para calculo de Energias_2");
    return(NULL);
  }
  
  /* Coeficientes de analisis */
  for (i=0; i<tam_reg; i++)
    for (j=0; j<tam_reg; j++)
      energ[0] += plano[tam_img*i+j]*plano[tam_img*i+j];

  /* Cuadrante diagonal */
  for (i=tam_reg; i<(2*tam_reg); i++)
    for (j=tam_reg; j<(2*tam_reg); j++)
      energ[1] += plano[tam_img*i+j]*plano[tam_img*i+j];

  return(energ);
  
} /* CalcularEnergA_2 */

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularCovA_2                                                   *
***************************************************************************
*DESCRIPCION: Calcula los coeficientes de covarianza de una imagen        *
*bidimensional. Es IMPRESCINDIBLE que se haya calculado la transformada   *
*en onditas hasta el maximo nivel de resolucion.                          *
*PARAMETROS:                                                              *
*   plano1,plano2,plano3: punteros a los planos de color de la imagen.    *
*   filas:                numero de filas y columnas de la region         *
*   tam_img:              tamano de la imagen.                            *
*SALIDA:                                                                  *
*   Vector con las covarianzas. Es un doble  puntero en el que para cada  *
*nivel de resolucion se almacena, en la posicion que corresponderia a la  *
*energia Ei, los coeficientes de covarianza correspondientes a comparar   * 
*los planos 12, 13 y 23 y en ese orden.                                   *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

void CalcularCovA_2(TIPO *plano1, TIPO *plano2, TIPO *plano3,
		     int tam_reg, int tam_img, TIPO ***covarianzas)
{
  TIPO **cov=*covarianzas;
  int i,j;

  /* Coeficientes de analisis */
  for(i=0;i<tam_reg; i++)
  {
    for (j=0; j<tam_reg; j++);
    {
      /* Planos 1 y 2 */
      cov[0][0] += plano1[tam_img*i+j]*plano2[tam_img*i+j];

      /* Planos 1 y 3 */
      cov[0][1] += plano1[tam_img*i+j]*plano3[tam_img*i+j];

      /* Planos 2 y 3 */
      cov[0][2] += plano2[tam_img*i+j]*plano3[tam_img*i+j];
    }
  }

  /* Cuadrante diagonal */
  for(i=tam_reg;i<(2*tam_reg); i++)
  {
    for (j=tam_reg; j<(2*tam_reg); j++);
    {
      /* Planos 1 y 2 */
      cov[1][0] += plano1[tam_img*i+j]*plano2[tam_img*i+j];

      /* Planos 1 y 3 */
      cov[1][1] += plano1[tam_img*i+j]*plano3[tam_img*i+j];

      /* Planos 2 y 3 */
      cov[1][2] += plano2[tam_img*i+j]*plano3[tam_img*i+j];
    }
  }
	  
} /* CalcularCovA_2 */


/* 11, 12, 13, 22, 23, 33 */
TIPO ***NormalizarCovA_2(TIPO ***e, TIPO ****cov, int tamano)
{
  int i, j, seguir;
  TIPO ***c=*cov, ***corr=NULL;

  corr=(TIPO ***)calloc(tamano, sizeof(TIPO **));
  if (corr)
  {
    seguir=1;
    for (i=0; (i<tamano) && (seguir); i++)
    {
      corr[i]=(TIPO **)calloc(2, sizeof(TIPO *));
      seguir = (corr[i] != NULL);
      for (j=0; j<2; j++)
      {
	/* 
	 j determina si accedemos a la parte calculada sobre coefs. de
	 analisis o sobre el cuadrante diagonal de coefs. de detalle
	*/
	corr[i][j]=(TIPO *)calloc(6,sizeof(TIPO ));
	seguir = (corr[i][j] != NULL);
	if (seguir)
        {
	  corr[i][j][0]=e[0][i][j];                        /* Plano 1*/

	  if ((e[0][i][j]==0) || (e[1][i][j]==0))
	    corr[i][j][1]=0;
	  else
	    corr[i][j][1]=c[i][j][0]/(e[0][i][j]*e[1][i][j]);/* Planos 1 y 2 */
	  
	  if ((e[0][i][j]==0) || (e[2][i][j]==0))
	    corr[i][j][2]=0;
	  else
	    corr[i][j][2]=c[i][j][1]/(e[0][i][j]*e[2][i][j]);/* Planos 1 y 3 */

	  corr[i][j][3]=e[1][i][j];                        /* Plano 2 */

	  if ((e[1][i][j]==0) || (e[2][i][j]==0))
	    corr[i][j][4]=0;
	  else
	    corr[i][j][4]=c[i][j][2]/(e[1][i][j]*e[2][i][j]);/* Planos 2 y 3 */
	  
	  corr[i][j][5]=e[2][i][j];                        /* Plano 3 */
	} /* if seguir */
      } /* for j */
    } /* for i */
    if (seguir)
    {
      for (i=0; i<tamano; i++)
      {
	for (j=0; j<2; j++)
	{
	  free(c[i][j]);
	  c[i][j]=NULL;
	}
	
	free(c[i]);
	c[i]=NULL;
      }
      free(c);
      c=NULL;
    }
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "correlaciones\n");
  } /* if corr */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular"
	   "correlaciones\n");
  
  return(corr);
} /* NormalizarCovA_2 */


/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYEnerg2                                                      *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de la    *
*   energia de los coeficientes de analisis de cada paso de la            *
*   transformada.                                                         *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
**************************************************************************/
TIPO  ***DescYEnerg2 (TIPO *c, TIPO *c2, TIPO *c3, int tamano, 
			TIPO ****covs) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux, *columnaAux2, *columnaAux3, ***covarianzas=*covs;

  int resolucion,i;
  TIPO ***energia=NULL,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
 
  /* Reserva de memoria para las energias */
  energia=(TIPO ***)calloc(3,sizeof(TIPO **));
  if (!energia)
  {
    perror("No se pudo reservar memoria para calcular energias");
    return(NULL);
  }
  for (i=0; i<3; i++)
  {
    energia[i]=(TIPO **)calloc(resolucion,sizeof(TIPO *));
    if (!energia[i])
    {
      perror("No se pudo reservar memoria para calcular energias");
      return(NULL);
    }
  } /* for i */

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=1;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  columnaAux2 = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux2)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  columnaAux3 = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux3)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }

/****** DESACTIVAMOS ESTA ORMALIZACION PARA QUE EQUIVALGA A MATLAB ***
#ifdef NORMALIZAR
  {
    int cont;
    for (cont=0; cont < tamano*tamano; cont++)
      c[cont] /= (TIPO)tamano;
  }
#endif
****/
  while (g >= 2) {
    for (cont_filas=0; cont_filas < g; cont_filas++){
      DecompositionStep (c + cont_filas*tamano, g);
      DecompositionStep (c2 + cont_filas*tamano, g);
      DecompositionStep (c3 + cont_filas*tamano, g);
    }
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++){
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
	columnaAux2[cont_filas] = c2 [cont_filas*tamano+cont_colum];
	columnaAux3[cont_filas] = c3 [cont_filas*tamano+cont_colum];
      }
      DecompositionStep (columnaAux, g);
      DecompositionStep (columnaAux2, g);
      DecompositionStep (columnaAux3, g);
      for (cont_filas=0; cont_filas < g; cont_filas++){
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
        c2 [cont_filas*tamano+cont_colum] = columnaAux2[cont_filas];
        c3 [cont_filas*tamano+cont_colum] = columnaAux3[cont_filas];
      }
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;

    energia[0][resolucion]=CalcularEnergA_2(c,g,tamano);
    energia[1][resolucion]=CalcularEnergA_2(c2,g,tamano);
    energia[2][resolucion]=CalcularEnergA_2(c3,g,tamano);

    CalcularCovA_2(c, c2, c3, g, tamano,&covarianzas[resolucion]);

    resolucion--;
  } /* while */
  return (energia);
} /* DescYEnerg */

int EscribirCorrA_2 (TIPO ***corr, int tamano, char *nombre)
{
  int i,j, k, retonno=0, items;
  FILE *fsal;

  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano) &&(!retonno); i++)
    {
      for (j=0; j<2; j++)
      {
	for (k=0; (k<6) && (!retonno); k++)
	{
	  items=fwrite(&corr[i][j][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirCorrA_2 */

TIPO ***LeerCorrA_2 (char *nombre, int tamano)
{
  int i, j, k, items, seguir;
  FILE *fent;
  TIPO ***corr;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    corr=(TIPO ***)calloc(tamano, sizeof(TIPO **));
    seguir=(corr != NULL);
    for (i=0; (i<tamano) &&(seguir); i++)
    {
      corr[i]=(TIPO **)calloc(2, sizeof(TIPO *));
      seguir=(corr[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	corr[i][j]=(TIPO *)calloc(6, sizeof(TIPO));
	seguir = (corr[i][j] != NULL);
	for (k=0; (k<6) && (seguir); k++)
        {
	  items=fread(&corr[i][j][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para las"
	     "correlaciones.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(corr);
  } /* else */
} /* LeerCorrA_2 */

void LiberarCorrA_2 (TIPO ****correlaciones, int tamano)
{
  TIPO ***corr=*correlaciones;
  int i,j;

  for (i=0; i<tamano; i++)
  {
    for (j=0; j<2; j++)
    {
      free(corr[i][j]);
      corr[i][j]=NULL;
    }
    free(corr[i]);
    corr[i]=NULL;
  }
  free(corr);
  corr=NULL;
} /* LiberarCorrA_2 */


/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
**   Se umbralizan los cuadrantes de coeficientes de detalle exigiendo   **
** que se supere el umbral en la posicion equivalente en los tres        **
** cuadrantes a la vez.                                                  **
**************************************************************************/

/**************************************************************************
NOMBRE: CalcularE_UmbralEq                                                *
***************************************************************************
*DESCRIPCION: Calcula la energia media de los coeficientes de analisis de *
*   un nivel de resolucion, que quedara determinado en base a los rangos  *
*   en los que nos moveremos.                                             *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   tam_img:  tamano de la imagen.                                        *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_UmbralEq(TIPO *plano,int cuadrante, int x, int y, 
			int tam_img, int tipo_umbral, TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,tmp;
  int i,j,num_pixels=0,inf_x,inf_y,sup_x,sup_y;

  if (cuadrante == V)
  {
    inf_x=x/2; 
    sup_x=x;
    inf_y=0;
    sup_y=y/2;
  }
  else if (cuadrante == D)
  {
    inf_x=x/2;
    sup_x=x;
    inf_y=y/2;
    sup_y=y;
  }
  else if (cuadrante == H)
  {
    inf_x=0;
    sup_x=x/2;
    inf_y=y/2;
    sup_y=y;
  }
  else if (cuadrante == A)
  {
    inf_x=0;
    sup_x=x;
    inf_y=0;
    sup_y=y;
  }

  
  for (i=inf_y; i<sup_y; i++)
  {
    for(j=inf_x;j<sup_x; j++)
    {
      e_media+=(plano[tam_img*i+j]*plano[tam_img*i+j]);
      num_pixels++;
    }
  }
  
  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (i=inf_y; i<sup_y; i++)
    {
      for(j=inf_x;j<sup_x; j++)
      {
	tmp = plano[tam_img*i+j] * plano[tam_img*i+j];
	tmp = tmp - e_media;
	varianza += tmp * tmp;
	
	/*
	varianza+=pow(pow(plano[tam_img*i+j],2)-e_media,2);
	*/
      }
    }
    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    valor = e_media + (varianza * factor);
    return(valor);
  }/* else */
} /* CalcularE_UmbralEq */

/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarEquivalentes                                           *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarEquivalentes (TIPO *plano, int tam_reg, int tam_img,
			      TIPO factor, int tipo_umbral)
{
  int i,j,k,l,tamano;
  TIPO e_umbral, *umb=NULL;
  
  tamano=2*tam_reg;
  /* tam_reg indica el limite de la region de coefs. de analisis y la de
     coefs. de detalle. Tamano es la region completa que ha cambiado tras
     el ultimo paso de la transformada.
  */

  umb=(TIPO *)calloc(tamano*tamano,sizeof(TIPO));

  if (umb)
  {
    /* Coeficientes de analisis */
    e_umbral=(CalcularE_UmbralEq(plano,A,tam_reg,tam_reg,tam_img,tipo_umbral,
				 factor));
    for (i=0; i<tam_reg; i++)
      for(j=0; j<tam_reg; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);
    
    /* Cuadrante Vertical */
    e_umbral=(CalcularE_UmbralEq(plano,V,tamano,tamano,tam_img,tipo_umbral,
				 factor));
    for (i=0; i<tam_reg; i++)
      for (j=tam_reg; j<tamano; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);

    /* Cuadrante Diagonal */
    e_umbral=(CalcularE_UmbralEq(plano,D,tamano,tamano,tam_img,tipo_umbral,
				 factor));
    for (i=tam_reg; i<tamano;i++)
      for (j=tam_reg; j<tamano; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			   plano[tam_img*i+j]) >= e_umbral);
    
    /* Cuadrante Horizontal */
    e_umbral=(CalcularE_UmbralEq(plano,H,tamano,tamano,tam_img,tipo_umbral,
				 factor));
    for (i=tam_reg; i<tamano; i++)
      for (j=0; j<tam_reg; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);

    /* Ahora buscamos las posiciones equivalentes y nos las quedamos en el
       cuadrante diagonal, que es el que nos interesa.
    */

    /* (i,j) es el cuadrante V, (k,j) es el diagonal y (k,l) es el H */

    for (i=0,k=tam_reg; i<tam_reg; i++,k++) /* k<tamano siempre */
    {
      for (j=tam_reg,l=0; j<tamano; j++,l++) /* l<tam_reg siempre */
      {
	if ((umb[tamano*i+j] != 1) || (umb[tamano*k+j] != 1) ||
	    (umb[tamano*k+l] != 1))
	{
	  umb[tamano*k+j]=0;
	} /* if */
      } /* for j */
    } /* for i */
      
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }
  /*
  for (i=0; i<tam_reg*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
	*/
  return(umb);
} /* UmbralizarEquivalentes */




/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
**   Se calculan los momentos para los coeficientes de analisis y para   **
** el cuadrante diagonal a la vez.                                       **
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMomAnalDet                                               *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional y *
* sobre los coeficientes de detalle del cuadrante diagonal.               *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMomAnalDet(TIPO *plano, int tam_reg, TIPO ***momento)
{
  TIPO mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int x,y;
  TIPO **m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;

  /* Coeficientes de anlisis */
  printf("\n\nCUADRANTE ANALISIS\n");

  CalcularMu3(plano,2*tam_reg,x,y,A,&mu11,&mu20,&mu02,&mu21,&mu12,
	      &mu30,&mu03);
  

  m_nivel[0][0]=mu20+mu02;
  
  m_nivel[0][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
 
  m_nivel[0][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
              ((mu03-3*mu21)*(mu03-3*mu21)));
 
  m_nivel[0][3]=(((mu30+mu12)*(mu30+mu12))+
              ((mu03+mu21)*(mu03+mu21)));
  
  m_nivel[0][4]=((mu30-3*mu12)*(mu30+mu12)*
              (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
              (mu03-3*mu21)*(mu03+mu21)*
              (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
  m_nivel[0][5]=((mu20-mu02)*
              (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
              4*mu11*(mu30+mu12)*(mu03+mu21));
  
  m_nivel[0][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		     (((mu30+mu12)*(mu30+mu12))-
		      3*((mu21+mu03)*(mu21+mu03)))+
		     (mu30-3*mu21)*(mu21+mu03)*
		     (((mu03+mu21)*(mu03+mu21))-
		      3*((mu30+mu12)*(mu30+mu12))));
 
  
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
  m_nivel[0][0],m_nivel[0][1],m_nivel[0][2],m_nivel[0][3],m_nivel[0][4],
  m_nivel[0][5],m_nivel[0][6]);
  printf("\n---------------------\n");
  


  /* Cuadrante diagonal */
  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;
 
  printf("\n\nCUADRANTE DIAGONAL\n");
  CalcularMu3(plano,2*tam_reg,2*x,2*y,D,&mu11,&mu20,&mu02,&mu21,&mu12,
	      &mu30,&mu03);
  printf("\n");
  m_nivel[1][0]=mu20+mu02;
  
  m_nivel[1][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
 
  m_nivel[1][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
              ((mu03-3*mu21)*(mu03-3*mu21)));
 
  m_nivel[1][3]=(((mu30+mu12)*(mu30+mu12))+
              ((mu03+mu21)*(mu03+mu21)));
  
  m_nivel[1][4]=((mu30-3*mu12)*(mu30+mu12)*
              (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
              (mu03-3*mu21)*(mu03+mu21)*
              (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
  m_nivel[1][5]=((mu20-mu02)*
              (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
              4*mu11*(mu30+mu12)*(mu03+mu21));
  
  m_nivel[1][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		     (((mu30+mu12)*(mu30+mu12))-
		      3*((mu21+mu03)*(mu21+mu03)))+
		     (mu30-3*mu21)*(mu21+mu03)*
		     (((mu03+mu21)*(mu03+mu21))-
		      3*((mu30+mu12)*(mu30+mu12))));
  
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
  m_nivel[1][0],m_nivel[1][1],m_nivel[1][2],m_nivel[1][3],m_nivel[1][4],
  m_nivel[1][5],m_nivel[1][6]);
  printf("\n=====================\n");
  


  return(0);
} /* CalcularMomAnalDet */

/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularE_UmbralDet                                              *
***************************************************************************
*DESCRIPCION: Calcula la energia media de los coeficientes de analisis de *
*   un nivel de resolucion, que quedara determinado en base a los rangos  *
*   en los que nos moveremos.                                             *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   tam_img:  tamano de la imagen.                                        *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_UmbralDet(TIPO *plano, int rango_x, int tam_img,
			 int tipo_umbral, TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,tmp;
  int i,j,num_pixels=0;

  for (i=rango_x; i<2*rango_x; i++)
  {
    for(j=rango_x;j<2*rango_x; j++)
    {
      e_media+=(plano[tam_img*i+j]*plano[tam_img*i+j]);
      num_pixels++;
    }
  }
  
  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (i=rango_x; i<2*rango_x; i++)
    {
      for(j=rango_x;j<2*rango_x; j++)
      {
	tmp = plano[tam_img*i+j] * plano[tam_img*i+j];
	tmp = tmp - e_media;
	varianza += tmp * tmp;

	/*
	varianza+=pow(pow(plano[tam_img*i+j],2)-e_media,2);
	*/
      }
    }
    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    valor = e_media + (varianza * factor);
    return(valor);
  } /* else */
} /* CalcularE_UmbralDet */

/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarAnalDet                                                *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarAnalDet (TIPO *plano, int tam_reg, int tam_img,
			 int tipo_umbral)
{
  int i,j,tamano;
  TIPO e_umbral, *umb=NULL;
  
  tamano=2*tam_reg;
  umb=(TIPO *)calloc(tamano*tamano,sizeof(TIPO));

  if (umb)
  {
    /* Coeficientes de analisis */
    /*
    printf("\nCUADRANTE DE ANALISIS\n");
    */
    e_umbral=CalcularE_UmbralAnal(plano,tam_reg,tam_img,tipo_umbral,1.0);
    for (i=0; i<tam_reg; i++)
      for(j=0; j<tam_reg; j++)
      {
	/*
	  printf("i=%d\tj=%d\t%.12f\n",i,j,plano[tamano*i+j]);
	*/
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);
	/*
	  printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	*/
      }
    
    /* Cuadrante Diagonal */
    e_umbral=CalcularE_UmbralDet(plano,tam_reg,tam_img,tipo_umbral,1.0);
    for (i=tam_reg; i<2*tam_reg;i++)
      for (j=tam_reg; j<2*tam_reg; j++)
      {
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			   plano[tam_img*i+j]) >= e_umbral);
	/*
	printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	*/
      }
 
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }

  /* 
  printf("Comienzo umbralizada\n"); 
  for (i=0; i<tam_reg*tam_img; i++)  
	printf("%.12f\n", plano[i]);
  printf("Fin umbralizada\n"); 
  */

  return(umb);
} /* UmbralizarAnalDet */


/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarUserAnalDet                                            *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   umbral:   umbral que se exige.                                        *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarUserAnalDet (TIPO *plano, int tam_reg, int tam_img,
			     TIPO factor, int tipo_umbral)
{
  int i,j,tamano;
  TIPO e_umbral, *umb=NULL;
  
  tamano=2*tam_reg;  
  umb=(TIPO *)calloc(tamano*tamano,sizeof(TIPO));

  if (umb)
  {
    /* Coeficientes de analisis */
    e_umbral=CalcularE_UmbralAnal(plano,tam_reg,tam_img,tipo_umbral,factor);
    for (i=0; i<tam_reg; i++)
      for(j=0; j<=tam_reg; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);
      
    /* Cuadrante Diagonal */
    e_umbral=CalcularE_UmbralDet(plano,tam_reg,tam_img,tipo_umbral,factor);
    for (i=tam_reg; i<2*tam_reg;i++)
      for (j=tam_reg; j<2*tam_reg; j++)
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			   plano[tam_img*i+j]) >= e_umbral);
 
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }
  /*
  for (i=0; i<tam_reg*columnas; i++)  
	printf("%.12f\n", plano_aux[i]);
	*/
  return(umb);
} /* UmbralizarUserAnal */



/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom2                                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada y en los coeficientes del cuadrante diagonal.            *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO  ***DescYMom2 (TIPO *c, int tamano, int hay_umbral, TIPO umbral_user,
		      int umbraliz_equiv) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i,j,resolucion,seguir;
  TIPO ***mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO ***)calloc(resolucion-1,sizeof(TIPO **));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      mom[i]=(TIPO **)calloc(2,sizeof(TIPO *));
      seguir=(mom[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	/* Se consideran los 7 momentos para el nivel de 
	   resolucion */
	mom[i][j]=(TIPO *)calloc(7,sizeof(TIPO));
	seguir=(mom[i][j] != NULL);
      } /* for j */
    } /* for i */
  } /* if mom */

  if (!seguir)
  {
    perror("No se pudo reservar memoria para los momentos");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    printf("\nRESOLUCION %d\n",resolucion);
    if (umbraliz_equiv)
    {
      if (hay_umbral)
	umb=UmbralizarEquivalentes(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarEquivalentes(c,g,tamano,1,VARIANZA);
    }
    else
    {
      if (hay_umbral)
	umb=UmbralizarUserAnalDet(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarAnalDet(c,g,tamano,VARIANZA);
    }

    CalcularMomAnalDet(umb,g,&mom[resolucion]);
    free(umb);
    umb=NULL;

    resolucion--;
  } /* while */
  return (mom);
} /* DescYMom2 */

/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomAnalDet                                               *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomAnalDet (TIPO ****m, int filas, int columnas, char *nombre)
{
  int i, j, k, tamano, retonno=0, items,num_mom;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (num_mom=0; num_mom<3; num_mom++)
    {
      for (i=0; (i<tamano-1) && (!retonno); i++)
      {
	for (j=0; (j<2) && (!retonno); j++)
	{
	  for (k=0; (k<7) && (!retonno); k++)
          {
	    /*
	    printf("m[%d][%d][%d][%d]=%.15f\n",num_mom,i,j,k,m[num_mom][i][j][k]);
	    */

	    items=fwrite(&m[num_mom][i][j][k], sizeof(TIPO), 1, fsal);
	    if (items != 1)
	    {
	      printf("\nError de escritura en %s\n", nombre);
	      perror(" ");
	      retonno=-1;
	    }
	  } /* for k */
	} /* for j */
      } /* for i */
    } /*for num_mom */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomAnalDet */

TIPO ****LeerMomAnalDet (char *nombre, int tamano)
{
  int i, j, k, items, seguir,num_mom;
  FILE *fent;
  TIPO ****m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ****)calloc(3,sizeof(TIPO ***));
    seguir=(m != NULL);
    if (seguir)
    {
      for (num_mom=0; num_mom<3; num_mom++)
      {
	m[num_mom]=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
	seguir=(m[num_mom] != NULL);
	for (i=0; (i<tamano-1) &&(seguir); i++)
        {
	  m[num_mom][i]=(TIPO **)calloc(2, sizeof(TIPO *));
	  seguir = (m[num_mom][i] != NULL);
	  for (j=0; (j<2) && (seguir); j++)
	  {
	    m[num_mom][i][j]=(TIPO *)calloc(7, sizeof(TIPO));
	    seguir = (m[num_mom][i][j] != NULL);
	    for (k=0; (k<7) && (seguir); k++)
	    {
	      items=fread(&m[num_mom][i][j][k], sizeof(TIPO), 1, fent);
	      if (items != 1)
	      {
		printf("\nError de lectura en %s\n", nombre);
		perror(" ");
		return(NULL);
	      }
	    } /* for k */
	  } /* for j */
	} /* for i */
      } /* for num_mom */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosTres */

void LiberarMomAnalDet (TIPO ****momentos, int tamano)
{
  TIPO ***m;
  int i,j,k;

  for (k=0; k<3; k++)
  {
    m=momentos[k];
    for (i=0; i<tamano-1; i++)
    {
      for (j=0; j<2; j++)
      {
	free(m[i][j]);
	m[i][j]=NULL;
      }
      free(m[i]);
      m[i]=NULL;
    }
    free(m);
    m=NULL;
  } /* for k */
} /* LiberarMomAnalDet */



/**************************FUNCION AUXILIAR********************************
**************************************************************************/
TIPO *ToMonocromo(TIPO **img,int tamano)
{
  int i,j;

  TIPO *gris=NULL,mayor;

  gris=(TIPO *)calloc(tamano*tamano,sizeof(TIPO));
  if (gris)
  {
    for (i=0; i<tamano; i++)
      for (j=0; j<tamano; j++)
      {
	mayor=max(img[0][tamano*i+j],
		  max(img[1][tamano*i+j],img[2][tamano*i+j]));
	
	gris[tamano*i+j]=mayor;
      }
  }
  else
    perror("No se pudo reservar memoria para paso a monocromo");

  return(gris);
}

/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
**   Se calculan los momentos de orden 3 para los coeficientes de        **
** analisis pero trabajando en monocromo.                                **
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentosTGrises                                          *
***************************************************************************
*DESCRIPCION: Escribe el vector de momentos en fichero.                   *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentosTGrises (TIPO **m, int filas, int columnas,
			     char *nombre)
{
  int i, k, tamano, retonno=0, items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano-1) &&(!retonno); i++)
    {
      for (k=0; (k<7) && (!retonno); k++)
      {
	/*
	printf("m[%d][%d]=%f\n",i,k,m[i][k]);
	*/

	items=fwrite(&m[i][k], sizeof(TIPO), 1, fsal);
	if (items != 1)
	{
	  printf("\nError de escritura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* for k */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentosTGrises */

TIPO **LeerMomentosTGrises (char *nombre, int tamano)
{
  int i, k, items, seguir;
  FILE *fent;
  TIPO **m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO **)calloc(tamano-1, sizeof(TIPO *));
    seguir=(m != NULL);
    for (i=0; (i<tamano-1) &&(seguir); i++)
    {
      m[i]=(TIPO *)calloc(7, sizeof(TIPO));
      seguir = (m[i] != NULL);
      for (k=0; (k<7) && (seguir); k++)
      {
	items=fread(&m[i][k], sizeof(TIPO), 1, fent);
	if (items != 1)
	{
	  printf("\nError de lectura en %s\n", nombre);
	  perror(" ");
	  return(NULL);
	}
      } /* for k */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosTGrises */

void LiberarMomentosTGrises (TIPO ***momentos, int tamano)
{
  TIPO **m;
  int i;

  m=*momentos;
  for (i=0; i<tamano-1; i++)
  {
    free(m[i]);
    m[i]=NULL;
  }
  free(m);
} /* LiberarMomentosTGrises */


/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
**   Se calculan los momentos de orden 3 para los coeficientes de        **
** analisis y para   el cuadrante diagonal a la vez pero trabajando en   **
** monocromo.                                                            ** 
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentosTDGrises                                         *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentosTDGrises (TIPO ***m, int filas, int columnas, 
			      char *nombre)
{
  int i, j, k, tamano, retonno=0,items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano-1) && (!retonno); i++)
    {
      for (j=0; (j<2) && (!retonno); j++)
      {
	for (k=0; (k<7) && (!retonno); k++)
        {
	  /*
	  printf("m[%d][%d]=%f\n",i,k,m[num_mom][i][k]);
	  */

	  items=fwrite(&m[i][j][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentosTDGrises */

TIPO ***LeerMomentosTDGrises (char *nombre, int tamano)
{
  int i, j, k, items, seguir;
  FILE *fent;
  TIPO ***m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
    seguir=(m != NULL);
    for (i=0; (i<tamano-1) &&(seguir); i++)
    {
      m[i]=(TIPO **)calloc(2, sizeof(TIPO *));
      seguir = (m[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	m[i][j]=(TIPO *)calloc(7, sizeof(TIPO));
	seguir = (m[i][j] != NULL);
	for (k=0; (k<7) && (seguir); k++)
	{
	  items=fread(&m[i][j][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosTDGrises */

void LiberarMomentosTDGrises(TIPO ****momentos, int tamano)
{
  TIPO ***m;
  int i,j;

  m=*momentos;

  for (i=0; i<tamano-1; i++)
  {
    for (j=0; j<2; j++)
    {
      free(m[i][j]);
      m[i][j]=NULL;
    }
    free(m[i]);
    m[i]=NULL;
  }
  free(m);
} /* LiberarMomentosTDGrises */


/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
** Calculo de momentos de orden 2 utilizando los valores los coeficientes**
** de analisis, en vez de los coeficientes de detalle.                   **
**************************************************************************/


/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMom2Anal                                                 *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1 y 2 sobre los               *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional.  *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMom2Anal(TIPO *plano, int tam_reg, TIPO **momento)
{
  TIPO mu20,mu02,mu11;
  int x,y;
  TIPO *m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;


  CalcularMu2(plano,tam_reg,x,y,A,&mu11,&mu20,&mu02);


  m_nivel[0]=mu20+mu02;
  m_nivel[1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n",m_nivel[0],m_nivel[1]);
  printf("\n---------------------\n");
  */

  return(0);
} /* CalcularMom2Anal */


/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom1_2                                                      *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada.                                                         *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO  **DescYMom1_2 (TIPO *c, int tamano, TIPO umbral_user) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i, resolucion,seguir;
  TIPO **mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO **)calloc(resolucion-1,sizeof(TIPO *));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
	/* Se consideran los 2 momentos para el nivel de 
	   resolucion */
	mom[i]=(TIPO *)calloc(2,sizeof(TIPO));
	seguir=(mom[i] != NULL);
    } /* for i */
  } /* if mom */

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;

    if (umbral_user)
      umb=UmbralizarUserAnal(c,g,tamano,umbral_user,VARIANZA);
    else
      umb=UmbralizarAnal(c,g,tamano,VARIANZA);

    CalcularMom2Anal(umb,g,&mom[resolucion]);
    free(umb);
    umb=NULL;

    resolucion--;
  } /* while */
  return (mom);
} /* DescYMom1_2 */


/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentos2Grises                                          *
***************************************************************************
*DESCRIPCION: Escribe el vector de momentos en fichero.                   *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentos2Grises (TIPO **m, int filas, int columnas,
			     char *nombre)
{
  int i, k, tamano, retonno=0, items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano-1) &&(!retonno); i++)
    {
      for (k=0; (k<2) && (!retonno); k++)
      {
	/*
	printf("m[%d][%d]=%f\n",i,k,m[i][k]);
	*/

	items=fwrite(&m[i][k], sizeof(TIPO), 1, fsal);
	if (items != 1)
	{
	  printf("\nError de escritura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* for k */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentos2Grises */

TIPO **LeerMomentos2Grises (char *nombre, int tamano)
{
  int i, k, items, seguir;
  FILE *fent;
  TIPO **m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO **)calloc(tamano-1, sizeof(TIPO *));
    seguir=(m != NULL);
    for (i=0; (i<tamano-1) &&(seguir); i++)
    {
      m[i]=(TIPO *)calloc(2, sizeof(TIPO));
      seguir = (m[i] != NULL);
      for (k=0; (k<2) && (seguir); k++)
      {
	items=fread(&m[i][k], sizeof(TIPO), 1, fent);
	if (items != 1)
	{
	  printf("\nError de lectura en %s\n", nombre);
	  perror(" ");
	  return(NULL);
	}
      } /* for k */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentos2Grises */

/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
**   Se calculan los momentos de orden 2 para los coeficientes de        **
** analisis y para   el cuadrante diagonal a la vez pero trabajando en   **
** monocromo.                                                            ** 
**************************************************************************/
/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMom2AnalDet                                              *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional y *
* sobre los coeficientes de detalle del cuadrante diagonal.               *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMom2AnalDet(TIPO *plano, int tam_reg, TIPO ***momento)
{
  TIPO mu20,mu02,mu11;
  int x,y;
  TIPO **m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;

  /* Coeficientes de anlisis */
  CalcularMu2(plano,2*tam_reg,x,y,A,&mu11,&mu20,&mu02);
  

  m_nivel[0][0]=mu20+mu02;
  m_nivel[0][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n",
  m_nivel[0][0],m_nivel[0][1]);
  printf("\n---------------------\n");
  */


  /* Cuadrante diagonal */
  mu20=0;mu02=0;
  mu11=0;
 
  CalcularMu2(plano,2*tam_reg,x,y,D,&mu11,&mu20,&mu02);
  
  m_nivel[1][0]=mu20+mu02;
  m_nivel[1][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
  m_nivel[1][0],m_nivel[1][1]);
  printf("\n=====================\n");
  */


  return(0);
} /* CalcularMom2AnalDet */

/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom1_2D                                                     *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada y en los coeficientes del cuadrante diagonal.            *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO  ***DescYMom1_2D (TIPO *c, int tamano, int hay_umbral, 
		       TIPO umbral_user, int umbraliz_equiv) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i,j,resolucion,seguir;
  TIPO ***mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO ***)calloc(resolucion-1,sizeof(TIPO **));
  if (mom)
  {    
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      mom[i]=(TIPO **)calloc(2,sizeof(TIPO *));
      seguir=(mom[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	/* Se consideran los 2 momentos para el nivel de 
	   resolucion */
	mom[i][j]=(TIPO *)calloc(2,sizeof(TIPO));
	seguir=(mom[i][j] != NULL);
      } /* for j */
    } /* for i */
  } /* if mom */

  if (!seguir)
  {
    perror("No se pudo reservar memoria para los momentos");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
   
    if (umbraliz_equiv)
    {
      if (hay_umbral)
	umb=UmbralizarEquivalentes(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarEquivalentes(c,g,tamano,1,VARIANZA);
    }
    else
    {
      if (hay_umbral)
	umb=UmbralizarUserAnalDet(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarAnalDet(c,g,tamano,VARIANZA);
    }
    CalcularMom2AnalDet(umb,g,&mom[resolucion]);
    free(umb);
    umb=NULL;

    resolucion--;
  } /* while */
  return (mom);
} /* DescYMom1_2D */


/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomentos2DGrises                                         *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomentos2DGrises (TIPO ***m, int filas, int columnas, 
			      char *nombre)
{
  int i, j, k, tamano, retonno=0,items;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano-1) && (!retonno); i++)
    {
      for (j=0; (j<2) && (!retonno); j++)
      {
	for (k=0; (k<2) && (!retonno); k++)
        {
	  /*
	  printf("m[%d][%d]=%f\n",i,k,m[num_mom][i][k]);
	  */

	  items=fwrite(&m[i][j][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomentos2DGrises */

TIPO ***LeerMomentos2DGrises (char *nombre, int tamano)
{
  int i, j, k, items, seguir;
  FILE *fent;
  TIPO ***m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    m=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
    seguir=(m != NULL);
    for (i=0; (i<tamano-1) && (seguir); i++)
    {
      m[i]=(TIPO **)calloc(2, sizeof(TIPO *));
      seguir = (m[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	m[i][j]=(TIPO *)calloc(2, sizeof(TIPO));
	seguir = (m[i][j] != NULL);
	for (k=0; (k<2) && (seguir); k++)
	{
	  items=fread(&m[i][j][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for j */
    } /* for i */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentos2DGrises */


/*
**************************** NUEVA TECNICA ********************************
   Calculo de energias en los coeficientes de analisis y de momentos en
los coeficientes de detalle del cuadrante diagonal.
***************************************************************************
*/

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMomTres_D                                                *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 3 de imagen bidimensional a   *
*   que se ha aplicado la transformada en onditas y se ha umbralizado.    *
*   Se calculan de forma individual sobre el cuadrante diagonal de        *
*   coeficientes de detalle.                                              *
*   de detalle.                                                           *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   Vector con los momentos. Es un triple puntero en el que para cada     *
* nivel de resolucion se almacenan los momentos de las tres regiones que  *
* surgen (los dos momentos de orden 2 para cada region: fi1 y fi2, y los  *
* cinco que vienen en el Hain para orden 3). Ademas se transforman las    *
* coordenadas de [0,511] al dominio [-1,1].                               *
*   NULL en caso de que haya algun problema.                              *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

TIPO **CalcularMomTres_D(TIPO *plano, int filas, int columnas)
{
  TIPO **m=NULL,mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int resolucion,i,seguir,x,y,res;
  TIPO tonta;
                           

  tonta=(log((TIPO)filas)/log(2.0));
  resolucion=tonta;

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */
 
  /* 
     A resolucion 0, cada cuadrante tiene un unico elemento. No tiene sentido
     calcular sus momentos.
  */
  m=(TIPO **)calloc(resolucion-1,sizeof(TIPO *));
  if (m)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      m[i]=(TIPO *)calloc(7,sizeof(TIPO));
      seguir=(m[i] != NULL);
    }

    /* Se ha reservado la memoria. Ahora vienen los calculos */

    if (seguir)
    {
      y=filas;
      x=columnas;

      for (res=resolucion-2; res >= 0; res--,x=x/2,y=y/2)
      {		
	mu20=0;mu02=0;
	mu11=0;
	mu12=0;mu21=0;
	mu03=0;mu30=0;

	CalcularMu3(plano,columnas,x,y,D,&mu11,&mu20,&mu02,&mu21,&mu12,
		    &mu30,&mu03);


	m[res][0]=mu20+mu02;
 
        m[res][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
	
        m[res][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		   ((mu03-3*mu21)*(mu03-3*mu21)));
 
        m[res][3]=(((mu30+mu12)*(mu30+mu12))+
		   ((mu03+mu21)*(mu03+mu21)));
	
        m[res][4]=((mu30-3*mu12)*(mu30+mu12)*
		   (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		   (mu03-3*mu21)*(mu03+mu21)*
		   (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
	
        m[res][5]=((mu20-mu02)*
		   (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		   4*mu11*(mu30+mu12)*(mu03+mu21));
	
        m[res][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		       (((mu30+mu12)*(mu30+mu12))-
			3*((mu21+mu03)*(mu21+mu03)))+
		       (mu30-3*mu21)*(mu21+mu03)*
		       (((mu03+mu21)*(mu03+mu21))-
			3*((mu30+mu12)*(mu30+mu12))));
 
        /*
	printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	       m[res][1][0],m[res][1][1],m[res][1][2],m[res][1][3],
	       m[res][1][4],m[res][1][5],m[res][1][6]);
	printf("Y mu02=%.12f   mu20=%.12f\n", mu02, mu20);
	printf("\n---------------------\n");
        */

      } /* for res */
    } /* if seguir */
    else
      printf("\nATENCION: no se pudo reservar memoria para calcular"
	     "los momentos\n");
  } /* if (m) */
  else
    printf("\nATENCION: no se pudo reservar memoria para calcular energias\n");

  return(m);
} /* CalcularMomTres_D */


int EscribirEn_Mom (TIPO ***em, int tamano, char *nombre)
{
  int i, k, retonno=0, items,plano;
  FILE *fsal;

  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    for (i=0; (i<tamano) && (!retonno); i++)
    {
      for (k=0; (k<6) && (!retonno); k++)
      {
	items=fwrite(&em[0][i][k], sizeof(TIPO), 1, fsal);
	if (items != 1)
	{
	  printf("\nError de escritura en %s\n", nombre);
	  perror(" ");
	  retonno=-1;
	}
      } /* for k */
    } /* for i */

    for (plano=1; plano < 4; plano++)
    {
      for (i=0; (i<tamano-1) &&(!retonno); i++)
      {
	for (k=0; (k<7) && (!retonno); k++)
	{
	  items=fwrite(&em[plano][i][k], sizeof(TIPO), 1, fsal);
	  if (items != 1)
	  {
	    printf("\nError de escritura en %s\n", nombre);
	    perror(" ");
	    retonno=-1;
	  }
	} /* for k */
      } /* for i */
    } /* for plano */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /* EscribirEn_Mom */

TIPO ***LeerEn_Mom (char *nombre, int tamano)
{
  int i, k, items, plano,seguir;
  FILE *fent;
  TIPO ***en_mom;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    en_mom=(TIPO ***)calloc(4,sizeof(TIPO **));
    seguir=(en_mom != NULL);

    /* Lectura de las energias */
    if (seguir)
    {
      en_mom[0]=(TIPO **)calloc(tamano, sizeof(TIPO *));
      seguir=(en_mom[0] != NULL);
      for (i=0; (i<tamano) &&(seguir); i++)
      {
	en_mom[0][i]=(TIPO *)calloc(6, sizeof(TIPO));
	seguir = (en_mom[0][i] != NULL);
	for (k=0; (k<6) && (seguir); k++)
	{
	  items=fread(&en_mom[0][i][k], sizeof(TIPO), 1, fent);
	  if (items != 1)
	  {
	    printf("\nError de lectura en %s\n", nombre);
	    perror(" ");
	    return(NULL);
	  }
	} /* for k */
      } /* for i */

      /* Lectura de los momentos */
      for (plano=1; (plano<4) && (seguir); plano++)
      {
	en_mom[plano]=(TIPO **)calloc(tamano-1, sizeof(TIPO *));
	seguir=(en_mom[0] != NULL);
	for (i=0; (i<tamano-1) && (seguir); i++)
	{
	  en_mom[plano][i]=(TIPO *)calloc(7, sizeof(TIPO));
	  seguir = (en_mom[plano][i] != NULL);
	  for (k=0; (k<7) && (seguir); k++)
	  {
	    items=fread(&en_mom[plano][i][k], sizeof(TIPO), 1, fent);
	    if (items != 1)
	    {
	      printf("\nError de lectura en %s\n", nombre);
	      perror(" ");
	      return(NULL);
	    }
	  } /* for k */
	} /* for i */
      } /* for plano */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los "
	     "momentos y energias.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(en_mom);
  } /* else */
} /* LeerEn_Mom */

void LiberarEn_Mom (TIPO ****en_mom, int tamano)
{
  TIPO ***aux=*en_mom;
  int i,plano;

  for (i=0; i<tamano; i++)
  {
    free(aux[0][i]);
    aux[0][i]=NULL;
  }
  free(aux[0]);
  aux[0]=NULL;
  for (plano=1; plano<4; plano++)
  {
    for (i=0; i<tamano-1; i++)
    {
      free(aux[plano][i]);
      aux[plano][i]=NULL;
    }
    free(aux[plano]);
    aux[plano]=NULL;
  }

  free(aux);
  aux=NULL;
} /* LiberarEn_Mom */


/*
**************************** NUEVA TECNICA ********************************
   Calculo de momentos en los coeficientes de analisis y en los
coeficientes de detalle del cuadrante diagonal, haciendo que sean una sola
region.
***************************************************************************
*/

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMom2AnalYDet                                               *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional y *
* sobre los coeficientes de detalle del cuadrante diagonal.               *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMom2AnalYDet(TIPO *plano, int tam_reg, TIPO **momento)
{
  TIPO mu20,mu02,mu11;
  int x,y;
  TIPO *m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;

  /* Coeficientes de anlisis */
  /*CalcularMu2(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02); */
  CalcularMu2(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02);

  m_nivel[0]=mu20+mu02;
  m_nivel[1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n",m_nivel[0],m_nivel[1]);
  printf("\n---------------------\n");
  */

  return(0);
} /* CalcularMom2AnalYDet */



/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom2AD                                                        *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada y en los coeficientes del cuadrante diagonal.            *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO  **DescYMom2AD (TIPO *c, int tamano, int hay_umbral, TIPO umbral_user,
		     int umbraliz_equiv) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i,resolucion,seguir;
  TIPO **mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO **)calloc(resolucion-1,sizeof(TIPO **));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      mom[i]=(TIPO *)calloc(2,sizeof(TIPO *));
      seguir=(mom[i] != NULL);
    } /* for i */
  } /* if mom */

  if (!seguir)
  {
    perror("No se pudo reservar memoria para los momentos");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    
    if (umbraliz_equiv)
    {
      if (hay_umbral)
	umb=UmbralizarEquivalentes(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarEquivalentes(c,g,tamano,1,VARIANZA);
    }
    else
    {
      /*printf("g=%d\n",g);*/
      if (hay_umbral)
	umb=UmbralizarUserAnalDet(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarAnalDet(c,g,tamano,VARIANZA);
    }

    CalcularMom2AnalYDet(umb,g,&mom[resolucion]);
    free(umb);
    umb=NULL;

    resolucion--;
  } /* while */
  return (mom);
} /* DescYMom2AD */


/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMom3AnalYDet                                             *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional y *
* sobre los coeficientes de detalle del cuadrante diagonal.               *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMom3AnalYDet(TIPO *plano, int tam_reg, TIPO **momento)
{
  TIPO mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int x,y;
  TIPO *m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;

  /* Coeficientes de anlisis */
  /*CalcularMu2(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02); */
  CalcularMu3(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02,&mu21,&mu12,&mu30,&mu03);

  m_nivel[0]=mu20+mu02;
  
  m_nivel[1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
 
  m_nivel[2]=(((mu30-3*mu12)*(mu30-3*mu12))+
              ((mu03-3*mu21)*(mu03-3*mu21)));
 
  m_nivel[3]=(((mu30+mu12)*(mu30+mu12))+
              ((mu03+mu21)*(mu03+mu21)));
  
  m_nivel[4]=((mu30-3*mu12)*(mu30+mu12)*
              (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
              (mu03-3*mu21)*(mu03+mu21)*
              (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
  m_nivel[5]=((mu20-mu02)*
              (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
              4*mu11*(mu30+mu12)*(mu03+mu21));
  
  m_nivel[6]=fabs((3*mu21-mu03)*(mu30+mu12)*
                  (((mu30+mu12)*(mu30+mu12))-
                   3*((mu21+mu03)*(mu21+mu03)))+
                  (mu30-3*mu21)*(mu21+mu03)*
                  (((mu03+mu21)*(mu03+mu21))-
                   3*((mu30+mu12)*(mu30+mu12))));

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	 m_nivel[0],m_nivel[1],m_nivel[2],m_nivel[3],m_nivel[4],m_nivel[5],
	 m_nivel[6]);
  printf("\n---------------------\n");
  */

  return(0);
} /* CalcularMom3AnalYDet */




/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMom3AD                                                      *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada y en los coeficientes del cuadrante diagonal.            *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO **DescYMom3AD (TIPO *c, int tamano, int hay_umbral, TIPO umbral_user,
		     int umbraliz_equiv) 
{
  int cont_filas, cont_colum;
  int g = tamano;
  TIPO *columnaAux;

  int i,resolucion,seguir;
  TIPO **mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;

  /* Reserva de memoria para los momentos */
  mom=(TIPO **)calloc(resolucion-1,sizeof(TIPO *));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      /* Se consideran los 7 momentos para el nivel de 
	   resolucion */
      mom[i]=(TIPO *)calloc(7,sizeof(TIPO));
      seguir=(mom[i] != NULL);
    } /* for i */
  } /* if mom */

  if (!seguir)
  {
    perror("No se pudo reservar memoria para los momentos");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    
    if (umbraliz_equiv)
    {
      if (hay_umbral)
	umb=UmbralizarEquivalentes(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarEquivalentes(c,g,tamano,1,VARIANZA);
    }
    else
    {
      if (hay_umbral)
	umb=UmbralizarUserAnalDet(c,g,tamano,umbral_user,VARIANZA);
      else
	umb=UmbralizarAnalDet(c,g,tamano,VARIANZA);
    }

    /*printf("\nNIVEL %d\n", resolucion);*/

    CalcularMom3AnalYDet(umb,g,&mom[resolucion]);
    free(umb);
    umb=NULL;

    resolucion--;

  } /* while */
  return (mom);
} /* DescYMom3AD */


/*************************** NUEVA TECNICA ********************************
**************************************************************************/

/**************************************************************************
** Calculo de los momentos de orden 2 o 3 sobre los cuadrantes diagonal  **
** y horizontal como si fueran una sola region, ademas de los de         **
** analisis y diagonales como si fueran una sola region                  **
**************************************************************************/

/******************* Cabecera de Funcion **********************************
*NOMBRE: CalcularMomAdYHv                                                 *
***************************************************************************
*DESCRIPCION: Calcula los momentos de orden 1, 2 y 3 sobre los            *
* coeficientes de analisis que resultan en un determinado paso intermedio *
* del calculo de la transformada en onditas de una imagen bidimensional y *
* sobre los coeficientes de detalle del cuadrante diagonal.               *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas que ocupan los valores medios (tamano en Y).*
*   columnas: numero de columnas que ocupan los valores (tamano en X).    *
*   m_nivel:  vector en el que hay que almacenar los momentos calculados. *
*SALIDA:                                                                  *
*   0 si todo fue bien.                                                   *
*  -1 en otro caso.                                                       *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/

int CalcularMomAdYHv(TIPO *plano, int tam_reg, TIPO ***momento,
		      int orden, int combinacion)
{
  TIPO mu20,mu02,mu11,mu30,mu03,mu12,mu21;
  int x,y;
  TIPO **m_nivel=*momento;
                           

  /*
    NOTA: el caso con coordenadas 0,0 no hay que considerarlo, pues al
    calcular los momentos siempre saldra 0

    Resolucion ha de valer 9, pero por lo anterior so'lo nos interesan 8
    partes.
  */

  
  y=tam_reg;
  x=tam_reg;
  
  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;

  /* Coeficientes de anlisis y cuadrante diagonal */
  if (orden == 2)
  {
    CalcularMu2(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02);
    m_nivel[0][0]=mu20+mu02;
    m_nivel[0][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
    /*
    printf("\n%.12f\n%.12f\n",m_nivel[0][0],m_nivel[0][1]);
    printf("----------------------\n");
    */
  }
  else /* (orden == 3) */
  {
    CalcularMu3(plano,2*tam_reg,x,y,M,&mu11,&mu20,&mu02,&mu21,&mu12,
		&mu30,&mu03);

    m_nivel[0][0]=mu20+mu02;
  
    m_nivel[0][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
 
    m_nivel[0][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		   ((mu03-3*mu21)*(mu03-3*mu21)));
    
    m_nivel[0][3]=(((mu30+mu12)*(mu30+mu12))+
		   ((mu03+mu21)*(mu03+mu21)));
  
    m_nivel[0][4]=((mu30-3*mu12)*(mu30+mu12)*
		   (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		   (mu03-3*mu21)*(mu03+mu21)*
		   (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
    m_nivel[0][5]=((mu20-mu02)*
		   (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		   4*mu11*(mu30+mu12)*(mu03+mu21));
    
    m_nivel[0][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		       (((mu30+mu12)*(mu30+mu12))-
			3*((mu21+mu03)*(mu21+mu03)))+
		       (mu30-3*mu21)*(mu21+mu03)*
		       (((mu03+mu21)*(mu03+mu21))-
			3*((mu30+mu12)*(mu30+mu12))));

  }


  mu20=0;mu02=0;
  mu11=0;
  mu12=0;mu21=0;
  mu03=0;mu30=0;

  /* Coeficientes de los cuadrantes vertical y horizontal */
  if (orden == 2)
  {
    CalcularMu2(plano,2*tam_reg,x,y,combinacion,&mu11,&mu20,&mu02);
    m_nivel[1][0]=mu20+mu02;
    m_nivel[1][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
  }
  else /* (orden == 3) */
  {
    CalcularMu3(plano,2*tam_reg,x,y,combinacion,&mu11,&mu20,&mu02,&mu21,&mu12,
		&mu30,&mu03);

    m_nivel[1][1]=(mu20-mu02)*(mu20-mu02)+4*(mu11*mu11);
 
    m_nivel[1][2]=(((mu30-3*mu12)*(mu30-3*mu12))+
		   ((mu03-3*mu21)*(mu03-3*mu21)));
    
    m_nivel[1][3]=(((mu30+mu12)*(mu30+mu12))+
		   ((mu03+mu21)*(mu03+mu21)));
  
    m_nivel[1][4]=((mu30-3*mu12)*(mu30+mu12)*
		   (((mu30+mu12)*(mu30+mu12))-3*((mu21+mu03)*(mu21+mu03)))+
		   (mu03-3*mu21)*(mu03+mu21)*
		   (((mu03+mu21)*(mu03+mu21))-3*((mu12+mu30)*(mu12+mu30))));
  
    m_nivel[1][5]=((mu20-mu02)*
		   (((mu30+mu12)*(mu30+mu12))-((mu21+mu03)*(mu21+mu03)))+
		   4*mu11*(mu30+mu12)*(mu03+mu21));
  
    m_nivel[1][6]=fabs((3*mu21-mu03)*(mu30+mu12)*
		       (((mu30+mu12)*(mu30+mu12))-
			3*((mu21+mu03)*(mu21+mu03)))+
		       (mu30-3*mu21)*(mu21+mu03)*
		       (((mu03+mu21)*(mu03+mu21))-
			3*((mu30+mu12)*(mu30+mu12))));   
  }

  /*
  printf("MOMENTOS\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n%.12f\n",
	 m_nivel[0],m_nivel[1],m_nivel[2],m_nivel[3],m_nivel[4],m_nivel[5],
	 m_nivel[6]);
  printf("\n---------------------\n");
  */

  return(0);
} /* CalcularMomAdYHv */


/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularE_UmbralHV                                               *
***************************************************************************
*DESCRIPCION: Calcula la energia media de los coeficientes de detalle de  *
*   los cuadrantes vertical y horizontal, tratandolos como si fueran una  *
*   misma region.                                                         *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   tam_img:  tamano de la imagen.                                        *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_UmbralHV(int region, TIPO *plano, int rango_x, 
			int tam_img, int tipo_umbral, TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,tmp;
  int i,j,num_pixels=0,i_inf,i_sup,j_inf,j_sup;

  if (region == V)
  {
    i_inf=0;
    i_sup=rango_x;
    j_inf=rango_x;
    j_sup=2*rango_x;
  }
  else /* if (region == H) */
  {
    j_inf=0;
    j_sup=rango_x;
    i_inf=rango_x;
    i_sup=2*rango_x;
  }


  for (i=i_inf; i<i_sup; i++)
  {
    for(j=j_inf;j<j_sup; j++)
    {
      e_media+=(plano[tam_img*i+j]*plano[tam_img*i+j]);
      num_pixels++;
    }
  }
  
  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (i=i_inf; i<i_sup; i++)
    {
      for(j=j_inf;j<j_sup; j++)
      {
	tmp = plano[tam_img*i+j] * plano[tam_img*i+j];
	tmp = tmp - e_media;
	varianza += tmp * tmp;

	/*
	varianza+=pow(pow(plano[tam_img*i+j],2)-e_media,2);
	*/
      }
    }
    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    valor = e_media + (varianza * factor);
    return(valor);
  } /* else */
} /* CalcularE_UmbralHV */


/************************* Cabecera de Funcion ****************************
*NOMBRE: CalcularE_UmbralComb                                             *
***************************************************************************
*DESCRIPCION: Calcula la energia media de los coeficientes de la          *
*   combinacion de los coeficientes de detalle de los cuadrantes vertical *
*   y horizontal. Despus calcula su desviacion tipica.                   *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   rango_x:  limite del nivel en X.                                      *
*   tam_img:  tamano de la imagen.                                        *
*SALIDA:                                                                  *
*   La energia media.                                                     *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
TIPO CalcularE_UmbralComb(TIPO *plano, int rango_x, int tam_img,
			  int tipo_umbral, TIPO factor)
{
  TIPO e_media=0,varianza=0,valor=0,elto,tmp1,tmp2;
  int i,j,num_pixels=0,i_inf,i_sup,j_inf,j_sup;

  i_inf=0;
  i_sup=rango_x;
  j_inf=rango_x;
  j_sup=2*rango_x;

  for (i=i_inf; i<i_sup; i++)
  {
    for(j=j_inf;j<j_sup; j++)
    {
      tmp1 = plano[tam_img*i+j] * plano[tam_img*i+j];
      tmp2 = plano[tam_img*j+i] * plano[tam_img*j+i];
      e_media += tmp1 * tmp2;
      
      /*
      e_media += (pow(plano[tam_img*i+j],2) + pow(plano[tam_img*j+i],2));
      */

      num_pixels++;
    }
  }
  
  e_media=e_media/(TIPO)num_pixels;
  if (tipo_umbral == MEDIA)
    return(e_media);
  else
  {
    for (i=i_inf; i<i_sup; i++)
    {
      for(j=j_inf;j<j_sup; j++)
      {
	tmp1 = plano[tam_img*i+j] * plano[tam_img*i+j];
	tmp2 = plano[tam_img*j+i] * plano[tam_img*j+i];
	elto = tmp1 + tmp2 - e_media;
	varianza += elto * elto;

	/*
	elto = (pow(plano[tam_img*i+j],2) + pow(plano[tam_img*j+i],2));
	varianza += pow(elto-e_media,2);
	*/

      }
    }
    varianza=varianza/num_pixels;
    varianza = sqrt(varianza);
    /*printf("%.12f\n",varianza);*/
    valor = e_media + (varianza * factor);
    return(valor);
  } /* else */
} /* CalcularE_UmbralComb */


/************************* Cabecera de Funcion ****************************
*NOMBRE: UmbralizarTodo                                                   *
***************************************************************************
*DESCRIPCION: Umbraliza la transformada en onditas de una imagen          *
*   bidimensional.                                                        *
*PARAMETROS:                                                              *
*   plano:    puntero a un plano sobre el que se realiza el calculo.      *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*SALIDA:                                                                  *
*   plano:    el plano ya umbralizado.                                    *
*VAR.GLOBALES: Ninguna                                                    *
*ALGORITMO: El umbral, en cada nivel de resolucion, sera la energia media *
*   de ese nivel de resolucion.                                           *
**************************************************************************/

TIPO *UmbralizarTodo (TIPO *plano, int tam_reg, int tam_img, int tipo_umbral,
		      int hay_umbral, TIPO factor, int combinacion)
{
  int i,j,tamano;
  TIPO e_umbral, *umb=NULL, elto_aux, tmp1, tmp2;
  
  tamano=2*tam_reg;
  umb=(TIPO *)calloc(tamano*tamano,sizeof(TIPO));

  if (!hay_umbral)
    factor=1.0;

  if (umb)
  {
    /* Coeficientes de analisis */
    /*
    printf("\nCUADRANTE DE ANALISIS\n");
    */
    e_umbral=CalcularE_UmbralAnal(plano,tam_reg,tam_img,tipo_umbral,factor);
    for (i=0; i<tam_reg; i++)
      for(j=0; j<tam_reg; j++)
      {
	/*
	  printf("i=%d\tj=%d\t%.12f\n",i,j,plano[tamano*i+j]);
	*/

	umb[tamano*i+j]=((plano[tam_img*i+j]*
			  plano[tam_img*i+j]) >= e_umbral);

	/*
	  printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	*/
      }
    
    /* Cuadrante Diagonal */
    e_umbral=CalcularE_UmbralDet(plano,tam_reg,tam_img,tipo_umbral,factor);
    for (i=tam_reg; i<2*tam_reg;i++)
      for (j=tam_reg; j<2*tam_reg; j++)
      {
	umb[tamano*i+j]=((plano[tam_img*i+j]*
			   plano[tam_img*i+j]) >= e_umbral);
	/*
	printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	*/
      }

    if (combinacion == U)
    {
      /* Cuadrante vertical */
      e_umbral=CalcularE_UmbralHV(V,plano,tam_reg,tam_img,tipo_umbral,factor);
      for (i=0; i<tam_reg;i++)
	for (j=tam_reg; j<2*tam_reg; j++)
        {
	  umb[tamano*i+j]=((plano[tam_img*i+j]*
			    plano[tam_img*i+j]) >= e_umbral);
	  /*
	    printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	  */
	}

      /* Cuadrante horizontal */
      e_umbral=CalcularE_UmbralHV(H,plano,tam_reg,tam_img,tipo_umbral,factor);
      for (i=tam_reg; i<2*tam_reg;i++)
	for (j=0; j<tam_reg; j++)
	{
	  umb[tamano*i+j]=((plano[tam_img*i+j]*
			    plano[tam_img*i+j]) >= e_umbral);
	  /*
	    printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	  */
	}
    } /* if (combinacion == U) */
    else if (combinacion == C)
    {
      /* 
      Cuadrantes vertical y horizontal combinados. El resultado luego se
      va a almacenar en el espacio que corresponde al cuadrante vertical,
      lo que se ha de tener en cuenta posteriormente para calcular los 
      momentos.
      */
      e_umbral=CalcularE_UmbralComb(plano,tam_reg,tam_img,tipo_umbral,factor);
      for (i=0; i<tam_reg;i++)
	for (j=tam_reg; j<2*tam_reg; j++)
        {
	  tmp1 = plano[tam_img*i+j] * plano[tam_img*i+j];
	  tmp2 = plano[tam_img*j+i] * plano[tam_img*j+i];
	  elto_aux = tmp1 + tmp2;
	  
	  /*
	  elto_aux = (pow(plano[tam_img*i+j],2) + pow(plano[tam_img*j+i],2));
	  */

	  umb[tamano*i+j]=(elto_aux >= e_umbral);
	  /*
	    printf("i=%d\tj=%d\t%.12f\n",i,j,umb[tamano*i+j]);
	  */
	}
    } /* if (combinacion == C) */
    else
    {
      printf("\nATENCION ---- ATENCION\n");
      printf("Se selecciono un valor erroneo en el tipo de combinacion de los"
	     "\ncuadrantes. Revisar el codigo C\n");
      exit(1);
    }
  } /* if umb*/
  else
  {
    printf("\nNO SE PUDO RESERVAR MEMORIA PARA UMBRALIZAR\n");
  }

  /* 
  printf("Comienzo umbralizada\n"); 
  for (i=0; i<tam_reg*tam_img; i++)  
	printf("%.12f\n", plano[i]);
  printf("Fin umbralizada\n"); 
  */

  return(umb);
} /* UmbralizarTodo */


/******************* Cabecera de Funcion **********************************
*NOMBRE: DescYMomTodos                                                    *
***************************************************************************
*                                                                         *
*DESCRIPCION: Descomposicion no estandar de una matriz y calculo de       *
*   momentos en los coeficientes de analisis de cada paso de la           *
*   transformada y en los coeficientes del cuadrante diagonal.            *
*PARAMETROS:                                                              *
*   c:        matriz a descomponer                                        *
*   tamano:   numero de filas y columnas de la matriz. Tiene qu ser       *
*             potencia de 2                                               *
*SALIDA:                                                                  *
*    Puntero con los momentos.                                            *
*    NULL si ha habido algun problema.                                    *
*VAR.GLOBALES:                                                            *
*ALGORITMO: Se normaliza. Por cada fila y columna potencia de 2 se llama  *
*    a DecompositionStep.                                                 *
*           Los momentos se almacenaran en orden inverso: mom[7] sera el  *
*    que corresponda a mayor resolucion y mom[0] al de menor.             *
**************************************************************************/
TIPO ***DescYMomTodos (TIPO *c, int tamano, int orden, int hay_umbral, 
		       TIPO umbral_user, int combinacion)
{
  int cont_filas, cont_colum,j;
  int g = tamano;
  TIPO *columnaAux;

  int i,resolucion,seguir;
  TIPO ***mom, *umb,tonta;


  tonta=(TIPO)(log((TIPO)tamano)/log(2.0));
  resolucion=tonta;
  /* Reserva de memoria para los momentos */
  mom=(TIPO ***)calloc(resolucion-1,sizeof(TIPO **));
  if (mom)
  {
    seguir=1;
    for (i=0; (i<resolucion-1) && (seguir); i++)
    {
      mom[i]=(TIPO **)calloc(2,sizeof(TIPO *));
      seguir=(mom[i] != NULL);
      for (j=0; (j<2) && (seguir); j++)
      {
	if (orden == 2)
	{
	  /*
	    Se consideran los 2 momentos de orden 2 para ese nivel de
	    resolucion 
	  */
	  mom[i][j]=(TIPO *)calloc(2,sizeof(TIPO));
	  seguir=(mom[i] != NULL);
	}
	else if (orden == 3)
	{
	  /* 
	     Se consideran los 7 momentos para el nivel de 
	     resolucion 
	  */
	  mom[i][j]=(TIPO *)calloc(7,sizeof(TIPO));
	  seguir=(mom[i][j] != NULL);
	}
	else
	{
	  printf("\nATENCION---ATENCION\n");
	  printf("\nError de programacion al elegir el orden de los "
		 "momentos\n");
	  exit(1);
	}
      } /* for j */
    } /* for i */
  } /* if mom */

  if (!seguir)
  {
    perror("No se pudo reservar memoria para los momentos");
    return(NULL);
  }

  /* Se reinicializa resolucion para el bucle while de descomposiciones */
  resolucion=tonta;
  /* Del ultimo nivel no hay que calcular momentos: son 0. Esto es porque
  queda un unico elemento. */
  resolucion-=2;

  columnaAux = (TIPO*) malloc (tamano*sizeof(TIPO));
  if ((tamano <= 0) || (!columnaAux)) {
    perror ("Error en NonstandardDecomposition");
    return (NULL);
  }
  while ((g >= 2) && (resolucion >=0)) {
    for (cont_filas=0; cont_filas < g; cont_filas++)
      DecompositionStep (c + cont_filas*tamano, g);
    for (cont_colum=0; cont_colum < g; cont_colum++) {
      for (cont_filas=0; cont_filas < g; cont_filas++)
        columnaAux[cont_filas] = c [cont_filas*tamano+cont_colum];
      DecompositionStep (columnaAux, g);
      for (cont_filas=0; cont_filas < g; cont_filas++)
        c [cont_filas*tamano+cont_colum] = columnaAux[cont_filas];
    }

    /*
    sprintf (nombre, "NonStdDecomp%03i.pgm", g);
    Escalar(c,tamano, tamano, nombre);
    */

    g >>= 1;
    
    umb=UmbralizarTodo(c,g,tamano,VARIANZA,hay_umbral,umbral_user,
		       combinacion);
   

    /*printf("\nNIVEL %d\n", resolucion);*/

    CalcularMomAdYHv(umb,g,&mom[resolucion],orden,combinacion);
    free(umb);
    umb=NULL;

    resolucion--;

  } /* while */
  return (mom);
} /* DescYMomTodos */

/******************* Cabecera de Funcion **********************************
*NOMBRE: EscribirMomTodos                                                 *
***************************************************************************
*DESCRIPCION: Escribe los tres vectores de momentos en fichero.           *
*PARAMETROS:                                                              *
*   m:        puntero con los tres vectores de momentos.                  *
*   filas:    numero de filas de la imagen (tamano en Y).                 *
*   columnas: numero de columnas de la imagen (tamano en X).              *
*   nombre:   nombre del fichero en el que se va a escribir.              *
*SALIDA:                                                                  *
*VAR.GLOBALES: Ninguna                                                    *
**************************************************************************/
int EscribirMomTodos (TIPO ****m, int filas, int columnas,
		      char *nombre, int orden)
{
  int i, j, k, tamano, retonno=0, items,num_mom, limite_k;
  FILE *fsal;
  TIPO tonta;

  tonta = (log((TIPO)filas)/log(2.0));
  tamano=tonta;
  fsal = fopen(nombre, "wb");
  if (!fsal)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    retonno=-1;
  }
  else
  {
    if (orden == 2)
      limite_k = 2;
    else /* (orden == 3) */
      limite_k = 7;

    for (num_mom=0; num_mom<3; num_mom++)
    {
      for (i=0; (i<tamano-1) && (!retonno); i++)
      {
	for (j=0; (j<2) && (!retonno); j++)
	{
	  for (k=0; (k<limite_k) && (!retonno); k++)
          {
	    /*
	    printf("m[%d][%d][%d][%d]=%.15f\n",num_mom,i,j,k,m[num_mom][i][j][k]);
	    */

	    items=fwrite(&m[num_mom][i][j][k], sizeof(TIPO), 1, fsal);
	    if (items != 1)
	    {
	      printf("\nError de escritura en %s\n", nombre);
	      perror(" ");
	      retonno=-1;
	    }
	  } /* for k */
	} /* for j */
      } /* for i */
    } /*for num_mom */
    retonno=tamano;
  } /* else */
  fclose(fsal);
  return(retonno);
} /*  EscribirMomTodos */

TIPO ****LeerMomTodos (char *nombre, int tamano, int orden)
{
  int i, j, k, items, seguir,num_mom,limite_k;
  FILE *fent;
  TIPO ****m=NULL;

  fent = fopen(nombre, "rb");
  if (!fent)
  {
    printf("\nError al abrir el fichero %s\n", nombre);
    perror(" ");
    return(NULL);
  }
  else
  {
    if (orden == 2)
      limite_k = 2;
    else /* (orden == 3) */
      limite_k = 7;

    m=(TIPO ****)calloc(3,sizeof(TIPO ***));
    seguir=(m != NULL);
    if (seguir)
    {
      for (num_mom=0; num_mom<3; num_mom++)
      {
	m[num_mom]=(TIPO ***)calloc(tamano-1, sizeof(TIPO **));
	seguir=(m[num_mom] != NULL);
	for (i=0; (i<tamano-1) &&(seguir); i++)
        {
	  m[num_mom][i]=(TIPO **)calloc(2, sizeof(TIPO *));
	  seguir = (m[num_mom][i] != NULL);
	  for (j=0; (j<2) && (seguir); j++)
	  {
	    m[num_mom][i][j]=(TIPO *)calloc(limite_k, sizeof(TIPO));
	    seguir = (m[num_mom][i][j] != NULL);
	    for (k=0; (k<limite_k) && (seguir); k++)
	    {
	      items=fread(&m[num_mom][i][j][k], sizeof(TIPO), 1, fent);
	      if (items != 1)
	      {
		printf("\nError de lectura en %s\n", nombre);
		perror(" ");
		return(NULL);
	      }
	    } /* for k */
	  } /* for j */
	} /* for i */
      } /* for num_mom */
    } /* if seguir */
    fclose(fent);
    if (!seguir)
    {
      printf("\nNo se pudo reservar suficiente memoria para los"
	     "momentos.\n");
      perror(" ");
      return(NULL);
    }
    else
      return(m);
  } /* else */
} /* LeerMomentosTodos */

void LiberarMomTodos (TIPO ****momentos, int tamano)
{
  TIPO ***m;
  int i,j,k;

  for (k=0; k<3; k++)
  {
    m=momentos[k];
    for (i=0; i<tamano-1; i++)
    {
      for (j=0; j<2; j++)
      {
	free(m[i][j]);
	m[i][j]=NULL;
      }
      free(m[i]);
      m[i]=NULL;
    }
    free(m);
    m=NULL;
  } /* for k */
} /* LiberarMomTodos */

